@@ -45,6 +45,7 @@
 #include "gtkutils.h"
 #include "prefs_common.h"
 #include "utils.h"
+#include "version.h"
 
 #include "pixmaps/sylpheed-logo.xpm"
 
@@ -91,7 +92,6 @@ static void about_create(void)
 	gtk_container_set_border_width(GTK_CONTAINER(window), 8);
 	gtk_widget_set_usize(window, 518, 358);
 	gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
-	/*gtk_window_set_policy(GTK_WINDOW(window), FALSE, FALSE, FALSE);*/
 	gtk_signal_connect(GTK_OBJECT(window), "delete_event",
 			   GTK_SIGNAL_FUNC(gtk_widget_hide_on_delete), NULL);
 	gtk_signal_connect(GTK_OBJECT(window), "key_press_event",
@@ -147,6 +147,9 @@ static void about_create(void)
 #endif
 #if USE_GPGME
 		   " GPGME"
+#endif
+#if USE_SSL
+		   " SSL"
 #endif
 	"");
 
