@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999,2000 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2001 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -44,15 +44,18 @@
 #include "utils.h"
 #include "alertpanel.h"
 
+#include "pixmaps/mark.xpm"
+
 typedef enum
 {
 	COL_DEFAULT	= 0,
-	COL_NAME	= 1,
-	COL_PROTOCOL	= 2,
-	COL_SERVER	= 3
+	COL_GETALL	= 1,
+	COL_NAME	= 2,
+	COL_PROTOCOL	= 3,
+	COL_SERVER	= 4
 } EditAccountColumnPos;
 
-# define N_EDIT_ACCOUNT_COLS	4
+# define N_EDIT_ACCOUNT_COLS	5
 
 #define PREFSBUFSIZE		1024
 
@@ -66,6 +69,9 @@ static struct EditAccount {
 	GtkWidget *close_btn;
 } edit_account;
 
+static GdkPixmap *markxpm;
+static GdkBitmap *markxpmmask;
+
 static void account_edit_create		(void);
 
 static void account_edit_prefs		(void);
@@ -80,6 +86,11 @@ static void account_edit_close		(void);
 static gint account_delete_event	(GtkWidget	*widget,
 					 GdkEventAny	*event,
 					 gpointer	 data);
+static void account_selected		(GtkCList	*clist,
+					 gint		 row,
+					 gint		 column,
+					 GdkEvent	*event,
+					 gpointer	 data);
 static void account_key_pressed		(GtkWidget	*widget,
 					 GdkEventKey	*event,
 					 gpointer	 data);
@@ -167,6 +178,28 @@ PrefsAccount *account_find_from_smtp_server(const gchar *address,
 	return NULL;
 }
 
+/*
+ * account_find_from_address:
+ * @address: Email address string.
+ *
+ * Find a mail (not news) account with the specified email address.
+ *
+ * Return value: The found account, or NULL if not found.
+ */
+PrefsAccount *account_find_from_address(const gchar *address)
+{
+	GList *cur;
+	PrefsAccount *ac;
+
+	for (cur = account_list; cur != NULL; cur = cur->next) {
+		ac = (PrefsAccount *)cur->data;
+		if (ac->protocol != A_NNTP && !strcmp2(address, ac->address))
+			return ac;
+	}
+
+	return NULL;
+}
+
 PrefsAccount *account_find_from_id(gint id)
 {
 	GList *cur;
@@ -202,12 +235,12 @@ GList *account_get_list(void)
 
 void account_edit_open(void)
 {
-	inc_autocheck_timer_remove();
+	inc_lock();
 
 	if (compose_get_compose_list()) {
 		alertpanel_notice(_("Some composing windows are open.\n"
 				    "Please close all the composing windows before editing the accounts."));
-		inc_autocheck_timer_set();
+		inc_unlock();
 		return;
 	}
 
@@ -230,7 +263,6 @@ void account_add(void)
 	PrefsAccount *ac_prefs;
 
 	ac_prefs = prefs_account_open(NULL);
-	inc_autocheck_timer_remove();
 
 	if (!ac_prefs) return;
 
@@ -251,10 +283,13 @@ void account_add(void)
 			folder = folder_new(F_NEWS, ac_prefs->account_name,
 					    ac_prefs->nntp_server);
 		}
+
 		folder->account = ac_prefs;
 		ac_prefs->folder = REMOTE_FOLDER(folder);
 		folder_add(folder);
-		folderview_update_all();
+		if (ac_prefs->protocol == A_IMAP4)
+			folder->create_tree(folder);
+		folderview_set_all();
 	}
 }
 
@@ -304,9 +339,12 @@ void account_set_missing_folder(void)
 				folder = folder_new(F_NEWS, ap->account_name,
 						    ap->nntp_server);
 			}
+
 			folder->account = ap;
 			ap->folder = REMOTE_FOLDER(folder);
 			folder_add(folder);
+			if (ap->protocol == A_IMAP4)
+				folder->create_tree(folder);
 		}
 	}
 }
@@ -367,6 +405,7 @@ static void account_edit_create(void)
 			    GTK_SIGNAL_FUNC (manage_window_focus_in), NULL);
 	gtk_signal_connect (GTK_OBJECT (window), "focus_out_event",
 			    GTK_SIGNAL_FUNC (manage_window_focus_out), NULL);
+	gtk_widget_realize(window);
 
 	vbox = gtk_vbox_new (FALSE, 12);
 	gtk_widget_show (vbox);
@@ -384,7 +423,8 @@ static void account_edit_create(void)
 					GTK_POLICY_AUTOMATIC,
 					GTK_POLICY_AUTOMATIC);
 
-	titles[COL_DEFAULT]  = "";
+	titles[COL_DEFAULT]  = "D";
+	titles[COL_GETALL]   = "G";
 	titles[COL_NAME]     = _("Name");
 	titles[COL_PROTOCOL] = _("Protocol");
 	titles[COL_SERVER]   = _("Server");
@@ -392,9 +432,10 @@ static void account_edit_create(void)
 	clist = gtk_clist_new_with_titles (N_EDIT_ACCOUNT_COLS, titles);
 	gtk_widget_show (clist);
 	gtk_container_add (GTK_CONTAINER (scrolledwin), clist);
-	gtk_clist_set_column_width (GTK_CLIST(clist), COL_DEFAULT , 16);
+	gtk_clist_set_column_width (GTK_CLIST(clist), COL_DEFAULT , 10);
+	gtk_clist_set_column_width (GTK_CLIST(clist), COL_GETALL  , 10);
 	gtk_clist_set_column_width (GTK_CLIST(clist), COL_NAME    , 100);
-	gtk_clist_set_column_width (GTK_CLIST(clist), COL_PROTOCOL, 70);
+	gtk_clist_set_column_width (GTK_CLIST(clist), COL_PROTOCOL, 100);
 	gtk_clist_set_column_width (GTK_CLIST(clist), COL_SERVER  , 100);
 	gtk_clist_set_selection_mode (GTK_CLIST(clist), GTK_SELECTION_BROWSE);
 
@@ -402,6 +443,9 @@ static void account_edit_create(void)
 		GTK_WIDGET_UNSET_FLAGS(GTK_CLIST(clist)->column[i].button,
 				       GTK_CAN_FOCUS);
 
+	gtk_signal_connect (GTK_OBJECT (clist), "select_row",
+			    GTK_SIGNAL_FUNC (account_selected), NULL);
+
 	vbox2 = gtk_vbox_new (FALSE, 0);
 	gtk_widget_show (vbox2);
 	gtk_box_pack_start (GTK_BOX (hbox), vbox2, FALSE, FALSE, 0);
@@ -444,7 +488,7 @@ static void account_edit_create(void)
 	gtk_widget_show (vbox2);
 	gtk_box_pack_start (GTK_BOX (hbox), vbox2, FALSE, FALSE, 0);
 
-	default_btn = gtk_button_new_with_label (_(" Set as usually used account "));
+	default_btn = gtk_button_new_with_label (_(" Set as default account "));
 	gtk_widget_show (default_btn);
 	gtk_box_pack_start (GTK_BOX (vbox2), default_btn, TRUE, FALSE, 0);
 	gtk_signal_connect (GTK_OBJECT(default_btn), "clicked",
@@ -460,6 +504,8 @@ static void account_edit_create(void)
 			    GTK_SIGNAL_FUNC (account_edit_close),
 			    NULL);
 
+	PIXMAP_CREATE(clist, markxpm, markxpmmask, mark_xpm);
+
 	edit_account.window    = window;
 	edit_account.clist     = clist;
 	edit_account.close_btn = close_btn;
@@ -481,7 +527,6 @@ static void account_edit_prefs(void)
 	Xstrdup_a(ac_name, ac_prefs->account_name, return);
 
 	prefs_account_open(ac_prefs);
-	inc_autocheck_timer_remove();
 
 	if (!prev_default && ac_prefs->is_default)
 		account_set_as_default(ac_prefs);
@@ -581,7 +626,7 @@ static void account_edit_close(void)
 
 	gtk_widget_hide(edit_account.window);
 
-	inc_autocheck_timer_set();
+	inc_unlock();
 }
 
 static gint account_delete_event(GtkWidget *widget, GdkEventAny *event,
@@ -591,6 +636,13 @@ static gint account_delete_event(GtkWidget *widget, GdkEventAny *event,
 	return TRUE;
 }
 
+static void account_selected(GtkCList *clist, gint row, gint column,
+			     GdkEvent *event, gpointer data)
+{
+	if (event && event->type == GDK_2BUTTON_PRESS)
+		account_edit_prefs();
+}
+
 static void account_key_pressed(GtkWidget *widget, GdkEventKey *event,
 				gpointer data)
 {
@@ -605,11 +657,25 @@ static gint account_clist_set_row(PrefsAccount *ac_prefs, gint row)
 	gchar *text[N_EDIT_ACCOUNT_COLS];
 
 	text[COL_DEFAULT] = ac_prefs->is_default ? "*" : "";
+	text[COL_GETALL] = (ac_prefs->protocol == A_POP3 ||
+			    ac_prefs->protocol == A_APOP) &&
+			    ac_prefs->recv_at_getall ? "*" : "";
 	text[COL_NAME] = ac_prefs->account_name;
-	text[COL_PROTOCOL] = ac_prefs->protocol == A_POP3  ? "POP3"  :
-			     ac_prefs->protocol == A_APOP  ? "APOP"  :
+#if USE_SSL
+	text[COL_PROTOCOL] = ac_prefs->protocol == A_POP3 ?
+			     (ac_prefs->ssl_pop ? "POP3 (SSL)" : "POP3") :
+			     ac_prefs->protocol == A_APOP ?
+			     (ac_prefs->ssl_pop ?
+			      "POP3 (APOP, SSL)" : "POP3 (APOP)") :
+			     ac_prefs->protocol == A_IMAP4 ?
+			     (ac_prefs->ssl_imap ? "IMAP4 (SSL)" : "IMAP4") :
+			     ac_prefs->protocol == A_NNTP ? "NNTP"  :  "";
+#else
+	text[COL_PROTOCOL] = ac_prefs->protocol == A_POP3  ? "POP3" :
+			     ac_prefs->protocol == A_APOP  ? "POP3 (APOP)" :
 			     ac_prefs->protocol == A_IMAP4 ? "IMAP4" :
-			     ac_prefs->protocol == A_NNTP  ? "NNTP"  :  "";
+			     ac_prefs->protocol == A_NNTP  ? "NNTP" : "";
+#endif
 	text[COL_SERVER] = ac_prefs->protocol == A_NNTP
 		? ac_prefs->nntp_server : ac_prefs->recv_server;
 
@@ -617,11 +683,19 @@ static gint account_clist_set_row(PrefsAccount *ac_prefs, gint row)
 		row = gtk_clist_append(clist, text);
 	else {
 		gtk_clist_set_text(clist, row, COL_DEFAULT, text[COL_DEFAULT]);
+		gtk_clist_set_text(clist, row, COL_GETALL, text[COL_GETALL]);
 		gtk_clist_set_text(clist, row, COL_NAME, text[COL_NAME]);
 		gtk_clist_set_text(clist, row, COL_PROTOCOL, text[COL_PROTOCOL]);
 		gtk_clist_set_text(clist, row, COL_SERVER, text[COL_SERVER]);
 	}
 
+	if (*text[COL_DEFAULT])
+		gtk_clist_set_pixmap(clist, row, COL_DEFAULT,
+				     markxpm, markxpmmask);
+	if (*text[COL_GETALL])
+		gtk_clist_set_pixmap(clist, row, COL_GETALL,
+				     markxpm, markxpmmask);
+
 	gtk_clist_set_row_data(clist, row, ac_prefs);
 
 	return row;
@@ -646,12 +720,16 @@ static void account_clist_set(void)
 		gint row;
 
 		row = account_clist_set_row((PrefsAccount *)cur->data, -1);
-		if ((PrefsAccount *)cur->data == cur_account)
+		if ((PrefsAccount *)cur->data == cur_account) {
 			gtk_clist_select_row(clist, row, -1);
+			gtkut_clist_set_focus_row(clist, row);
+		}
 	}
 
-	if (prev_row >= 0)
+	if (prev_row >= 0) {
 		gtk_clist_select_row(clist, prev_row, -1);
+		gtkut_clist_set_focus_row(clist, prev_row);
+	}
 
 	gtk_clist_thaw(clist);
 }
