@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999,2000 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2001 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -414,8 +414,12 @@ CodeConvFunc conv_get_code_conv_func(const gchar *charset)
 		code_conv = conv_jistodisp;
 	else if (!strcasecmp(charset, CS_US_ASCII))
 		code_conv = conv_ustodisp;
+	else if (!strncasecmp(charset, CS_ISO_8859_1, 10))
+		code_conv = conv_latintodisp;
+#if !HAVE_LIBJCONV
 	else if (!strncasecmp(charset, "ISO-8859-", 9))
 		code_conv = conv_latintodisp;
+#endif
 	else if (!strcasecmp(charset, CS_SHIFT_JIS) ||
 		 !strcasecmp(charset, "SHIFT-JIS")  ||
 		 !strcasecmp(charset, "SJIS")       ||
@@ -695,6 +699,20 @@ const gchar *conv_get_outgoing_charset_str(void)
 	return str ? str : "US-ASCII";
 }
 
+const gchar *conv_get_current_locale(void)
+{
+	gchar *cur_locale;
+
+	cur_locale = g_getenv("LC_ALL");
+	if (!cur_locale) cur_locale = g_getenv("LANG");
+	if (!cur_locale) cur_locale = setlocale(LC_CTYPE, NULL);
+
+	debug_print("current locale: %s\n",
+		    cur_locale ? cur_locale : "(none)");
+
+	return cur_locale;
+}
+
 void conv_unmime_header_overwrite(gchar *str)
 {
 	gchar *buf;
@@ -704,17 +722,24 @@ void conv_unmime_header_overwrite(gchar *str)
 	cur_charset = conv_get_current_charset();
 
 #if HAVE_LIBJCONV
-	Xstrdup_a(buf, str, {return;});
+	Xstrdup_a(buf, str, return);
 	outlen = strlen(str) + 1;
 	UnMimeHeaderConv(buf, str, outlen);
-	if (cur_charset == C_EUC_JP)
-		conv_unreadable_eucjp(str);
+	if (cur_charset == C_EUC_JP) {
+		gchar *tmp;
+		gint len;
+
+		len = strlen(str) * 2 + 1;
+		Xalloca(tmp, len, return);
+		conv_jistodisp(tmp, len, str);
+		strncpy2(str, tmp, outlen);
+	}
 #else
 	if (cur_charset == C_EUC_JP) {
 		gchar *tmp;
 		gint len;
 
-		Xstrdup_a(buf, str, {return;});
+		Xstrdup_a(buf, str, return);
 		outlen = strlen(str) + 1;
 		UnMimeHeader(buf);
 		len = strlen(buf) * 2 + 1;
@@ -732,26 +757,23 @@ void conv_unmime_header(gchar *outbuf, gint outlen, const gchar *str,
 	gchar *buf;
 	CharSet cur_charset;
 
-	Xstrdup_a(buf, str, {return;});
 	cur_charset = conv_get_current_charset();
+	Xstrdup_a(buf, str, return);
 
 #if HAVE_LIBJCONV
 	UnMimeHeaderConv(buf, outbuf, outlen);
-	if (cur_charset == C_EUC_JP)
-		conv_unreadable_eucjp(outbuf);
 #else
 	UnMimeHeader(buf);
+	strncpy2(outbuf, buf, outlen);
+#endif
 	if (cur_charset == C_EUC_JP) {
-		gchar *tmp;
 		gint len;
 
-		len = strlen(buf) * 2 + 1;
-		Xalloca(tmp, len, {strncpy2(outbuf, buf, outlen); return;});
-		conv_jistodisp(tmp, len, buf);
-		strncpy2(outbuf, tmp, outlen);
-	} else
+		len = strlen(outbuf) * 2 + 1;
+		Xalloca(buf, len, return);
+		conv_jistodisp(buf, len, outbuf);
 		strncpy2(outbuf, buf, outlen);
-#endif
+	}
 }
 
 #define MAX_ENCLEN	75
@@ -840,6 +862,7 @@ void conv_encode_header(gchar *dest, gint len, const gchar *src,
 				mbl = wctomb(tmpp, *wtmpp);
 				if (mbl == -1) {
 					g_warning("invalid wide character\n");
+					wtmpp++;
 					continue;
 				}
 				/* g_free(raw); */
@@ -874,23 +897,21 @@ void conv_encode_header(gchar *dest, gint len, const gchar *src,
 							break;
 						}
 					}
-				} else {
-					if ((line_len + tlen + mbl +
-					     (*(wtmpp + 1) ? 0 : nspc) +
-					     (line_len > 1 ? 1 : 0))
-					    > MAX_LINELEN) {
-						g_free(raw_new);
-						if (1 + tlen + mbl +
-						    (*(wtmpp + 1) ? 0 : nspc)
-						    >= MAX_LINELEN) {
-							*tmpp = '\0';
-							break;
-						}
-						*destp++ = '\n';
-						*destp++ = ' ';
-						line_len = 1;
-						continue;
+				} else if ((line_len + tlen + mbl +
+					    (*(wtmpp + 1) ? 0 : nspc) +
+					    (line_len > 1 ? 1 : 0))
+					   > MAX_LINELEN) {
+					g_free(raw_new);
+					if (1 + tlen + mbl +
+					    (*(wtmpp + 1) ? 0 : nspc)
+					    >= MAX_LINELEN) {
+						*tmpp = '\0';
+						break;
 					}
+					*destp++ = '\n';
+					*destp++ = ' ';
+					line_len = 1;
+					continue;
 				}
 
 				tmpp += mbl;
@@ -1068,22 +1089,20 @@ void conv_encode_header(gchar *dest, gint len, const gchar *src,
 							break;
 						}
 					}
-				} else {
-					if ((line_len + tlen + mbl +
-					     (*(wtmpp + 1) ? 0 : nspc) +
-					     (line_len > 1 ? 1 : 0))
-					    > MAX_LINELEN) {
-						if (1 + tlen + mbl +
-						    (*(wtmpp + 1) ? 0 : nspc)
-						    >= MAX_LINELEN) {
-							*tmpp = '\0';
-							break;
-						}
-						*destp++ = '\n';
-						*destp++ = ' ';
-						line_len = 1;
-						continue;
+				} else if ((line_len + tlen + mbl +
+					    (*(wtmpp + 1) ? 0 : nspc) +
+					    (line_len > 1 ? 1 : 0))
+					   > MAX_LINELEN) {
+					if (1 + tlen + mbl +
+					    (*(wtmpp + 1) ? 0 : nspc)
+					    >= MAX_LINELEN) {
+						*tmpp = '\0';
+						break;
 					}
+					*destp++ = '\n';
+					*destp++ = ' ';
+					line_len = 1;
+					continue;
 				}
 
 				tmpp += mbl;
