@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999,2000 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2001 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,17 +32,12 @@ typedef struct _AttachInfo	AttachInfo;
 #include "addressbook.h"
 #include "prefs_account.h"
 
-typedef enum
-{
-	COMPOSE_MAIL,
-	COMPOSE_NEWS
-} ComposeMode;
-
 typedef enum
 {
 	COMPOSE_TO,
 	COMPOSE_CC,
-	COMPOSE_BCC
+	COMPOSE_BCC,
+	COMPOSE_NEWSGROUPS
 } ComposeEntryType;
 
 typedef enum
@@ -50,14 +45,17 @@ typedef enum
 	COMPOSE_REPLY,
 	COMPOSE_REPLY_WITH_QUOTE,
 	COMPOSE_REPLY_WITHOUT_QUOTE,
+	COMPOSE_REPLY_TO_SENDER,
+	COMPOSE_REPLY_TO_SENDER_WITH_QUOTE,
+	COMPOSE_REPLY_TO_SENDER_WITHOUT_QUOTE,
 	COMPOSE_REPLY_TO_ALL,
 	COMPOSE_REPLY_TO_ALL_WITH_QUOTE,
 	COMPOSE_REPLY_TO_ALL_WITHOUT_QUOTE,
 	COMPOSE_FORWARD,
 	COMPOSE_FORWARD_AS_ATTACH,
 	COMPOSE_NEW,
-	COMPOSE_REEDIT_DRAFT
-} ComposeReplyMode;
+	COMPOSE_REEDIT
+} ComposeMode;
 
 struct _Compose
 {
@@ -112,7 +110,7 @@ struct _Compose
 
 	GtkItemFactory *popupfactory;
 
-	ComposeReplyMode mode;
+	ComposeMode mode;
 
 	MsgInfo *targetinfo;
 
@@ -169,7 +167,8 @@ void compose_new_with_recipient	(PrefsAccount	*account,
 
 void compose_reply		(MsgInfo	*msginfo,
 				 gboolean	 quote,
-				 gboolean	 to_all);
+				 gboolean	 to_all,
+				 gboolean	 ignore_replyto);
 void compose_forward		(MsgInfo	*msginfo,
 				 gboolean	 as_attach);
 void compose_reedit		(MsgInfo	*msginfo);
