@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999,2000 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2001 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,7 +32,6 @@
 #  include <sys/param.h>
 #endif
 
-#define PROG_VERSION		"Sylpheed version "VERSION
 #define INBOX_DIR		"inbox"
 #define OUTBOX_DIR		"outbox"
 #define QUEUE_DIR		"queue"
@@ -45,8 +44,10 @@
 #define COMMON_RC		"sylpheedrc"
 #define ACCOUNT_RC		"accountrc"
 #define FILTER_RC		"filterrc"
+#define CUSTOM_HEADER_RC	"customheaderrc"
 #define DISPLAY_HEADER_RC	"dispheaderrc"
 #define MENU_RC			"menurc"
+#define NEWSGROUP_LIST		".newsgroup_list"
 #define ADDRESS_BOOK		"addressbook.xml"
 #define MANUAL_HTML_INDEX	"sylpheed.html"
 #define HOMEPAGE_URI		"http://sylpheed.good-day.net/"
@@ -61,6 +62,7 @@
 #define DEFAULT_INC_PROGRAM	"inc"
 /* #define DEFAULT_INC_PATH	"/usr/bin/imget" */
 /* #define DEFAULT_INC_PROGRAM	"imget" */
+#define DEFAULT_SENDMAIL_CMD	"/usr/sbin/sendmail -t"
 
 #ifdef _PATH_MAILDIR
 #  define DEFAULT_SPOOL_PATH	_PATH_MAILDIR
@@ -84,12 +86,13 @@
 #define CTREE_INDENT			18
 #define FOLDER_SPACING			4
 #define COLOR_DIM			35000
+#define UI_REFRESH_INTERVAL		50000
 
-#define NORMAL_FONT	"-*-helvetica-medium-r-normal--12-*," \
-			"-*-*-medium-r-normal--12-*-*-*-*-*-*-*,*"
-#define BOLD_FONT	"-*-helvetica-bold-r-normal--12-*," \
-			"-*-*-bold-r-normal--12-*-*-*-*-*-*-*,*"
-#define SMALL_FONT	"-*-helvetica-medium-r-normal--10-*," \
-			"-*-*-medium-r-normal--10-*-*-*-*-*-*-*,*"
+#define DEFAULT_NORMAL_FONT	"-*-helvetica-medium-r-normal--12-*," \
+				"-*-*-medium-r-normal--12-*-*-*-*-*-*-*,*"
+#define DEFAULT_BOLD_FONT	"-*-helvetica-bold-r-normal--12-*," \
+				"-*-*-bold-r-normal--12-*-*-*-*-*-*-*,*"
+#define DEFAULT_SMALL_FONT	"-*-helvetica-medium-r-normal--10-*," \
+				"-*-*-medium-r-normal--10-*-*-*-*-*-*-*,*"
 
 #endif /* __DEFS_H__ */
