@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999,2000 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2001 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -139,6 +139,9 @@ FolderItem *folder_item_new(const gchar *name, const gchar *path)
 	item->unread = 0;
 	item->total = 0;
 	item->last_num = -1;
+	item->no_sub = FALSE;
+	item->no_select = FALSE;
+	item->collapsed = FALSE;
 	item->parent = NULL;
 	item->folder = NULL;
 	item->data = NULL;
@@ -180,6 +183,15 @@ void folder_item_remove(FolderItem *item)
 	g_node_destroy(node);
 }
 
+void folder_item_destroy(FolderItem *item)
+{
+	g_return_if_fail(item != NULL);
+
+	g_free(item->name);
+	g_free(item->path);
+	g_free(item);
+}
+
 void folder_set_ui_func(Folder *folder, FolderUIFunc func, gpointer data)
 {
 	g_return_if_fail(folder != NULL);
@@ -230,6 +242,13 @@ void folder_tree_destroy(Folder *folder)
 {
 	/* TODO: destroy all FolderItem before */
 	g_node_destroy(folder->node);
+
+	folder->inbox = NULL;
+	folder->outbox = NULL;
+	folder->draft = NULL;
+	folder->queue = NULL;
+	folder->trash = NULL;
+	folder->node = NULL;
 }
 
 void folder_add(Folder *folder)
@@ -266,8 +285,11 @@ gint folder_read_list(void)
 {
 	GNode *node;
 	XMLNode *xmlnode;
+	gchar *path;
 
-	node = xml_parse_file(folder_get_list_path());
+	path = folder_get_list_path();
+	if (!is_file_exist(path)) return -1;
+	node = xml_parse_file(path);
 	if (!node) return -1;
 
 	xmlnode = node->data;
@@ -358,13 +380,14 @@ FolderItem *folder_find_item_from_path(const gchar *path)
 
 Folder *folder_get_default_folder(void)
 {
-	return FOLDER(folder_list->data);
+	return folder_list ? FOLDER(folder_list->data) : NULL;
 }
 
 FolderItem *folder_get_default_inbox(void)
 {
 	Folder *folder;
 
+	if (!folder_list) return NULL;
 	folder = FOLDER(folder_list->data);
 	g_return_val_if_fail(folder != NULL, NULL);
 	return folder->inbox;
@@ -374,6 +397,7 @@ FolderItem *folder_get_default_outbox(void)
 {
 	Folder *folder;
 
+	if (!folder_list) return NULL;
 	folder = FOLDER(folder_list->data);
 	g_return_val_if_fail(folder != NULL, NULL);
 	return folder->outbox;
@@ -383,6 +407,7 @@ FolderItem *folder_get_default_draft(void)
 {
 	Folder *folder;
 
+	if (!folder_list) return NULL;
 	folder = FOLDER(folder_list->data);
 	g_return_val_if_fail(folder != NULL, NULL);
 	return folder->draft;
@@ -392,6 +417,7 @@ FolderItem *folder_get_default_queue(void)
 {
 	Folder *folder;
 
+	if (!folder_list) return NULL;
 	folder = FOLDER(folder_list->data);
 	g_return_val_if_fail(folder != NULL, NULL);
 	return folder->queue;
@@ -401,6 +427,7 @@ FolderItem *folder_get_default_trash(void)
 {
 	Folder *folder;
 
+	if (!folder_list) return NULL;
 	folder = FOLDER(folder_list->data);
 	g_return_val_if_fail(folder != NULL, NULL);
 	return folder->trash;
@@ -487,7 +514,8 @@ gchar *folder_item_fetch_msg(FolderItem *item, gint num)
 	return folder->fetch_msg(folder, item, num);
 }
 
-gint folder_item_add_msg(FolderItem *dest, const gchar *file)
+gint folder_item_add_msg(FolderItem *dest, const gchar *file,
+			 gboolean remove_source)
 {
 	Folder *folder;
 	gint num;
@@ -499,7 +527,7 @@ gint folder_item_add_msg(FolderItem *dest, const gchar *file)
 	folder = dest->folder;
 	if (dest->last_num < 0) folder->scan(folder, dest);
 
-	num = folder->add_msg(folder, dest, file);
+	num = folder->add_msg(folder, dest, file, remove_source);
 	if (num > 0) dest->last_num = num;
 
 	return num;
@@ -576,13 +604,18 @@ gint folder_item_copy_msgs_with_dest(FolderItem *dest, GSList *msglist)
 gint folder_item_remove_msg(FolderItem *item, gint num)
 {
 	Folder *folder;
+	gint ret;
 
 	g_return_val_if_fail(item != NULL, -1);
 
 	folder = item->folder;
 	if (item->last_num < 0) folder->scan(folder, item);
 
-	return folder->remove_msg(folder, item, num);
+	ret = folder->remove_msg(folder, item, num);
+	if (ret == 0 && num == item->last_num)
+		folder->scan(folder, item);
+
+	return ret;
 }
 
 gint folder_item_remove_all_msg(FolderItem *item)
@@ -687,11 +720,16 @@ static void folder_init(Folder *folder, FolderType type, const gchar *name)
 	case F_IMAP:
 		folder->get_msg_list        = imap_get_msg_list;
 		folder->fetch_msg           = imap_fetch_msg;
+		folder->add_msg             = imap_add_msg;
 		folder->move_msg            = imap_move_msg;
 		folder->move_msgs_with_dest = imap_move_msgs_with_dest;
+		folder->copy_msg            = imap_copy_msg;
+		folder->copy_msgs_with_dest = imap_copy_msgs_with_dest;
 		folder->remove_msg          = imap_remove_msg;
 		folder->remove_all_msg      = imap_remove_all_msg;
 		folder->scan                = imap_scan_folder;
+		folder->scan_tree           = imap_scan_tree;
+		folder->create_tree         = imap_create_tree;
 		folder->create_folder       = imap_create_folder;
 		folder->remove_folder       = imap_remove_folder;
 		break;
@@ -711,7 +749,7 @@ static void folder_init(Folder *folder, FolderType type, const gchar *name)
 		break;
 	case F_IMAP:
 	case F_NEWS:
-		REMOTE_FOLDER(folder)->session   = NULL;
+		REMOTE_FOLDER(folder)->session = NULL;
 		break;
 	default:
 	}
@@ -757,6 +795,7 @@ static gboolean folder_build_tree(GNode *node, gpointer data)
 	const gchar *name = NULL;
 	const gchar *path = NULL;
 	PrefsAccount *account = NULL;
+	gboolean no_sub = FALSE, no_select = FALSE, collapsed = FALSE;
 	gint mtime = 0, new = 0, unread = 0, total = 0;
 
 	g_return_val_if_fail(node->data != NULL, FALSE);
@@ -794,8 +833,7 @@ static gboolean folder_build_tree(GNode *node, gpointer data)
 			account = account_find_from_id(atoi(attr->value));
 			if (!account) g_warning("account_id: %s not found\n",
 						attr->value);
-		}
-		else if (!strcmp(attr->name, "mtime"))
+		} else if (!strcmp(attr->name, "mtime"))
 			mtime = atoi(attr->value);
 		else if (!strcmp(attr->name, "new"))
 			new = atoi(attr->value);
@@ -803,6 +841,12 @@ static gboolean folder_build_tree(GNode *node, gpointer data)
 			unread = atoi(attr->value);
 		else if (!strcmp(attr->name, "total"))
 			total = atoi(attr->value);
+		else if (!strcmp(attr->name, "no_sub"))
+			no_sub = *attr->value == '1' ? TRUE : FALSE;
+		else if (!strcmp(attr->name, "no_select"))
+			no_select = *attr->value == '1' ? TRUE : FALSE;
+		else if (!strcmp(attr->name, "collapsed"))
+			collapsed = *attr->value == '1' ? TRUE : FALSE;
 	}
 
 	item = folder_item_new(name, path);
@@ -812,6 +856,9 @@ static gboolean folder_build_tree(GNode *node, gpointer data)
 	item->new = new;
 	item->unread = unread;
 	item->total = total;
+	item->no_sub = no_sub;
+	item->no_select = no_select;
+	item->collapsed = collapsed;
 	item->parent = FOLDER_ITEM(node->parent->data);
 	item->folder = folder;
 	switch (stype) {
@@ -837,6 +884,7 @@ static gboolean folder_read_folder_func(GNode *node, gpointer data)
 	const gchar *name = NULL;
 	const gchar *path = NULL;
 	PrefsAccount *account = NULL;
+	gboolean collapsed = FALSE;
 
 	if (g_node_depth(node) != 2) return FALSE;
 	g_return_val_if_fail(node->data != NULL, FALSE);
@@ -871,7 +919,8 @@ static gboolean folder_read_folder_func(GNode *node, gpointer data)
 			account = account_find_from_id(atoi(attr->value));
 			if (!account) g_warning("account_id: %s not found\n",
 						attr->value);
-		}
+		} else if (!strcmp(attr->name, "collapsed"))
+			collapsed = *attr->value == '1' ? TRUE : FALSE;
 	}
 
 	folder = folder_new(type, name, path);
@@ -883,6 +932,7 @@ static gboolean folder_read_folder_func(GNode *node, gpointer data)
 	g_node_destroy(folder->node);
 	folder->node = node;
 	folder_add(folder);
+	FOLDER_ITEM(node->data)->collapsed = collapsed;
 
 	g_node_traverse(node, G_PRE_ORDER, G_TRAVERSE_ALL, -1,
 			folder_build_tree, folder);
@@ -934,6 +984,8 @@ static void folder_write_list_recursive(GNode *node, gpointer data)
 		if (folder->account)
 			fprintf(fp, " account_id=\"%d\"",
 				folder->account->account_id);
+		if (item->collapsed && node->children)
+			fputs(" collapsed=\"1\"", fp);
 	} else {
 		fprintf(fp, "<folderitem type=\"%s\"",
 			folder_item_stype_str[item->stype]);
@@ -948,8 +1000,14 @@ static void folder_write_list_recursive(GNode *node, gpointer data)
 			fputs("\"", fp);
 		}
 		if (item->account)
-			fprintf(fp, " account_id = \"%d\"",
+			fprintf(fp, " account_id=\"%d\"",
 				item->account->account_id);
+		if (item->no_sub)
+			fputs(" no_sub=\"1\"", fp);
+		if (item->no_select)
+			fputs(" no_select=\"1\"", fp);
+		if (item->collapsed && node->children)
+			fputs(" collapsed=\"1\"", fp);
 		fprintf(fp,
 			" mtime=\"%ld\" new=\"%d\" unread=\"%d\" total=\"%d\"",
 			item->mtime, item->new, item->unread, item->total);
