@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999,2000 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2001 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -43,6 +43,10 @@ typedef struct _FolderItem	FolderItem;
 #define LOCAL_FOLDER(obj)	((LocalFolder *)obj)
 #define REMOTE_FOLDER(obj)	((RemoteFolder *)obj)
 
+#define FOLDER_IS_LOCAL(obj)	(FOLDER_TYPE(obj) == F_MH      || \
+				 FOLDER_TYPE(obj) == F_MBOX    || \
+				 FOLDER_TYPE(obj) == F_MAILDIR)
+
 #define MH_FOLDER(obj)		((MHFolder *)obj)
 #define MBOX_FOLDER(obj)	((MboxFolder *)obj)
 #define MAILDIR_FOLDER(obj)	((MaildirFolder *)obj)
@@ -107,7 +111,8 @@ struct _Folder
 					 gint		 num);
 	gint     (*add_msg)		(Folder		*folder,
 					 FolderItem	*dest,
-					 const gchar	*file);
+					 const gchar	*file,
+					 gboolean	 remove_source);
 	gint     (*move_msg)		(Folder		*folder,
 					 FolderItem	*dest,
 					 MsgInfo	*msginfo);
@@ -175,6 +180,8 @@ struct _MaildirFolder
 struct _IMAPFolder
 {
 	RemoteFolder rfolder;
+
+	GList *namespace;	/* list of IMAPNameSpace */
 };
 
 struct _NewsFolder
@@ -200,6 +207,11 @@ struct _FolderItem
 
 	gint last_num;
 
+	/* special flags */
+	guint no_sub    : 1; /* no child allowed? */
+	guint no_select : 1; /* not selectable?   */
+	guint collapsed : 1; /* collapsed item    */
+
 	FolderItem *parent;
 
 	Folder *folder;
@@ -225,6 +237,7 @@ FolderItem *folder_item_new	(const gchar	*name,
 void        folder_item_append	(FolderItem	*parent,
 				 FolderItem	*item);
 void        folder_item_remove	(FolderItem	*item);
+void        folder_item_destroy	(FolderItem	*item);
 void        folder_set_ui_func	(Folder		*folder,
 				 FolderUIFunc	 func,
 				 gpointer	 data);
@@ -254,7 +267,8 @@ void   folder_item_scan_foreach		(GHashTable	*table);
 gchar *folder_item_fetch_msg		(FolderItem	*item,
 					 gint		 num);
 gint   folder_item_add_msg		(FolderItem	*dest,
-					 const gchar	*file);
+					 const gchar	*file,
+					 gboolean	 remove_source);
 gint   folder_item_move_msg		(FolderItem	*dest,
 					 MsgInfo	*msginfo);
 gint   folder_item_move_msgs_with_dest	(FolderItem	*dest,
