@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999,2000 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2001 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -43,6 +43,7 @@
 #include "folderview.h"
 #include "summaryview.h"
 #include "inputdialog.h"
+#include "grouplistdialog.h"
 #include "manage_window.h"
 #include "alertpanel.h"
 #include "menu.h"
@@ -90,6 +91,9 @@ static GList *folderview_list = NULL;
 static GdkFont *normalfont;
 static GdkFont *boldfont;
 
+static GtkStyle *bold_style;
+static GtkStyle *bold_color_style;
+
 static GdkPixmap *inboxxpm;
 static GdkBitmap *inboxxpmmask;
 static GdkPixmap *outboxxpm;
@@ -133,6 +137,12 @@ static void folderview_selected		(GtkCTree	*ctree,
 					 GtkCTreeNode	*row,
 					 gint		 column,
 					 FolderView	*folderview);
+static void folderview_tree_expanded	(GtkCTree	*ctree,
+					 GtkCTreeNode	*node,
+					 FolderView	*folderview);
+static void folderview_tree_collapsed	(GtkCTree	*ctree,
+					 GtkCTreeNode	*node,
+					 FolderView	*folderview);
 static void folderview_popup_close	(GtkMenuShell	*menu_shell,
 					 FolderView	*folderview);
 static void folderview_col_resized	(GtkCList	*clist,
@@ -140,6 +150,10 @@ static void folderview_col_resized	(GtkCList	*clist,
 					 gint		 width,
 					 FolderView	*folderview);
 
+static void folderview_update_tree_cb	(FolderView	*folderview,
+					 guint		 action,
+					 GtkWidget	*widget);
+
 static void folderview_new_folder_cb	(FolderView	*folderview,
 					 guint		 action,
 					 GtkWidget	*widget);
@@ -198,6 +212,7 @@ static GtkItemFactoryEntry folderview_mail_popup_entries[] =
 	{N_("/_Rename folder..."),	NULL, folderview_rename_folder_cb, 0, NULL},
 	{N_("/_Delete folder"),		NULL, folderview_delete_folder_cb, 0, NULL},
 	{N_("/---"),			NULL, NULL, 0, "<Separator>"},
+	{N_("/_Update folder tree"),	NULL, folderview_update_tree_cb, 0, NULL},
 	{N_("/Remove _mailbox"),	NULL, folderview_remove_mailbox_cb, 0, NULL},
 	{N_("/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/_Property..."),		NULL, NULL, 0, NULL}
@@ -209,7 +224,8 @@ static GtkItemFactoryEntry folderview_imap_popup_entries[] =
 	{N_("/_Rename folder..."),	NULL, NULL, 0, NULL},
 	{N_("/_Delete folder"),		NULL, folderview_rm_imap_folder_cb, 0, NULL},
 	{N_("/---"),			NULL, NULL, 0, "<Separator>"},
-	{N_("/Remove _IMAP4 server"),	NULL, folderview_rm_imap_server_cb, 0, NULL},
+	{N_("/_Update folder tree"),	NULL, folderview_update_tree_cb, 0, NULL},
+	{N_("/Remove _IMAP4 account"),	NULL, folderview_rm_imap_server_cb, 0, NULL},
 	{N_("/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/_Property..."),		NULL, NULL, 0, NULL}
 };
@@ -220,7 +236,7 @@ static GtkItemFactoryEntry folderview_news_popup_entries[] =
 					 NULL, folderview_new_news_group_cb, 0, NULL},
 	{N_("/_Remove newsgroup"),	 NULL, folderview_rm_news_group_cb, 0, NULL},
 	{N_("/---"),			 NULL, NULL, 0, "<Separator>"},
-	{N_("/Remove _news server"),	 NULL, folderview_rm_news_server_cb, 0, NULL},
+	{N_("/Remove _news account"),	 NULL, folderview_rm_news_server_cb, 0, NULL},
 	{N_("/---"),			 NULL, NULL, 0, "<Separator>"},
 	{N_("/_Property..."),		 NULL, NULL, 0, NULL}
 };
@@ -252,7 +268,6 @@ FolderView *folderview_create(void)
 	gtk_widget_set_usize(scrolledwin,
 			     prefs_common.folderview_width,
 			     prefs_common.folderview_height);
-	//		     COL_FOLDER_WIDTH + COL_NUM_WIDTH - 3, -1);
 
 	ctree = gtk_ctree_new_with_titles(N_FOLDER_COLS, COL_FOLDER, titles);
 	gtk_container_add(GTK_CONTAINER(scrolledwin), ctree);
@@ -275,7 +290,7 @@ FolderView *folderview_create(void)
 	gtk_ctree_set_expander_style(GTK_CTREE(ctree),
 				     GTK_CTREE_EXPANDER_SQUARE);
 	gtk_ctree_set_indent(GTK_CTREE(ctree), CTREE_INDENT);
-	//gtk_clist_set_reorderable(GTK_CLIST(ctree), TRUE);
+
 	/* don't let title buttons take key focus */
 	for (i = 0; i < N_FOLDER_COLS; i++)
 		GTK_WIDGET_UNSET_FLAGS(GTK_CLIST(ctree)->column[i].button,
@@ -312,6 +327,14 @@ FolderView *folderview_create(void)
 			   folderview);
 	gtk_signal_connect(GTK_OBJECT(ctree), "tree_select_row",
 			   GTK_SIGNAL_FUNC(folderview_selected), folderview);
+
+	gtk_signal_connect_after(GTK_OBJECT(ctree), "tree_expand",
+				 GTK_SIGNAL_FUNC(folderview_tree_expanded),
+				 folderview);
+	gtk_signal_connect_after(GTK_OBJECT(ctree), "tree_collapse",
+				 GTK_SIGNAL_FUNC(folderview_tree_collapsed),
+				 folderview);
+
 	gtk_signal_connect(GTK_OBJECT(ctree), "resize_column",
 			   GTK_SIGNAL_FUNC(folderview_col_resized),
 			   folderview);
@@ -363,15 +386,21 @@ void folderview_init(FolderView *folderview)
 
 	PIXMAP_CREATE(ctree, inboxxpm, inboxxpmmask, inbox_xpm);
 	PIXMAP_CREATE(ctree, outboxxpm, outboxxpmmask, outbox_xpm);
-	PIXMAP_CREATE(ctree, folderxpm, folderxpmmask, DIRECTORY_CLOSE_XPM);
-	PIXMAP_CREATE(ctree, folderopenxpm, folderopenxpmmask,
-		      DIRECTORY_OPEN_XPM);
+	PIXMAP_CREATE(ctree, folderxpm, folderxpmmask, dir_close_xpm);
+	PIXMAP_CREATE(ctree, folderopenxpm, folderopenxpmmask, dir_open_xpm);
 	PIXMAP_CREATE(ctree, trashxpm, trashxpmmask, trash_xpm);
 
 	if (!normalfont)
-		normalfont = gdk_fontset_load(NORMAL_FONT);
+		normalfont = gdk_fontset_load(prefs_common.normalfont);
 	if (!boldfont)
-		boldfont = gdk_fontset_load(BOLD_FONT);
+		boldfont = gdk_fontset_load(prefs_common.boldfont);
+
+	if (!bold_style) {
+		bold_style = gtk_style_copy(gtk_widget_get_style(ctree));
+		bold_style->font = boldfont;
+		bold_color_style = gtk_style_copy(bold_style);
+		bold_color_style->fg[GTK_STATE_NORMAL] = folderview->color_new;
+	}
 }
 
 void folderview_set(FolderView *folderview)
@@ -399,6 +428,14 @@ void folderview_set(FolderView *folderview)
 	STATUSBAR_POP(mainwin);
 }
 
+void folderview_set_all(void)
+{
+	GList *list;
+
+	for (list = folderview_list; list != NULL; list = list->next)
+		folderview_set((FolderView *)list->data);
+}
+
 void folderview_select(FolderView *folderview, FolderItem *item)
 {
 	GtkCTree *ctree = GTK_CTREE(folderview->ctree);
@@ -423,6 +460,8 @@ static void folderview_select_node(FolderView *folderview, GtkCTreeNode *node)
 		gtk_widget_grab_focus(folderview->summaryview->ctree);
 	else
 		gtk_widget_grab_focus(folderview->ctree);
+
+	gtkut_ctree_expand_parent_all(ctree, node);
 }
 
 void folderview_unselect(FolderView *folderview)
@@ -434,36 +473,42 @@ void folderview_unselect(FolderView *folderview)
 	folderview->selected = folderview->opened = NULL;
 }
 
+static GtkCTreeNode *folderview_find_next_unread(GtkCTree *ctree,
+						 GtkCTreeNode *node)
+{
+	FolderItem *item;
+
+	if (node)
+		node = gtkut_ctree_node_next(ctree, node);
+	else
+		node = GTK_CTREE_NODE(GTK_CLIST(ctree)->row_list);
+
+	for (; node != NULL; node = gtkut_ctree_node_next(ctree, node)) {
+		item = gtk_ctree_node_get_row_data(ctree, node);
+		if (item && item->unread > 0 && item->stype != F_TRASH)
+			return node;
+	}
+
+	return NULL;
+}
+
 void folderview_select_next_unread(FolderView *folderview)
 {
 	GtkCTree *ctree = GTK_CTREE(folderview->ctree);
-	GtkCTreeNode *node;
-	FolderItem *item;
+	GtkCTreeNode *node = NULL;
 
-	if (folderview->opened)
-		node = GTK_CTREE_NODE_NEXT(folderview->opened);
-	else
-		node = gtk_ctree_node_nth(ctree, 1);
-
-	for (; node != NULL; node = GTK_CTREE_NODE_NEXT(node)) {
-		item = gtk_ctree_node_get_row_data(ctree, node);
-		if (item && (item->unread > 0 || item->new > 0) &&
-		    item->stype != F_TRASH) {
-			folderview_select_node(folderview, node);
-			return;
-		}
+	if ((node = folderview_find_next_unread(ctree, folderview->opened))
+	    != NULL) {
+		folderview_select_node(folderview, node);
+		return;
 	}
-	if (!folderview->opened) return;
+
+	if (!folderview->opened ||
+	    folderview->opened == GTK_CTREE_NODE(GTK_CLIST(ctree)->row_list))
+		return;
 	/* search again from the first node */
-	for (node = gtk_ctree_node_nth(ctree, 1); node != NULL;
-	     node = GTK_CTREE_NODE_NEXT(node)) {
-		item = gtk_ctree_node_get_row_data(ctree, node);
-		if (item && (item->unread > 0 || item->new > 0) &&
-		    item->stype != F_TRASH) {
-			folderview_select_node(folderview, node);
-			return;
-		}
-	}
+	if ((node = folderview_find_next_unread(ctree, NULL)) != NULL)
+		folderview_select_node(folderview, node);
 }
 
 void folderview_update_msg_num(FolderView *folderview, GtkCTreeNode *row,
@@ -506,6 +551,15 @@ static void folderview_scan_tree_func(Folder *folder, FolderItem *item,
 				      gpointer data)
 {
 	GList *list;
+	gchar *rootpath;
+
+	if (FOLDER_IS_LOCAL(folder))
+		rootpath = LOCAL_FOLDER(folder)->rootpath;
+	else if (folder->type == F_IMAP && folder->account &&
+		 folder->account->recv_server)
+		rootpath = folder->account->recv_server;
+	else
+		return;
 
 	for (list = folderview_list; list != NULL; list = list->next) {
 		FolderView *folderview = (FolderView *)list->data;
@@ -514,12 +568,11 @@ static void folderview_scan_tree_func(Folder *folder, FolderItem *item,
 
 		if (item->path)
 			str = g_strdup_printf(_("Scanning folder %s%c%s ..."),
-					      LOCAL_FOLDER(folder)->rootpath,
-					      G_DIR_SEPARATOR,
+					      rootpath, G_DIR_SEPARATOR,
 					      item->path);
 		else
 			str = g_strdup_printf(_("Scanning folder %s ..."),
-					      LOCAL_FOLDER(folder)->rootpath);
+					      rootpath);
 
 		STATUSBAR_PUSH(mainwin, str);
 		STATUSBAR_POP(mainwin);
@@ -550,6 +603,26 @@ static GtkWidget *label_window_create(const gchar *str)
 	return window;
 }
 
+void folderview_update_tree(Folder *folder)
+{
+	GtkWidget *window;
+
+	g_return_if_fail(folder != NULL);
+
+	if (!folder->scan_tree) return;
+
+	window = label_window_create(_("Updating folder tree..."));
+
+	folder_set_ui_func(folder, folderview_scan_tree_func, NULL);
+	folder->scan_tree(folder);
+	folder_set_ui_func(folder, NULL, NULL);
+
+	folder_write_list();
+	folderview_set_all();
+
+	gtk_widget_destroy(window);
+}
+
 void folderview_update_all(void)
 {
 	GList *list;
@@ -568,24 +641,144 @@ void folderview_update_all(void)
 	}
 
 	folder_write_list();
+	folderview_set_all();
+	gtk_widget_destroy(window);
+}
+
+void folderview_update_all_node(void)
+{
+	GList *list;
+	FolderItem *item;
+	FolderView *folderview;
+	GtkCTree *ctree;
+	GtkCTreeNode *node;
+	GtkWidget *window;
+
+	window = label_window_create(_("Updating all folders..."));
 
 	for (list = folderview_list; list != NULL; list = list->next) {
-		FolderView *folderview = (FolderView *)list->data;
+		folderview = (FolderView *)list->data;
+		ctree = GTK_CTREE(folderview->ctree);
 
-		folderview_set(folderview);
+		for (node = GTK_CTREE_NODE(GTK_CLIST(ctree)->row_list);
+		     node != NULL; node = gtkut_ctree_node_next(ctree, node)) {
+			item = gtk_ctree_node_get_row_data(ctree, node);
+			if (!item || !FOLDER_IS_LOCAL(item->folder) ||
+			    !item->path)
+				continue;
+			folderview_scan_tree_func(item->folder, item, NULL);
+			folder_item_scan(item);
+			folderview_update_node(folderview, node);
+		}
 	}
 
+	folder_write_list();
 	gtk_widget_destroy(window);
 }
 
+static gboolean folderview_search_new_recursive(GtkCTree *ctree,
+						GtkCTreeNode *node)
+{
+	FolderItem *item;
+
+	if (node) {
+		item = gtk_ctree_node_get_row_data(ctree, node);
+		if (item) {
+			if (item->new > 0 ||
+			    (item->stype == F_QUEUE && item->total > 0))
+				return TRUE;
+		}
+		node = GTK_CTREE_ROW(node)->children;
+	} else
+		node = GTK_CTREE_NODE(GTK_CLIST(ctree)->row_list);
+
+	while (node) {
+		if (folderview_search_new_recursive(ctree, node) == TRUE)
+			return TRUE;
+		node = GTK_CTREE_ROW(node)->sibling;
+	}
+
+	return FALSE;
+}
+
+static gboolean folderview_have_new_children(FolderView *folderview,
+					     GtkCTreeNode *node)
+{
+	GtkCTree *ctree = GTK_CTREE(folderview->ctree);
+
+	if (!node)
+		node = GTK_CTREE_NODE(GTK_CLIST(ctree)->row_list);
+	if (!node)
+		return FALSE;
+
+	node = GTK_CTREE_ROW(node)->children;
+
+	while (node) {
+		if (folderview_search_new_recursive(ctree, node) == TRUE)
+			return TRUE;
+		node = GTK_CTREE_ROW(node)->sibling;
+	}
+
+	return FALSE;
+}
+
+static gboolean folderview_search_unread_recursive(GtkCTree *ctree,
+						   GtkCTreeNode *node)
+{
+	FolderItem *item;
+
+	if (node) {
+		item = gtk_ctree_node_get_row_data(ctree, node);
+		if (item) {
+			if (item->unread > 0 ||
+			    (item->stype == F_QUEUE && item->total > 0))
+				return TRUE;
+		}
+		node = GTK_CTREE_ROW(node)->children;
+	} else
+		node = GTK_CTREE_NODE(GTK_CLIST(ctree)->row_list);
+
+	while (node) {
+		if (folderview_search_unread_recursive(ctree, node) == TRUE)
+			return TRUE;
+		node = GTK_CTREE_ROW(node)->sibling;
+	}
+
+	return FALSE;
+}
+
+static gboolean folderview_have_unread_children(FolderView *folderview,
+						GtkCTreeNode *node)
+{
+	GtkCTree *ctree = GTK_CTREE(folderview->ctree);
+
+	if (!node)
+		node = GTK_CTREE_NODE(GTK_CLIST(ctree)->row_list);
+	if (!node)
+		return FALSE;
+
+	node = GTK_CTREE_ROW(node)->children;
+
+	while (node) {
+		if (folderview_search_unread_recursive(ctree, node) == TRUE)
+			return TRUE;
+		node = GTK_CTREE_ROW(node)->sibling;
+	}
+
+	return FALSE;
+}
+
 static void folderview_update_node(FolderView *folderview, GtkCTreeNode *node)
 {
 	GtkCTree *ctree = GTK_CTREE(folderview->ctree);
-	GtkStyle *style, *prev_style, *ctree_style;
+	GtkStyle *style = NULL;
 	FolderItem *item;
 	GdkPixmap *xpm, *openxpm;
 	GdkBitmap *mask, *openmask;
 	gchar *name;
+	gchar *str;
+	gboolean add_unread_mark;
+	gboolean use_bold, use_color;
 
 	item = gtk_ctree_node_get_row_data(ctree, node);
 	g_return_if_fail(item != NULL);
@@ -638,10 +831,29 @@ static void folderview_update_node(FolderView *folderview, GtkCTreeNode *node)
 		} else
 			name = g_strdup(item->name);
 	}
-	if (item->unread > 0 && prefs_common.display_folder_unread) {
-		gchar *str;
 
-		str = g_strdup_printf("%s (%d)", name, item->unread);
+	if (!GTK_CTREE_ROW(node)->expanded &&
+	    folderview_have_unread_children(folderview, node))
+		add_unread_mark = TRUE;
+	else
+		add_unread_mark = FALSE;
+
+	if (item->stype == F_QUEUE && item->total > 0 &&
+	    prefs_common.display_folder_unread) {
+		str = g_strdup_printf("%s (%d%s)", name, item->total,
+				      add_unread_mark ? "+" : "");
+		gtk_ctree_set_node_info(ctree, node, str, FOLDER_SPACING,
+					xpm, mask, openxpm, openmask,
+					FALSE, GTK_CTREE_ROW(node)->expanded);
+		g_free(str);
+	} else if ((item->unread > 0 || add_unread_mark) &&
+		 prefs_common.display_folder_unread) {
+
+		if (item->unread > 0)
+			str = g_strdup_printf("%s (%d%s)", name, item->unread,
+					      add_unread_mark ? "+" : "");
+		else
+			str = g_strdup_printf("%s (+)", name);
 		gtk_ctree_set_node_info(ctree, node, str, FOLDER_SPACING,
 					xpm, mask, openxpm, openmask,
 					FALSE, GTK_CTREE_ROW(node)->expanded);
@@ -656,51 +868,41 @@ static void folderview_update_node(FolderView *folderview, GtkCTreeNode *node)
 		gtk_ctree_node_set_text(ctree, node, COL_NEW,    "-");
 		gtk_ctree_node_set_text(ctree, node, COL_UNREAD, "-");
 		gtk_ctree_node_set_text(ctree, node, COL_TOTAL,  "-");
-		return;
 	} else {
 		gtk_ctree_node_set_text(ctree, node, COL_NEW,    itos(item->new));
 		gtk_ctree_node_set_text(ctree, node, COL_UNREAD, itos(item->unread));
 		gtk_ctree_node_set_text(ctree, node, COL_TOTAL,  itos(item->total));
 	}
 
-	if (item->stype == F_TRASH) return;
-
-	ctree_style = gtk_widget_get_style(GTK_WIDGET(ctree));
-	prev_style = gtk_ctree_node_get_row_style(ctree, node);
-	if (!prev_style)
-		prev_style = ctree_style;
-	style = gtk_style_copy(prev_style);
-
-	if (style) {
-		gboolean use_bold, use_color;
-
-		if (item->stype == F_QUEUE) {
-			/* highlight queue folder if there are any messages */
-			use_bold = use_color = (item->total > 0);
-		} else {
-			/* if unread messages exist, print with bold font */
-			use_bold = (item->unread > 0);
-			/* if new messages exist, print with colored letter */
-			use_color = (item->new > 0);
-		}
-
-		if (use_bold && boldfont)
-			style->font = boldfont;
-		else
-			style->font = ctree_style->font;
-
-		if (use_color) {
-			style->fg[GTK_STATE_NORMAL]   = folderview->color_new;
-			style->fg[GTK_STATE_SELECTED] = folderview->color_new;
-		} else {
-			style->fg[GTK_STATE_NORMAL] =
-				ctree_style->fg[GTK_STATE_NORMAL];
-			style->fg[GTK_STATE_SELECTED] =
-				ctree_style->fg[GTK_STATE_SELECTED];
-		}
-
-		gtk_ctree_node_set_row_style(ctree, node, style);
+	if (item->stype == F_TRASH) {
+		use_bold = use_color = FALSE;
+	} else if (item->stype == F_QUEUE) {
+		/* highlight queue folder if there are any messages */
+		use_bold = use_color = (item->total > 0);
+	} else {
+		/* if unread messages exist, print with bold font */
+		use_bold = (item->unread > 0) || add_unread_mark;
+		/* if new messages exist, print with colored letter */
+		use_color =
+			(item->new > 0) ||
+			(add_unread_mark &&
+			 folderview_have_new_children(folderview, node));
 	}
+
+	gtk_ctree_node_set_foreground(ctree, node, NULL);
+
+	if (use_bold && use_color)
+		style = bold_color_style;
+	else if (use_bold)
+		style = bold_style;
+	else if (use_color)
+		gtk_ctree_node_set_foreground(ctree, node,
+					      &folderview->color_new);
+
+	gtk_ctree_node_set_row_style(ctree, node, style);
+
+	if ((node = gtkut_ctree_find_collapsed_parent(ctree, node)) != NULL)
+		folderview_update_node(folderview, node);
 }
 
 void folderview_update_item(FolderItem *item, gboolean update_summary)
@@ -755,8 +957,18 @@ static gboolean folderview_gnode_func(GtkCTree *ctree, guint depth,
 static void folderview_expand_func(GtkCTree *ctree, GtkCTreeNode *node,
 				   gpointer data)
 {
-	if (GTK_CTREE_ROW(node)->children)
-		gtk_ctree_expand(ctree, node);
+	FolderView *folderview = (FolderView *)data;
+	FolderItem *item;
+
+	if (GTK_CTREE_ROW(node)->children) {
+		item = gtk_ctree_node_get_row_data(ctree, node);
+		g_return_if_fail(item != NULL);
+
+		if (!item->collapsed)
+			gtk_ctree_expand(ctree, node);
+		else
+			folderview_update_node(folderview, node);
+	}
 }
 
 #define SET_SPECIAL_FOLDER(ctree, item) \
@@ -945,15 +1157,17 @@ static void folderview_button_pressed(GtkWidget *ctree, GdkEventButton *event,
 	menu_set_insensitive_all(GTK_MENU_SHELL(folderview->imap_popup));
 	menu_set_insensitive_all(GTK_MENU_SHELL(folderview->news_popup));
 
-	if (folder->type == F_MH && item->parent == NULL) {
+	if (FOLDER_IS_LOCAL(folder) && item->parent == NULL) {
 		menu_set_sensitive(folderview->mail_factory,
 				   "/Create new folder...", TRUE);
+		menu_set_sensitive(folderview->mail_factory,
+				   "/Update folder tree", TRUE);
 		menu_set_sensitive(folderview->mail_factory,
 				   "/Remove mailbox", TRUE);
-	} else if (folder->type == F_MH && item->stype != F_NORMAL) {
+	} else if (FOLDER_IS_LOCAL(folder) && item->stype != F_NORMAL) {
 		menu_set_sensitive(folderview->mail_factory,
 				   "/Create new folder...", TRUE);
-	} else if (folder->type == F_MH) {
+	} else if (FOLDER_IS_LOCAL(folder)) {
 		menu_set_sensitive(folderview->mail_factory,
 				   "/Create new folder...", TRUE);
 		menu_set_sensitive(folderview->mail_factory,
@@ -964,8 +1178,13 @@ static void folderview_button_pressed(GtkWidget *ctree, GdkEventButton *event,
 		menu_set_sensitive(folderview->imap_factory,
 				   "/Create new folder...", TRUE);
 		menu_set_sensitive(folderview->imap_factory,
-				   "/Remove IMAP4 server", TRUE);
-	} else if (folder->type == F_IMAP && item->parent != NULL) {
+				   "/Update folder tree", TRUE);
+		menu_set_sensitive(folderview->imap_factory,
+				   "/Remove IMAP4 account", TRUE);
+	} else if (folder->type == F_IMAP && item->stype != F_NORMAL) {
+		menu_set_sensitive(folderview->imap_factory,
+				   "/Create new folder...", TRUE);
+	} else if (folder->type == F_IMAP) {
 		menu_set_sensitive(folderview->imap_factory,
 				   "/Create new folder...", TRUE);
 		menu_set_sensitive(folderview->imap_factory,
@@ -974,15 +1193,15 @@ static void folderview_button_pressed(GtkWidget *ctree, GdkEventButton *event,
 		menu_set_sensitive(folderview->news_factory,
 				   "/Subscribe to newsgroup...", TRUE);
 		menu_set_sensitive(folderview->news_factory,
-				   "/Remove news server", TRUE);
-	} else if (folder->type == F_NEWS && item->parent != NULL) {
+				   "/Remove news account", TRUE);
+	} else if (folder->type == F_NEWS) {
 		menu_set_sensitive(folderview->news_factory,
 				   "/Subscribe to newsgroup...", TRUE);
 		menu_set_sensitive(folderview->news_factory,
 				   "/Remove newsgroup", TRUE);
 	}
 
-	if (folder->type == F_MH)
+	if (FOLDER_IS_LOCAL(folder))
 		gtk_menu_popup(GTK_MENU(folderview->mail_popup), NULL, NULL,
 			       NULL, NULL, event->button, event->time);
 	else if (folder->type == F_IMAP)
@@ -998,7 +1217,8 @@ static void folderview_button_released(GtkWidget *ctree, GdkEventButton *event,
 {
 	if (!event) return;
 
-	if (event->button == 1 && folderview->opened != NULL) {
+	if (event->button == 1 && folderview->open_folder == FALSE &&
+	    folderview->opened != NULL) {
 		gtk_ctree_select(GTK_CTREE(ctree), folderview->opened);
 		gtkut_ctree_set_focus_row(GTK_CTREE(ctree),
 					  folderview->opened);
@@ -1040,6 +1260,7 @@ static void folderview_selected(GtkCTree *ctree, GtkCTreeNode *row,
 				gint column, FolderView *folderview)
 {
 	static gboolean can_select = TRUE;	/* exclusive lock */
+	gboolean opened;
 	FolderItem *item;
 
 	folderview->selected = row;
@@ -1056,7 +1277,6 @@ static void folderview_selected(GtkCTree *ctree, GtkCTreeNode *row,
 	}
 
 	if (!folderview->open_folder) return;
-	folderview->open_folder = FALSE;
 
 	item = gtk_ctree_node_get_row_data(ctree, row);
 	if (!item) return;
@@ -1072,13 +1292,47 @@ static void folderview_selected(GtkCTree *ctree, GtkCTreeNode *row,
 	    !GTK_CTREE_ROW(folderview->opened)->children)
 		gtk_ctree_collapse(ctree, folderview->opened);
 
-	folderview->opened = row;
+	/* ungrab the mouse event */
+	if (GTK_WIDGET_HAS_GRAB(ctree)) {
+		gtk_grab_remove(GTK_WIDGET(ctree));
+		if (gdk_pointer_is_grabbed())
+			gdk_pointer_ungrab(GDK_CURRENT_TIME);
+	}
 
-	summary_show(folderview->summaryview, item, FALSE);
+	opened = summary_show(folderview->summaryview, item, FALSE);
 
+	if (!opened) {
+		gtk_ctree_select(ctree, folderview->opened);
+		gtkut_ctree_set_focus_row(ctree, folderview->opened);
+	} else
+		folderview->opened = row;
+
+	folderview->open_folder = FALSE;
 	can_select = TRUE;
 }
 
+static void folderview_tree_expanded(GtkCTree *ctree, GtkCTreeNode *node,
+				     FolderView *folderview)
+{
+	FolderItem *item;
+
+	item = gtk_ctree_node_get_row_data(ctree, node);
+	g_return_if_fail(item != NULL);
+	item->collapsed = FALSE;
+	folderview_update_node(folderview, node);
+}
+
+static void folderview_tree_collapsed(GtkCTree *ctree, GtkCTreeNode *node,
+				      FolderView *folderview)
+{
+	FolderItem *item;
+
+	item = gtk_ctree_node_get_row_data(ctree, node);
+	g_return_if_fail(item != NULL);
+	item->collapsed= TRUE;
+	folderview_update_node(folderview, node);
+}
+
 static void folderview_popup_close(GtkMenuShell *menu_shell,
 				   FolderView *folderview)
 {
@@ -1132,6 +1386,23 @@ static GtkCTreeNode *folderview_find_by_name(GtkCTree *ctree,
 	return NULL;
 }
 
+static void folderview_update_tree_cb(FolderView *folderview, guint action,
+				      GtkWidget *widget)
+{
+	GtkCTree *ctree = GTK_CTREE(folderview->ctree);
+	FolderItem *item;
+
+	if (!folderview->selected) return;
+
+	summary_show(folderview->summaryview, NULL, FALSE);
+
+	item = gtk_ctree_node_get_row_data(ctree, folderview->selected);
+	g_return_if_fail(item != NULL);
+	g_return_if_fail(item->folder != NULL);
+
+	folderview_update_tree(item->folder);
+}
+
 static void folderview_new_folder_cb(FolderView *folderview, guint action,
 				     GtkWidget *widget)
 {
@@ -1169,8 +1440,13 @@ static void folderview_new_folder_cb(FolderView *folderview, guint action,
 	}
 
 	new_item = item->folder->create_folder(item->folder, item, new_folder);
+	if (!new_item) {
+		alertpanel_error(_("Can't create the folder `%s'."),
+				 new_folder);
+		g_free(new_folder);
+		return;
+	}
 	g_free(new_folder);
-	if (!new_item) return;
 
 	gtk_clist_freeze(GTK_CLIST(ctree));
 
@@ -1275,7 +1551,8 @@ static void folderview_delete_folder_cb(FolderView *folderview, guint action,
 	if (avalue != G_ALERTDEFAULT) return;
 
 	if (item->folder->remove_folder(item->folder, item) < 0) {
-		g_warning(_("can't remove folder `%s'\n"), item->path);
+		alertpanel_error(_("Can't remove the folder `%s'."),
+				 item->path);
 		return;
 	}
 
@@ -1332,6 +1609,7 @@ static void folderview_new_imap_folder_cb(FolderView *folderview, guint action,
 	FolderItem *item;
 	FolderItem *new_item;
 	gchar *new_folder;
+	gchar *p;
 
 	if (!folderview->selected) return;
 
@@ -1341,12 +1619,16 @@ static void folderview_new_imap_folder_cb(FolderView *folderview, guint action,
 	g_return_if_fail(item->folder->type == F_IMAP);
 	g_return_if_fail(item->folder->account != NULL);
 
-	new_folder = input_dialog(_("New folder"),
-				  _("Input the name of new folder:"),
-				  _("NewFolder"));
+	new_folder = input_dialog
+		(_("New folder"),
+		 _("Input the name of new folder:\n"
+		   "(if you want to create a folder to store subfolders,\n"
+		   " append `/' at the end of the name)"),
+		 _("NewFolder"));
 	if (!new_folder) return;
 
-	if (strchr(new_folder, G_DIR_SEPARATOR) != NULL) {
+	if ((p = strchr(new_folder, G_DIR_SEPARATOR)) != NULL &&
+	    *(p + 1) != '\0') {
 		alertpanel_error(_("`%c' can't be included in folder name."),
 				 G_DIR_SEPARATOR);
 		g_free(new_folder);
@@ -1362,8 +1644,13 @@ static void folderview_new_imap_folder_cb(FolderView *folderview, guint action,
 	}
 
 	new_item = item->folder->create_folder(item->folder, item, new_folder);
+	if (!new_item) {
+		alertpanel_error(_("Can't create the folder `%s'."),
+				 new_folder);
+		g_free(new_folder);
+		return;
+	}
 	g_free(new_folder);
-	if (!new_item) return;
 
 	gtk_clist_freeze(GTK_CLIST(ctree));
 
@@ -1406,7 +1693,12 @@ static void folderview_rm_imap_folder_cb(FolderView *folderview, guint action,
 	if (avalue != G_ALERTDEFAULT) return;
 
 	if (item->folder->remove_folder(item->folder, item) < 0) {
-		g_warning(_("can't remove folder `%s'\n"), item->path);
+		alertpanel_error(_("Can't remove the folder `%s'."),
+				 item->path);
+		if (folderview->opened == folderview->selected)
+			summary_show(folderview->summaryview,
+				     folderview->summaryview->folder_item,
+				     FALSE);
 		return;
 	}
 
@@ -1438,9 +1730,9 @@ static void folderview_rm_imap_server_cb(FolderView *folderview, guint action,
 	g_return_if_fail(item->folder->type == F_IMAP);
 	g_return_if_fail(item->folder->account != NULL);
 
-	message = g_strdup_printf(_("Really delete IMAP4 server `%s'?"),
+	message = g_strdup_printf(_("Really delete IMAP4 account `%s'?"),
 				  item->folder->name);
-	avalue = alertpanel(_("Delete IMAP4 server"), message,
+	avalue = alertpanel(_("Delete IMAP4 account"), message,
 			    _("Yes"), _("+No"), NULL);
 	g_free(message);
 
@@ -1481,8 +1773,7 @@ static void folderview_new_news_group_cb(FolderView *folderview, guint action,
 	g_return_if_fail(item->folder->type == F_NEWS);
 	g_return_if_fail(item->folder->account != NULL);
 
-	new_group = input_dialog(_("Subscribe newsgroup"),
-				 _("Input subscribing newsgroup:"), NULL);
+	new_group = grouplist_dialog(item->folder);
 	if (!new_group) return;
 
 	if (GTK_CTREE_ROW(folderview->selected)->parent != NULL)
@@ -1570,9 +1861,9 @@ static void folderview_rm_news_server_cb(FolderView *folderview, guint action,
 	g_return_if_fail(item->folder->type == F_NEWS);
 	g_return_if_fail(item->folder->account != NULL);
 
-	message = g_strdup_printf(_("Really delete news server `%s'?"),
+	message = g_strdup_printf(_("Really delete news account `%s'?"),
 				  item->folder->name);
-	avalue = alertpanel(_("Delete news server"), message,
+	avalue = alertpanel(_("Delete news account"), message,
 			    _("Yes"), _("+No"), NULL);
 	g_free(message);
 
@@ -1602,30 +1893,42 @@ static gboolean folderview_drag_motion_cb(GtkWidget      *widget,
 					  FolderView     *folderview)
 {
 	gint row, column;
-	FolderItem *item;
-	GtkCTreeNode *node;
+	FolderItem *item, *current_item;
+	GtkCTreeNode *node = NULL;
+	gboolean acceptable = FALSE;
 
 	if (gtk_clist_get_selection_info(GTK_CLIST(widget),
 					 x - 24, y - 24, &row, &column)) {
 		node = gtk_ctree_node_nth(GTK_CTREE(widget), row);
 		item = gtk_ctree_node_get_row_data(GTK_CTREE(widget), node);
+		current_item = folderview->summaryview->folder_item;
 		if (item != NULL &&
 		    item->path != NULL &&
-		    folderview->summaryview->folder_item != item) {
-			if (item->folder->type != F_NEWS) {
-				gtk_ctree_select(GTK_CTREE(widget), node);
-				gdk_drag_status(context,
-						context->suggested_action,
-						time);
-				return TRUE;
+		    current_item != NULL &&
+		    current_item != item) {
+			switch (item->folder->type){
+			case F_MH:
+				if (current_item->folder->type == F_MH)
+				    acceptable = TRUE;
+				break;
+			case F_IMAP:
+				if (current_item->folder->account == item->folder->account)
+				    acceptable = TRUE;
+				break;
+			default:
 			}
 		}
 	}
 
-	gtk_ctree_select(GTK_CTREE(widget), folderview->opened);
-	gdk_drag_status(context, 0, time);
+	if (acceptable) {
+		gtk_ctree_select(GTK_CTREE(widget), node);
+		gdk_drag_status(context, context->suggested_action, time);
+	} else {
+		gtk_ctree_select(GTK_CTREE(widget), folderview->opened);
+		gdk_drag_status(context, 0, time);
+	}
 
-	return FALSE;
+	return acceptable;
 }
 
 static void folderview_drag_leave_cb(GtkWidget      *widget,
