@@ -168,6 +168,47 @@ gint gtkut_ctree_get_nth_from_node(GtkCTree *ctree, GtkCTreeNode *node)
 	return g_list_position(GTK_CLIST(ctree)->row_list, (GList *)node);
 }
 
+/* get the next node, including the invisible one */
+GtkCTreeNode *gtkut_ctree_node_next(GtkCTree *ctree, GtkCTreeNode *node)
+{
+	GtkCTreeNode *parent;
+
+	if (!node) return NULL;
+
+	if (GTK_CTREE_ROW(node)->children)
+		return GTK_CTREE_ROW(node)->children;
+
+	if (GTK_CTREE_ROW(node)->sibling)
+		return GTK_CTREE_ROW(node)->sibling;
+
+	for (parent = GTK_CTREE_ROW(node)->parent; parent != NULL;
+	     parent = GTK_CTREE_ROW(parent)->parent) {
+		if (GTK_CTREE_ROW(parent)->sibling)
+			return GTK_CTREE_ROW(parent)->sibling;
+	}
+
+	return NULL;
+}
+
+GtkCTreeNode *gtkut_ctree_find_collapsed_parent(GtkCTree *ctree,
+						GtkCTreeNode *node)
+{
+	if (!node) return NULL;
+
+	while ((node = GTK_CTREE_ROW(node)->parent) != NULL) {
+		if (!GTK_CTREE_ROW(node)->expanded)
+			return node;
+	}
+
+	return NULL;
+}
+
+void gtkut_ctree_expand_parent_all(GtkCTree *ctree, GtkCTreeNode *node)
+{
+	while ((node = gtkut_ctree_find_collapsed_parent(ctree, node)) != NULL)
+		gtk_ctree_expand(ctree, node);
+}
+
 void gtkut_ctree_set_focus_row(GtkCTree *ctree, GtkCTreeNode *node)
 {
 	gtkut_clist_set_focus_row(GTK_CLIST(ctree),
