@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999,2000 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2001 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -37,6 +37,7 @@
 #include "procheader.h"
 #include "procmsg.h"
 #include "codeconv.h"
+#include "prefs_common.h"
 #include "utils.h"
 
 static GdkFont *normalfont;
@@ -88,9 +89,9 @@ HeaderWindow *header_window_create(void)
 void header_window_init(HeaderWindow *headerwin)
 {
 	//if (!normalfont)
-	//	normalfont = gdk_fontset_load(NORMAL_FONT);
+	//	normalfont = gdk_fontset_load(prefs_common.normalfont);
 	if (!boldfont)
-		boldfont = gdk_fontset_load(BOLD_FONT);
+		boldfont = gdk_fontset_load(prefs_common.boldfont);
 }
 
 void header_window_show(HeaderWindow *headerwin, MsgInfo *msginfo)
@@ -155,10 +156,10 @@ void header_window_show(HeaderWindow *headerwin, MsgInfo *msginfo)
 void header_window_show_cb(gpointer data, guint action, GtkWidget *widget)
 {
 	MainWindow *mainwin = data;
-	GtkCTreeNode *node = mainwin->summaryview->displayed;
+	GtkCTreeNode *node = mainwin->summaryview->selected;
 	MsgInfo *msginfo;
 
-	if (node && !GTK_WIDGET_VISIBLE(mainwin->headerwin->window)) {
+	if (node) {
 		msginfo = gtk_ctree_node_get_row_data
 			(GTK_CTREE(mainwin->summaryview->ctree), node);
 		header_window_show(mainwin->headerwin, msginfo);
