@@ -78,6 +78,8 @@ void imageview_show_image(ImageView *imageview, MimeInfo *mimeinfo,
 
 	if (!pixbuf) {
 		g_warning(_("Can't load the image."));	
+		if (imageview->image)
+			gtk_widget_hide(imageview->image);
 		return;
 	}
 
@@ -92,11 +94,11 @@ void imageview_show_image(ImageView *imageview, MimeInfo *mimeinfo,
 		gtk_scrolled_window_add_with_viewport
 			(GTK_SCROLLED_WINDOW(imageview->scrolledwin),
 			 imageview->image);
-
-		gtk_widget_show(imageview->image);
 	} else
 		gtk_pixmap_set(GTK_PIXMAP(imageview->image), pixmap, mask);
 
+	gtk_widget_show(imageview->image);
+
 	gdk_pixbuf_unref(pixbuf);
 
 	if (imageview->messageview->mainwin)
@@ -113,6 +115,8 @@ void imageview_show_image(ImageView *imageview, MimeInfo *mimeinfo,
 
 	if (!im) {
 		g_warning(_("Can't load the image."));	
+		if (imageview->image)
+			gtk_widget_hide(imageview->image);
 		return;
 	}
 
@@ -128,13 +132,13 @@ void imageview_show_image(ImageView *imageview, MimeInfo *mimeinfo,
 		gtk_scrolled_window_add_with_viewport
 			(GTK_SCROLLED_WINDOW(imageview->scrolledwin),
 			 imageview->image);
-
-		gtk_widget_show(imageview->image);
 	} else
 		gtk_pixmap_set(GTK_PIXMAP(imageview->image),
 			       gdk_imlib_move_image(im),
 			       gdk_imlib_move_mask(im));      
 
+	gtk_widget_show(imageview->image);
+
 	gdk_imlib_destroy_image(im);
 
 	if (imageview->messageview->mainwin)
