@@ -227,7 +227,7 @@ static void import_destsel_cb(GtkWidget *widget, gpointer data)
 {
 	FolderItem *dest;
 
-	dest = foldersel_folder_sel(NULL);
+	dest = foldersel_folder_sel(NULL, NULL);
 	if (dest && dest->path)
 		gtk_entry_set_text(GTK_ENTRY(dest_entry), dest->path);
 }
