@@ -27,6 +27,7 @@
 #include <gtk/gtkmain.h>
 #include <gtk/gtkwindow.h>
 #include <gtk/gtksignal.h>
+#include <gtk/gtkprogressbar.h>
 #include <stdio.h>
 #include <unistd.h>
 #include <string.h>
@@ -47,6 +48,7 @@
 #include "account.h"
 #include "procmsg.h"
 #include "socket.h"
+#include "ssl.h"
 #include "pop.h"
 #include "recv.h"
 #include "mbox.h"
@@ -65,17 +67,20 @@
 #include "pixmaps/complete.xpm"
 #include "pixmaps/error.xpm"
 
-GdkPixmap *currentxpm;
-GdkBitmap *currentxpmmask;
-GdkPixmap *errorxpm;
-GdkBitmap *errorxpmmask;
-GdkPixmap *okxpm;
-GdkBitmap *okxpmmask;
+static guint inc_lock_count = 0;
+
+static GdkPixmap *currentxpm;
+static GdkBitmap *currentxpmmask;
+static GdkPixmap *errorxpm;
+static GdkBitmap *errorxpmmask;
+static GdkPixmap *okxpm;
+static GdkBitmap *okxpmmask;
 
 #define MSGBUFSIZE	8192
 
-static void inc_finished		(MainWindow		*mainwin);
-static void inc_account_mail		(PrefsAccount		*account,
+static void inc_finished		(MainWindow		*mainwin,
+					 gboolean		 new_messages);
+static gint inc_account_mail		(PrefsAccount		*account,
 					 MainWindow		*mainwin);
 
 static IncProgressDialog *inc_progress_dialog_create	(void);
@@ -85,7 +90,7 @@ static IncSession *inc_session_new	(PrefsAccount		*account);
 static void inc_session_destroy		(IncSession		*session);
 static Pop3State *inc_pop3_state_new	(PrefsAccount		*account);
 static void inc_pop3_state_destroy	(Pop3State		*state);
-static void inc_start			(IncProgressDialog	*inc_dialog);
+static gint inc_start			(IncProgressDialog	*inc_dialog);
 static IncState inc_pop3_session_do	(IncSession		*session);
 static gint pop3_automaton_terminate	(SockInfo		*source,
 					 Automaton		*atm);
@@ -98,6 +103,7 @@ static gint connection_check_cb		(Automaton	*atm);
 #endif
 
 static void inc_pop3_recv_func		(SockInfo	*sock,
+					 gint		 count,
 					 gint		 read_bytes,
 					 gpointer	 data);
 
@@ -110,12 +116,27 @@ static gint inc_spool			(void);
 static gint get_spool			(FolderItem	*dest,
 					 const gchar	*mbox);
 
-static gint inc_autocheck_func		(gpointer	 data);
+static void inc_autocheck_timer_set_interval	(guint		 interval);
+static gint inc_autocheck_func			(gpointer	 data);
 
-static void inc_finished(MainWindow *mainwin)
+/**
+ * inc_finished:
+ * @mainwin: Main window.
+ * @new_messages: TRUE if some messages have been received.
+ * 
+ * Update the folder view and the summary view after receiving
+ * messages.  If @new_messages is FALSE, this function avoids unneeded
+ * updating.
+ **/
+static void inc_finished(MainWindow *mainwin, gboolean new_messages)
 {
 	FolderItem *item;
 
+	if (prefs_common.scan_all_after_inc)
+		folderview_update_all_node();
+
+	if (!new_messages && !prefs_common.scan_all_after_inc) return;
+
 	if (prefs_common.open_inbox_on_inc) {
 		item = cur_account && cur_account->inbox
 			? folder_find_item_from_path(cur_account->inbox)
@@ -131,51 +152,44 @@ static void inc_finished(MainWindow *mainwin)
 
 void inc_mail(MainWindow *mainwin)
 {
+	gint new_msgs = 0;
+
+	if (inc_lock_count) return;
+
 	inc_autocheck_timer_remove();
 	summary_write_cache(mainwin->summaryview);
+	main_window_lock(mainwin);
 
-	if (prefs_common.use_extinc && prefs_common.extinc_path) {
-		gint pid;
-
+	if (prefs_common.use_extinc && prefs_common.extinc_cmd) {
 		/* external incorporating program */
-		if ((pid = fork()) < 0) {
-			perror("fork");
+		if (execute_command_line(prefs_common.extinc_cmd, FALSE) < 0) {
+			main_window_unlock(mainwin);
 			inc_autocheck_timer_set();
 			return;
 		}
 
-		if (pid == 0) {
-			execlp(prefs_common.extinc_path,
-			       g_basename(prefs_common.extinc_path),
-			       NULL);
-
-			/* this will be called when failed */
-			perror("exec");
-			_exit(1);
-		}
-
-		/* wait until child process is terminated */
-		waitpid(pid, NULL, 0);
-
-		if (prefs_common.inc_local) inc_spool();
+		if (prefs_common.inc_local)
+			new_msgs = inc_spool();
 	} else {
-		if (prefs_common.inc_local) inc_spool();
+		if (prefs_common.inc_local)
+			new_msgs = inc_spool();
 
-		inc_account_mail(cur_account, mainwin);
+		new_msgs += inc_account_mail(cur_account, mainwin);
 	}
 
-	inc_finished(mainwin);
+	inc_finished(mainwin, new_msgs > 0);
+	main_window_unlock(mainwin);
 	inc_autocheck_timer_set();
 }
 
-static void inc_account_mail(PrefsAccount *account, MainWindow *mainwin)
+static gint inc_account_mail(PrefsAccount *account, MainWindow *mainwin)
 {
 	IncProgressDialog *inc_dialog;
 	IncSession *session;
 	gchar *text[3];
 
 	session = inc_session_new(account);
-	if (!session) return;
+	if (!session) return 0;
 
 	inc_dialog = inc_progress_dialog_create();
 	inc_dialog->queue_list = g_list_append(inc_dialog->queue_list, session);
@@ -187,21 +201,28 @@ static void inc_account_mail(PrefsAccount *account, MainWindow *mainwin)
 	text[2] = _("Standby");
 	gtk_clist_append(GTK_CLIST(inc_dialog->dialog->clist), text);
 
-	inc_start(inc_dialog);
+	return inc_start(inc_dialog);
 }
 
 void inc_all_account_mail(MainWindow *mainwin)
 {
 	GList *list, *queue_list = NULL;
 	IncProgressDialog *inc_dialog;
+	gint new_msgs = 0;
+
+	if (inc_lock_count) return;
 
 	inc_autocheck_timer_remove();
 	summary_write_cache(mainwin->summaryview);
+	main_window_lock(mainwin);
 
-	if (prefs_common.inc_local) inc_spool();
+	if (prefs_common.inc_local)
+		new_msgs = inc_spool();
 
 	list = account_get_list();
 	if (!list) {
+		inc_finished(mainwin, new_msgs > 0);
+		main_window_unlock(mainwin);
 		inc_autocheck_timer_set();
 		return;
 	}
@@ -218,6 +239,8 @@ void inc_all_account_mail(MainWindow *mainwin)
 	}
 
 	if (!queue_list) {
+		inc_finished(mainwin, new_msgs > 0);
+		main_window_unlock(mainwin);
 		inc_autocheck_timer_set();
 		return;
 	}
@@ -237,9 +260,10 @@ void inc_all_account_mail(MainWindow *mainwin)
 		gtk_clist_append(GTK_CLIST(inc_dialog->dialog->clist), text);
 	}
 
-	inc_start(inc_dialog);
+	new_msgs += inc_start(inc_dialog);
 
-	inc_finished(mainwin);
+	inc_finished(mainwin, new_msgs > 0);
+	main_window_unlock(mainwin);
 	inc_autocheck_timer_set();
 }
 
@@ -257,18 +281,21 @@ static IncProgressDialog *inc_progress_dialog_create(void)
 			   GTK_SIGNAL_FUNC(inc_cancel), dialog);
 	gtk_signal_connect(GTK_OBJECT(progress->window), "delete_event",
 			   GTK_SIGNAL_FUNC(gtk_true), NULL);
-	manage_window_set_transient(GTK_WINDOW(progress->window));
+	/* manage_window_set_transient(GTK_WINDOW(progress->window)); */
 
 	progress_dialog_set_value(progress, 0.0);
 
-	gtk_widget_show(progress->window);
-
 	PIXMAP_CREATE(progress->clist, okxpm, okxpmmask, complete_xpm);
 	PIXMAP_CREATE(progress->clist,
 		      currentxpm, currentxpmmask, continue_xpm);
 	PIXMAP_CREATE(progress->clist, errorxpm, errorxpmmask, error_xpm);
 
-	gtk_widget_show_now(progress->window);
+	if (prefs_common.recv_dialog_mode == RECV_DIALOG_ALWAYS ||
+	    (prefs_common.recv_dialog_mode == RECV_DIALOG_ACTIVE &&
+	     manage_window_get_focus_window())) {
+		dialog->show_dialog = TRUE;
+		gtk_widget_show_now(progress->window);
+	}
 
 	dialog->dialog = progress;
 	dialog->queue_list = NULL;
@@ -280,12 +307,16 @@ static void inc_progress_dialog_clear(IncProgressDialog *inc_dialog)
 {
 	progress_dialog_set_value(inc_dialog->dialog, 0.0);
 	progress_dialog_set_label(inc_dialog->dialog, "");
+	gtk_progress_bar_update
+		(GTK_PROGRESS_BAR(inc_dialog->mainwin->progressbar), 0.0);
 }
 
 static void inc_progress_dialog_destroy(IncProgressDialog *inc_dialog)
 {
 	g_return_if_fail(inc_dialog != NULL);
 
+	gtk_progress_bar_update
+		(GTK_PROGRESS_BAR(inc_dialog->mainwin->progressbar), 0.0);
 	progress_dialog_destroy(inc_dialog->dialog);
 
 	g_free(inc_dialog);
@@ -355,13 +386,18 @@ static void inc_pop3_state_destroy(Pop3State *state)
 	g_free(state);
 }
 
-static void inc_start(IncProgressDialog *inc_dialog)
+static gint inc_start(IncProgressDialog *inc_dialog)
 {
 	IncSession *session;
 	GtkCList *clist = GTK_CLIST(inc_dialog->dialog->clist);
 	Pop3State *pop3_state;
 	IncState inc_state;
 	gint num = 0;
+	gint error_num = 0;
+	gint new_msgs = 0;
+
+	//gtk_label_set_text(GTK_LABEL(inc_dialog->mainwin->statuslabel),
+	//		   _("Receiving"));
 
 	while (inc_dialog->queue_list != NULL) {
 		session = inc_dialog->queue_list->data;
@@ -369,6 +405,8 @@ static void inc_start(IncProgressDialog *inc_dialog)
 
 		inc_progress_dialog_clear(inc_dialog);
 
+		gtk_clist_moveto(clist, num, -1, 1.0, 0.0);
+
 		pop3_state->user = g_strdup(pop3_state->ac_prefs->userid);
 		if (pop3_state->ac_prefs->passwd)
 			pop3_state->pass =
@@ -388,8 +426,10 @@ static void inc_start(IncProgressDialog *inc_dialog)
 			pass = input_dialog_with_invisible(_("Input password"),
 							   message, NULL);
 			g_free(message);
-			manage_window_focus_in(inc_dialog->mainwin->window,
-					       NULL, NULL);
+			if (inc_dialog->show_dialog)
+				manage_window_focus_in
+					(inc_dialog->mainwin->window,
+					 NULL, NULL);
 			if (pass) {
 				pop3_state->ac_prefs->tmp_pass = g_strdup(pass);
 				pop3_state->pass = pass;
@@ -407,27 +447,33 @@ static void inc_start(IncProgressDialog *inc_dialog)
 		/* begin POP3 session */
 		inc_state = inc_pop3_session_do(session);
 
-		if (inc_state == INC_SUCCESS || inc_state == INC_CANCEL) {
+		if (inc_state == INC_SUCCESS) {
 			gtk_clist_set_pixmap(clist, num, 0, okxpm, okxpmmask);
 			gtk_clist_set_text(clist, num, 2, _("Done"));
+		} else if (inc_state == INC_CANCEL) {
+			gtk_clist_set_pixmap(clist, num, 0, okxpm, okxpmmask);
+			gtk_clist_set_text(clist, num, 2, _("Cancelled"));
 		} else {
 			gtk_clist_set_pixmap(clist, num, 0, errorxpm, errorxpmmask);
-			gtk_clist_set_text(clist, num, 2, _("Error"));
-		}
-
-		if (pop3_state->error_val == PS_AUTHFAIL) {
-			manage_window_focus_in(inc_dialog->dialog->window, NULL, NULL);
-			alertpanel_error
-				(_("Authorization for %s on %s failed"),
-				 pop3_state->user,
-				 pop3_state->ac_prefs->recv_server);
+			if (inc_state == INC_CONNECT_ERROR)
+				gtk_clist_set_text(clist, num, 2,
+						   _("Connection failed"));
+			else if (inc_state == INC_AUTH_FAILED)
+				gtk_clist_set_text(clist, num, 2,
+						   _("Auth failed"));
+			else
+				gtk_clist_set_text(clist, num, 2, _("Error"));
 		}
 
 		statusbar_pop_all();
-		manage_window_focus_in(inc_dialog->mainwin->window, NULL, NULL);
 
-		folder_item_scan_foreach(pop3_state->folder_table);
-		folderview_update_item_foreach(pop3_state->folder_table);
+		new_msgs += pop3_state->cur_total_num;
+
+		if (!prefs_common.scan_all_after_inc) {
+			folder_item_scan_foreach(pop3_state->folder_table);
+			folderview_update_item_foreach
+				(pop3_state->folder_table);
+		}
 
 		if (pop3_state->error_val == PS_AUTHFAIL &&
 		    pop3_state->ac_prefs->tmp_pass) {
@@ -437,9 +483,12 @@ static void inc_start(IncProgressDialog *inc_dialog)
 
 		inc_write_uidl_list(pop3_state);
 
-		if (inc_state != INC_SUCCESS) {
-			inc_put_error(inc_state);
-			break;
+		if (inc_state != INC_SUCCESS && inc_state != INC_CANCEL) {
+			error_num++;
+			if (inc_state == INC_NOSPACE) {
+				inc_put_error(inc_state);
+				break;
+			}
 		}
 
 		inc_session_destroy(session);
@@ -449,6 +498,18 @@ static void inc_start(IncProgressDialog *inc_dialog)
 		num++;
 	}
 
+	//gtk_label_set_text(GTK_LABEL(inc_dialog->mainwin->statuslabel), "");
+
+	if (error_num) {
+		if (inc_dialog->show_dialog)
+			manage_window_focus_in(inc_dialog->dialog->window,
+					       NULL, NULL);
+		alertpanel_error(_("Some errors occured while getting mail."));
+		if (inc_dialog->show_dialog)
+			manage_window_focus_out(inc_dialog->dialog->window,
+						NULL, NULL);
+	}
+
 	while (inc_dialog->queue_list != NULL) {
 		session = inc_dialog->queue_list->data;
 		inc_session_destroy(session);
@@ -457,8 +518,9 @@ static void inc_start(IncProgressDialog *inc_dialog)
 	}
 
 	inc_progress_dialog_destroy(inc_dialog);
-}
 
+	return new_msgs;
+}
 
 static IncState inc_pop3_session_do(IncSession *session)
 {
@@ -510,14 +572,21 @@ static IncState inc_pop3_session_do(IncSession *session)
 	atm->num = POP3_GREETING_RECV;
 
 	server = pop3_state->ac_prefs->recv_server;
+#if USE_SSL
+	port = pop3_state->ac_prefs->set_popport ?
+		pop3_state->ac_prefs->popport :
+		pop3_state->ac_prefs->ssl_pop ? 995 : 110;
+#else
 	port = pop3_state->ac_prefs->set_popport ?
 		pop3_state->ac_prefs->popport : 110;
+#endif
 
 	buf = g_strdup_printf(_("Connecting to POP3 server: %s ..."), server);
 	log_message("%s\n", buf);
 	progress_dialog_set_label(inc_dialog->dialog, buf);
 	g_free(buf);
 	GTK_EVENTS_FLUSH();
+	statusbar_pop_all();
 
 #if USE_THREADS
 	if ((sockinfo = sock_connect_with_thread(server, port)) == NULL) {
@@ -526,21 +595,25 @@ static IncState inc_pop3_session_do(IncSession *session)
 #endif
 		log_warning(_("Can't connect to POP3 server: %s:%d\n"),
 			    server, port);
-		manage_window_focus_in(inc_dialog->dialog->window, NULL, NULL);
-		alertpanel_error(_("Can't connect to POP3 server: %s:%d"),
-				 server, port);
-		manage_window_focus_out(inc_dialog->dialog->window, NULL, NULL);
 		pop3_automaton_terminate(NULL, atm);
 		automaton_destroy(atm);
-
-		return INC_ERROR;
+		return INC_CONNECT_ERROR;
 	}
 
+#if USE_SSL
+	if (pop3_state->ac_prefs->ssl_pop && !ssl_init_socket(sockinfo)) {
+		pop3_automaton_terminate(NULL, atm);
+		automaton_destroy(atm);
+		return INC_CONNECT_ERROR;
+	}
+#endif
+
 	/* :WK: Hmmm, with the later sock_gdk_input, we have 2 references
 	 * to the sock structure - implement a reference counter?? */
 	pop3_state->sockinfo = sockinfo;
 	atm->help_sock = sockinfo;
 
+	log_verbosity_set(TRUE);
 	recv_set_ui_func(inc_pop3_recv_func, session);
 
 #if USE_THREADS
@@ -554,6 +627,7 @@ static IncState inc_pop3_session_do(IncSession *session)
 
 	gtk_main();
 
+	log_verbosity_set(FALSE);
 	recv_set_ui_func(NULL, NULL);
 
 #if USE_THREADS
@@ -657,19 +731,15 @@ static gint connection_check_cb(Automaton *atm)
 	if (sockinfo->state == CONN_LOOKUPFAILED ||
 	    sockinfo->state == CONN_FAILED) {
 		atm->timeout_tag = 0;
-		pop3_automaton_terminate(sockinfo->sock, atm);
 		log_warning(_("Can't connect to POP3 server: %s:%d\n"),
 			    sockinfo->hostname, sockinfo->port);
-		manage_window_focus_in(inc_dialog->dialog->window, NULL, NULL);
-		alertpanel_error(_("Can't connect to POP3 server: %s:%d"),
-				 sockinfo->hostname, sockinfo->port);
-		manage_window_focus_out(inc_dialog->dialog->window, NULL, NULL);
+		pop3_automaton_terminate(sockinfo, atm);
 		return FALSE;
 	} else if (sockinfo->state == CONN_ESTABLISHED) {
 		atm->timeout_tag = 0;
-		atm->tag = gdk_input_add(sockinfo->sock,
-					 atm->state[atm->num].condition,
-					 automaton_input_cb, atm);
+		atm->tag = sock_gdk_input_add(sockinfo,
+					      atm->state[atm->num].condition,
+					      automaton_input_cb, atm);
 		return FALSE;
 	} else {
 		return TRUE;
@@ -677,7 +747,8 @@ static gint connection_check_cb(Automaton *atm)
 }
 #endif
 
-static void inc_pop3_recv_func(SockInfo *sock, gint read_bytes, gpointer data)
+static void inc_pop3_recv_func(SockInfo *sock, gint count, gint read_bytes,
+			       gpointer data)
 {
 	gchar buf[MSGBUFSIZE];
 	IncSession *session = (IncSession *)data;
@@ -685,19 +756,24 @@ static void inc_pop3_recv_func(SockInfo *sock, gint read_bytes, gpointer data)
 	IncProgressDialog *inc_dialog = session->data;
 	ProgressDialog *dialog = inc_dialog->dialog;
 	gint cur_total;
+	gchar *total_size;
 
 	cur_total = state->cur_total_bytes + read_bytes;
 	if (cur_total > state->total_bytes)
 		cur_total = state->total_bytes;
 
+	Xstrdup_a(total_size, to_human_readable(state->total_bytes), return);
 	g_snprintf(buf, sizeof(buf),
-		   _("Retrieving message (%d / %d) (%d / %d bytes)"),
+		   _("Retrieving message (%d / %d) (%s / %s)"),
 		   state->cur_msg, state->count,
-		   cur_total, state->total_bytes);
+		   to_human_readable(cur_total), total_size);
 	progress_dialog_set_label(dialog, buf);
 
 	progress_dialog_set_percentage
 		(dialog, (gfloat)cur_total / (gfloat)state->total_bytes);
+	gtk_progress_bar_update
+		(GTK_PROGRESS_BAR(inc_dialog->mainwin->progressbar),
+		 (gfloat)cur_total / (gfloat)state->total_bytes);
 	GTK_EVENTS_FLUSH();
 }
 
@@ -706,6 +782,7 @@ void inc_progress_update(Pop3State *state, Pop3Phase phase)
 	gchar buf[MSGBUFSIZE];
 	IncProgressDialog *inc_dialog = state->session->data;
 	ProgressDialog *dialog = inc_dialog->dialog;
+	gchar *total_size;
 
 	switch (phase) {
 	case POP3_GREETING_RECV:
@@ -716,7 +793,7 @@ void inc_progress_update(Pop3State *state, Pop3Phase phase)
 	case POP3_GETAUTH_PASS_RECV:
 	case POP3_GETAUTH_APOP_SEND:
 	case POP3_GETAUTH_APOP_RECV:
-		progress_dialog_set_label(dialog, _("Authorizing..."));
+		progress_dialog_set_label(dialog, _("Authenticating..."));
 		break;
 	case POP3_GETRANGE_STAT_SEND:
 	case POP3_GETRANGE_STAT_RECV:
@@ -740,15 +817,21 @@ void inc_progress_update(Pop3State *state, Pop3Phase phase)
 		break;
 	case POP3_RETR_SEND:
 	case POP3_RETR_RECV:
+		Xstrdup_a(total_size, to_human_readable(state->total_bytes), return);
 		g_snprintf(buf, sizeof(buf),
-			   _("Retrieving message (%d / %d) (%d / %d bytes)"),
+			   _("Retrieving message (%d / %d) (%s / %s)"),
 			   state->cur_msg, state->count,
-			   state->cur_total_bytes, state->total_bytes);
+			   to_human_readable(state->cur_total_bytes),
+			   total_size);
 		progress_dialog_set_label(dialog, buf);
 		progress_dialog_set_percentage
 			(dialog,
 			 (gfloat)(state->cur_total_bytes) /
 			 (gfloat)(state->total_bytes));
+		gtk_progress_bar_update
+			(GTK_PROGRESS_BAR(inc_dialog->mainwin->progressbar),
+			 (gfloat)(state->cur_total_bytes) /
+			 (gfloat)(state->total_bytes));
 		break;
 	case POP3_DELETE_SEND:
 	case POP3_DELETE_RECV:
@@ -798,12 +881,11 @@ gint inc_drop_message(const gchar *file, Pop3State *state)
 				    GINT_TO_POINTER(1));
 	}
 
-	if (folder_item_add_msg(dropfolder, file) < 0) {
+	if (folder_item_add_msg(dropfolder, file, TRUE) < 0) {
 		unlink(file);
 		return -1;
 	}
 
-	unlink(file);
 	return 0;
 }
 
@@ -891,10 +973,12 @@ static gint get_spool(FolderItem *dest, const gchar *mbox)
 	unlock_mbox(mbox, lockfd, LOCK_FLOCK);
 
 	if (folder_table) {
-		folder_item_scan_foreach(folder_table);
-		folderview_update_item_foreach(folder_table);
+		if (!prefs_common.scan_all_after_inc) {
+			folder_item_scan_foreach(folder_table);
+			folderview_update_item_foreach(folder_table);
+		}
 		g_hash_table_destroy(folder_table);
-	} else {
+	} else if (!prefs_common.scan_all_after_inc) {
 		folder_item_scan(dest);
 		folderview_update_item(dest, FALSE);
 	}
@@ -902,6 +986,17 @@ static gint get_spool(FolderItem *dest, const gchar *mbox)
 	return msgs;
 }
 
+void inc_lock(void)
+{
+	inc_lock_count++;
+}
+
+void inc_unlock(void)
+{
+	if (inc_lock_count > 0)
+		inc_lock_count--;
+}
+
 static guint autocheck_timer = 0;
 static gpointer autocheck_data = NULL;
 
@@ -911,21 +1006,26 @@ void inc_autocheck_timer_init(MainWindow *mainwin)
 	inc_autocheck_timer_set();
 }
 
-void inc_autocheck_timer_set(void)
+static void inc_autocheck_timer_set_interval(guint interval)
 {
 	inc_autocheck_timer_remove();
 
 	if (prefs_common.autochk_newmail && autocheck_data) {
 		autocheck_timer = gtk_timeout_add
-			(prefs_common.autochk_itv * 60000,
-			 inc_autocheck_func,
-			 autocheck_data);
+			(interval, inc_autocheck_func, autocheck_data);
+		debug_print("added timer = %d\n", autocheck_timer);
 	}
 }
 
+void inc_autocheck_timer_set(void)
+{
+	inc_autocheck_timer_set_interval(prefs_common.autochk_itv * 60000);
+}
+
 void inc_autocheck_timer_remove(void)
 {
 	if (autocheck_timer) {
+		debug_print("removed timer = %d\n", autocheck_timer);
 		gtk_timeout_remove(autocheck_timer);
 		autocheck_timer = 0;
 	}
@@ -935,6 +1035,12 @@ static gint inc_autocheck_func(gpointer data)
 {
 	MainWindow *mainwin = (MainWindow *)data;
 
+	if (inc_lock_count) {
+		debug_print("autocheck is locked.\n");
+		inc_autocheck_timer_set_interval(1000);
+		return FALSE;
+	}
+
 	inc_all_account_mail(mainwin);
 
 	return FALSE;
