@@ -27,6 +27,7 @@
 #include <gtk/gtkvbox.h>
 #include <gtk/gtkcontainer.h>
 #include <gtk/gtkstatusbar.h>
+#include <gtk/gtkprogressbar.h>
 #include <gtk/gtkhpaned.h>
 #include <gtk/gtkvpaned.h>
 #include <gtk/gtkcheckmenuitem.h>
@@ -71,6 +72,7 @@
 #include "codeconv.h"
 #include "about.h"
 #include "manual.h"
+#include "version.h"
 
 #define AC_LABEL_WIDTH	240
 
@@ -135,6 +137,8 @@ static void toolbar_account_button_pressed	(GtkWidget	*widget,
 static void ac_label_button_pressed		(GtkWidget	*widget,
 						 GdkEventButton	*event,
 						 gpointer	 data);
+static void ac_menu_popup_closed		(GtkMenuShell	*menu_shell,
+						 gpointer	 data);
 
 static gint main_window_close_cb (GtkWidget	*widget,
 				  GdkEventAny	*event,
@@ -251,6 +255,12 @@ static void mark_as_unread_cb		(MainWindow	*mainwin,
 static void mark_as_read_cb		(MainWindow	*mainwin,
 					 guint		 action,
 					 GtkWidget	*widget);
+static void mark_all_read_cb		(MainWindow	*mainwin,
+					 guint		 action,
+					 GtkWidget	*widget);
+static void add_address_cb		(MainWindow	*mainwin,
+					 guint		 action,
+					 GtkWidget	*widget);
 
 static void set_charset_cb		(MainWindow	*mainwin,
 					 guint		 action,
@@ -289,9 +299,24 @@ static void next_cb		 (MainWindow	*mainwin,
 				  guint		 action,
 				  GtkWidget	*widget);
 
+static void prev_unread_cb	 (MainWindow	*mainwin,
+				  guint		 action,
+				  GtkWidget	*widget);
 static void next_unread_cb	 (MainWindow	*mainwin,
 				  guint		 action,
 				  GtkWidget	*widget);
+static void prev_marked_cb	 (MainWindow	*mainwin,
+				  guint		 action,
+				  GtkWidget	*widget);
+static void next_marked_cb	 (MainWindow	*mainwin,
+				  guint		 action,
+				  GtkWidget	*widget);
+static void prev_labeled_cb	 (MainWindow	*mainwin,
+				  guint		 action,
+				  GtkWidget	*widget);
+static void next_labeled_cb	 (MainWindow	*mainwin,
+				  guint		 action,
+				  GtkWidget	*widget);
 
 static void goto_folder_cb	 (MainWindow	*mainwin,
 				  guint		 action,
@@ -361,12 +386,15 @@ static GtkItemFactoryEntry mainwin_entries[] =
 	{N_("/_View/_Toolbar/Icon _and text"),	NULL, toggle_toolbar_cb, TOOLBAR_BOTH, "<RadioItem>"},
 	{N_("/_View/_Toolbar/_Icon"),		NULL, toggle_toolbar_cb, TOOLBAR_ICON, "/View/Toolbar/Icon and text"},
 	{N_("/_View/_Toolbar/_Text"),		NULL, toggle_toolbar_cb, TOOLBAR_TEXT, "/View/Toolbar/Icon and text"},
-	{N_("/_View/_Toolbar/_Non-display"),	NULL, toggle_toolbar_cb, TOOLBAR_NONE, "/View/Toolbar/Icon and text"},
+	{N_("/_View/_Toolbar/_None"),		NULL, toggle_toolbar_cb, TOOLBAR_NONE, "/View/Toolbar/Icon and text"},
 	{N_("/_View/_Status bar"),		NULL, toggle_statusbar_cb, 0, "<ToggleItem>"},
 	{N_("/_View/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/_View/Separate f_older tree"),	NULL, separate_widget_cb, SEPARATE_FOLDER, "<ToggleItem>"},
 	{N_("/_View/Separate m_essage view"),	NULL, separate_widget_cb, SEPARATE_MESSAGE, "<ToggleItem>"},
 	{N_("/_View/---"),			NULL, NULL, 0, "<Separator>"},
+	{N_("/_View/View _source"),		"<control>U", view_source_cb, 0, NULL},
+	{N_("/_View/Show all _header"),		"<control>H", header_window_show_cb, 0, NULL},
+	{N_("/_View/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/_View/_Code set"),		NULL, NULL, 0, "<Branch>"},
 	{N_("/_View/_Code set/_Auto detect"),
 	 NULL, set_charset_cb, C_AUTO, "<RadioItem>"},
@@ -442,8 +470,8 @@ static GtkItemFactoryEntry mainwin_entries[] =
 #undef CODESET_ACTION
 
 	{N_("/_Message"),			NULL, NULL, 0, "<Branch>"},
-	{N_("/_Message/Rece_ive new mail"),	"<alt>I",	inc_mail_cb, 0, NULL},
-	{N_("/_Message/Receive from _all accounts"),
+	{N_("/_Message/Get new ma_il"),	"<alt>I",	inc_mail_cb, 0, NULL},
+	{N_("/_Message/Get from _all accounts"),
 						"<shift><alt>I", inc_all_account_mail_cb, 0, NULL},
 	{N_("/_Message/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/_Message/Send queued messa_ges"),
@@ -451,26 +479,31 @@ static GtkItemFactoryEntry mainwin_entries[] =
 	{N_("/_Message/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/_Message/Compose _new message"),	"<alt>N",	compose_cb, 0, NULL},
 	{N_("/_Message/_Reply"),		"<alt>R", 	reply_cb, COMPOSE_REPLY, NULL},
+	{N_("/_Message/Repl_y to sender"),	"<control><alt>R", reply_cb, COMPOSE_REPLY_TO_SENDER, NULL},
 	{N_("/_Message/Reply to a_ll"),		"<shift><alt>R", reply_cb, COMPOSE_REPLY_TO_ALL, NULL},
-	{N_("/_Message/_Forward"),		"<control>F",	reply_cb, COMPOSE_FORWARD, NULL},
-	{N_("/_Message/Forward as an a_ttachment"),
+	{N_("/_Message/_Forward"),		"<control>F", reply_cb, COMPOSE_FORWARD, NULL},
+	{N_("/_Message/Forward as a_ttachment"),
 						"<shift><control>F", reply_cb, COMPOSE_FORWARD_AS_ATTACH, NULL},
 	{N_("/_Message/---"),			NULL, NULL, 0, "<Separator>"},
+	{N_("/_Message/Re-_edit"),		NULL, reedit_cb, 0, NULL},
+	{N_("/_Message/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/_Message/M_ove..."),		"<alt>O", move_to_cb, 0, NULL},
 	{N_("/_Message/_Copy..."),		NULL, copy_to_cb, 0, NULL},
 	{N_("/_Message/_Delete"),		"<alt>D", delete_cb,  0, NULL},
+	{N_("/_Message/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/_Message/_Mark"),			NULL, NULL, 0, "<Branch>"},
 	{N_("/_Message/_Mark/_Mark"),		NULL, mark_cb,   0, NULL},
 	{N_("/_Message/_Mark/_Unmark"),		NULL, unmark_cb, 0, NULL},
 	{N_("/_Message/_Mark/---"),		NULL, NULL, 0, "<Separator>"},
 	{N_("/_Message/_Mark/Mark as unr_ead"),	NULL, mark_as_unread_cb, 0, NULL},
-	{N_("/_Message/_Mark/Mark it as _being read"),
+	{N_("/_Message/_Mark/Mark as rea_d"),
 						NULL, mark_as_read_cb, 0, NULL},
+	{N_("/_Message/_Mark/Mark all _read"),	NULL, mark_all_read_cb, 0, NULL},
+	{N_("/_Message/---"),			NULL, NULL, 0, "<Separator>"},
+	{N_("/_Message/Add sender to address boo_k"),
+						NULL, add_address_cb, 0, NULL},
 	{N_("/_Message/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/_Message/Open in new _window"),	"<shift><control>N", open_msg_cb, 0, NULL},
-	{N_("/_Message/View _source"),		"<control>U", view_source_cb, 0, NULL},
-	{N_("/_Message/Show all _header"),	"<control>H",	header_window_show_cb,	0, NULL},
-	{N_("/_Message/Re_edit"),		NULL, reedit_cb, 0, NULL},
 
 	{N_("/_Summary"),			NULL, NULL, 0, "<Branch>"},
 	{N_("/_Summary/_Delete duplicated messages"),
@@ -479,11 +512,25 @@ static GtkItemFactoryEntry mainwin_entries[] =
 	{N_("/_Summary/E_xecute"),		"<alt>X", execute_summary_cb, 0, NULL},
 	{N_("/_Summary/_Update"),		"<alt>U", update_summary_cb,  0, NULL},
 	{N_("/_Summary/---"),			NULL, NULL, 0, "<Separator>"},
-	{N_("/_Summary/_Prev message"),		NULL, prev_cb, 0, NULL},
-	{N_("/_Summary/_Next message"),		NULL, next_cb, 0, NULL},
-	{N_("/_Summary/N_ext unread message"),	NULL, next_unread_cb, 0, NULL},
-	{N_("/_Summary/---"),			NULL, NULL, 0, "<Separator>"},
-	{N_("/_Summary/_Go to other folder"),	"<alt>G", goto_folder_cb, 0, NULL},
+	{N_("/_Summary/Go _to"),		NULL, NULL, 0, "<Branch>"},
+	{N_("/_Summary/Go _to/_Prev message"),	NULL, prev_cb, 0, NULL},
+	{N_("/_Summary/Go _to/_Next message"),	NULL, next_cb, 0, NULL},
+	{N_("/_Summary/Go _to/---"),		NULL, NULL, 0, "<Separator>"},
+	{N_("/_Summary/Go _to/P_rev unread message"),
+						NULL, prev_unread_cb, 0, NULL},
+	{N_("/_Summary/Go _to/N_ext unread message"),
+						NULL, next_unread_cb, 0, NULL},
+	{N_("/_Summary/Go _to/---"),		NULL, NULL, 0, "<Separator>"},
+	{N_("/_Summary/Go _to/Prev _marked message"),
+						NULL, prev_marked_cb, 0, NULL},
+	{N_("/_Summary/Go _to/Next m_arked message"),
+						NULL, next_marked_cb, 0, NULL},
+	{N_("/_Summary/Go _to/---"),		NULL, NULL, 0, "<Separator>"},
+	{N_("/_Summary/Go _to/Prev _labeled message"),
+						NULL, prev_labeled_cb, 0, NULL},
+	{N_("/_Summary/Go _to/Next la_beled message"),
+						NULL, next_labeled_cb, 0, NULL},
+	{N_("/_Summary/_Go to other folder..."),"<alt>G", goto_folder_cb, 0, NULL},
 	{N_("/_Summary/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/_Summary/_Sort"),			NULL, NULL, 0, "<Branch>"},
 	{N_("/_Summary/_Sort/Sort by _number"),	NULL, sort_summary_cb, SORT_BY_NUMBER, NULL},
@@ -491,6 +538,12 @@ static GtkItemFactoryEntry mainwin_entries[] =
 	{N_("/_Summary/_Sort/Sort by _date"),	NULL, sort_summary_cb, SORT_BY_DATE, NULL},
 	{N_("/_Summary/_Sort/Sort by _from"),	NULL, sort_summary_cb, SORT_BY_FROM, NULL},
 	{N_("/_Summary/_Sort/Sort by _subject"),NULL, sort_summary_cb, SORT_BY_SUBJECT, NULL},
+	{N_("/_Summary/_Sort/Sort by _color label"),
+						NULL, sort_summary_cb, SORT_BY_LABEL, NULL},
+	{N_("/_Summary/_Sort/Sort by _mark"),	NULL, sort_summary_cb, SORT_BY_MARK, NULL},
+	{N_("/_Summary/_Sort/Sort by _unread"),	NULL, sort_summary_cb, SORT_BY_UNREAD, NULL},
+	{N_("/_Summary/_Sort/Sort by a_ttachment"),
+						NULL, sort_summary_cb, SORT_BY_MIME, NULL},
 	{N_("/_Summary/_Sort/---"),		NULL, NULL, 0, "<Separator>"},
 	{N_("/_Summary/_Sort/_Attract by subject"),
 						NULL, attract_by_subject_cb, 0, NULL},
@@ -519,7 +572,7 @@ static GtkItemFactoryEntry mainwin_entries[] =
 
 	{N_("/_Help"),				NULL, NULL, 0, "<LastBranch>"},
 	{N_("/_Help/_Manual"),			NULL, NULL, 0, "<Branch>"},
-	{N_("/_Help/_Manual/_English"),		NULL, NULL, MANUAL_LANG_EN, NULL},
+	{N_("/_Help/_Manual/_English"),		NULL, manual_open_cb, MANUAL_LANG_EN, NULL},
 	{N_("/_Help/_Manual/_Japanese"),	NULL, manual_open_cb, MANUAL_LANG_JA, NULL},
 	{N_("/_Help/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/_Help/_About"),			NULL, about_show, 0, NULL}
@@ -535,6 +588,8 @@ MainWindow *main_window_create(SeparateType type)
 	GtkWidget *vbox_body;
 	GtkWidget *hbox_stat;
 	GtkWidget *statusbar;
+	GtkWidget *progressbar;
+	GtkWidget *statuslabel;
 	GtkWidget *ac_button;
 	GtkWidget *ac_label;
 
@@ -542,8 +597,8 @@ MainWindow *main_window_create(SeparateType type)
 	SummaryView *summaryview;
 	MessageView *messageview;
 	GdkColormap *colormap;
-	GdkColor color[5];
-	gboolean success[5];
+	GdkColor color[3];
+	gboolean success[3];
 	guint n_menu_entries;
 	GtkItemFactory *ifactory;
 	GtkWidget *ac_menu;
@@ -594,6 +649,13 @@ MainWindow *main_window_create(SeparateType type)
 	statusbar = statusbar_create();
 	gtk_box_pack_start(GTK_BOX(hbox_stat), statusbar, TRUE, TRUE, 0);
 
+	progressbar = gtk_progress_bar_new();
+	gtk_widget_set_usize(progressbar, 120, 1);
+	gtk_box_pack_start(GTK_BOX(hbox_stat), progressbar, FALSE, FALSE, 0);
+
+	statuslabel = gtk_label_new("");
+	gtk_box_pack_start(GTK_BOX(hbox_stat), statuslabel, FALSE, FALSE, 0);
+
 	ac_button = gtk_button_new();
 	gtk_button_set_relief(GTK_BUTTON(ac_button), GTK_RELIEF_NONE);
 	GTK_WIDGET_UNSET_FLAGS(ac_button, GTK_CAN_FOCUS);
@@ -625,15 +687,17 @@ MainWindow *main_window_create(SeparateType type)
 
 	messageview->mainwin     = mainwin;
 
-	mainwin->window    = window;
-	mainwin->vbox      = vbox;
-	mainwin->menubar   = menubar;
-	mainwin->handlebox = handlebox;
-	mainwin->vbox_body = vbox_body;
-	mainwin->hbox_stat = hbox_stat;
-	mainwin->statusbar = statusbar;
-	mainwin->ac_button = ac_button;
-	mainwin->ac_label  = ac_label;
+	mainwin->window      = window;
+	mainwin->vbox        = vbox;
+	mainwin->menubar     = menubar;
+	mainwin->handlebox   = handlebox;
+	mainwin->vbox_body   = vbox_body;
+	mainwin->hbox_stat   = hbox_stat;
+	mainwin->statusbar   = statusbar;
+	mainwin->progressbar = progressbar;
+	mainwin->statuslabel = statuslabel;
+	mainwin->ac_button   = ac_button;
+	mainwin->ac_label    = ac_label;
 
 	/* set context IDs for status bar */
 	mainwin->mainwin_cid = gtk_statusbar_get_context_id
@@ -650,24 +714,16 @@ MainWindow *main_window_create(SeparateType type)
 	summaryview->color_dim.red = summaryview->color_dim.green =
 		summaryview->color_dim.blue = COLOR_DIM;
 
-	summaryview->color_normal.red = summaryview->color_normal.green =
-		summaryview->color_normal.blue = 0;
-
 	folderview->color_new.red = (guint16)55000;
 	folderview->color_new.green = folderview->color_new.blue = 15000;
 
-	folderview->color_normal.red = folderview->color_normal.green =
-		folderview->color_normal.blue = 0;
-
 	color[0] = summaryview->color_marked;
 	color[1] = summaryview->color_dim;
-	color[2] = summaryview->color_normal;
-	color[3] = folderview->color_new;
-	color[4] = folderview->color_normal;
+	color[2] = folderview->color_new;
 
 	colormap = gdk_window_get_colormap(window->window);
-	gdk_colormap_alloc_colors(colormap, color, 5, FALSE, TRUE, success);
-	for (i = 0; i < 5; i++) {
+	gdk_colormap_alloc_colors(colormap, color, 3, FALSE, TRUE, success);
+	for (i = 0; i < 3; i++) {
 		if (success[i] == FALSE)
 			g_warning(_("MainWindow: color allocation %d failed\n"), i);
 	}
@@ -685,7 +741,7 @@ MainWindow *main_window_create(SeparateType type)
 	switch (prefs_common.toolbar_style) {
 	case TOOLBAR_NONE:
 		menuitem = gtk_item_factory_get_item
-			(ifactory, "/View/Toolbar/Non-display");
+			(ifactory, "/View/Toolbar/None");
 		break;
 	case TOOLBAR_ICON:
 		menuitem = gtk_item_factory_get_item
@@ -710,14 +766,15 @@ MainWindow *main_window_create(SeparateType type)
 			   prefs_common.enable_thread ? FALSE : TRUE);
 	menu_set_sensitive(ifactory, "/Summary/Unthread view",
 			   prefs_common.enable_thread ? TRUE : FALSE);
-	menu_set_sensitive(ifactory, "/Help/Manual/English", FALSE);
 
 	/* set account selection menu */
 	ac_menu = gtk_item_factory_get_widget
 		(ifactory, "/Configuration/Change current account");
+	gtk_signal_connect(GTK_OBJECT(ac_menu), "selection_done",
+			   GTK_SIGNAL_FUNC(ac_menu_popup_closed), mainwin);
 	mainwin->ac_menu = ac_menu;
 
-	main_window_set_toolbar_sensitive(mainwin, FALSE);
+	main_window_set_toolbar_sensitive(mainwin);
 
 	/* show main window */
 	gtk_widget_set_uposition(mainwin->window,
@@ -734,6 +791,7 @@ MainWindow *main_window_create(SeparateType type)
 	header_window_init(mainwin->headerwin);
 	log_window_init(mainwin->logwin);
 
+	mainwin->lock_count = 0;
 	mainwin->cursor_count = 0;
 
 	if (!watch_cursor)
@@ -746,6 +804,7 @@ MainWindow *main_window_create(SeparateType type)
 
 void main_window_cursor_wait(MainWindow *mainwin)
 {
+
 	if (mainwin->cursor_count == 0)
 		gdk_window_set_cursor(mainwin->window->window, watch_cursor);
 
@@ -765,6 +824,30 @@ void main_window_cursor_normal(MainWindow *mainwin)
 	gdk_flush();
 }
 
+/* lock / unlock the user-interface */
+void main_window_lock(MainWindow *mainwin)
+{
+	if (mainwin->lock_count == 0)
+		gtk_widget_set_sensitive(mainwin->ac_button, FALSE);
+
+	mainwin->lock_count++;
+
+	main_window_set_menu_sensitive(mainwin);
+	main_window_set_toolbar_sensitive(mainwin);
+}
+
+void main_window_unlock(MainWindow *mainwin)
+{
+	if (mainwin->lock_count)
+		mainwin->lock_count--;
+
+	main_window_set_menu_sensitive(mainwin);
+	main_window_set_toolbar_sensitive(mainwin);
+
+	if (mainwin->lock_count == 0)
+		gtk_widget_set_sensitive(mainwin->ac_button, TRUE);
+}
+
 void main_window_reflect_prefs_all(void)
 {
 	GList *cur;
@@ -774,13 +857,13 @@ void main_window_reflect_prefs_all(void)
 		mainwin = (MainWindow *)cur->data;
 
 		main_window_show_cur_account(mainwin);
-		if (cur_account) {
-			gtk_widget_set_sensitive(mainwin->get_btn,    TRUE);
-			gtk_widget_set_sensitive(mainwin->getall_btn, TRUE);
-		} else {
-			gtk_widget_set_sensitive(mainwin->get_btn,    FALSE);
-			gtk_widget_set_sensitive(mainwin->getall_btn, FALSE);
-		}
+		main_window_set_menu_sensitive(mainwin);
+		main_window_set_toolbar_sensitive(mainwin);
+
+		if (prefs_common.immediate_exec)
+			gtk_widget_hide(mainwin->exec_btn);
+		else
+			gtk_widget_show(mainwin->exec_btn);
 
 		summary_change_display_item(mainwin->summaryview);
 		summary_redisplay_msg(mainwin->summaryview);
@@ -840,10 +923,11 @@ static void main_window_show_cur_account(MainWindow *mainwin)
 	gtk_window_set_title(GTK_WINDOW(mainwin->window), buf);
 	g_free(buf);
 
-	buf = g_strdup_printf(_("Current account: %s"), ac_name);
-	gtk_label_set_text(GTK_LABEL(mainwin->ac_label), buf);
+	//buf = g_strdup_printf(_("Current account: %s"), ac_name);
+	//gtk_label_set_text(GTK_LABEL(mainwin->ac_label), buf);
+	gtk_label_set_text(GTK_LABEL(mainwin->ac_label), ac_name);
 	gtk_widget_queue_resize(mainwin->ac_button);
-	g_free(buf);
+	//g_free(buf);
 
 	g_free(ac_name);
 }
@@ -1000,76 +1084,145 @@ void main_window_add_mailbox(MainWindow *mainwin)
 	folderview_set(mainwin->folderview);
 }
 
-void main_window_set_toolbar_sensitive(MainWindow *mainwin, gboolean sensitive)
+typedef enum
 {
-	gtk_widget_set_sensitive(mainwin->reply_btn,    sensitive);
-	gtk_widget_set_sensitive(mainwin->replyall_btn, sensitive);
-	gtk_widget_set_sensitive(mainwin->fwd_btn,      sensitive);
-	gtk_widget_set_sensitive(mainwin->exec_btn,     sensitive);
-	gtk_widget_set_sensitive(mainwin->next_btn,     sensitive);
+	M_UNLOCKED            = 1 << 0,
+	M_MSG_EXIST           = 1 << 1,
+	M_TARGET_EXIST        = 1 << 2,
+	M_SINGLE_TARGET_EXIST = 1 << 3,
+	M_EXEC                = 1 << 4,
+	M_ALLOW_REEDIT        = 1 << 5,
+	M_HAVE_ACCOUNT        = 1 << 6
+} SensitiveCond;
 
-	if (!mainwin->summaryview->folder_item ||
-	    mainwin->summaryview->folder_item->folder->type == F_NEWS)
-		gtk_widget_set_sensitive(mainwin->delete_btn, FALSE);
-	else
-		gtk_widget_set_sensitive(mainwin->delete_btn, sensitive);
-}
-
-void main_window_set_menu_sensitive(MainWindow *mainwin, gint selection)
+static SensitiveCond main_window_get_current_state(MainWindow *mainwin)
 {
-	GtkItemFactory *ifactory;
-	gboolean sens;
-	gboolean exec;
+	SensitiveCond state = 0;
+	SummarySelection selection;
 
-	ifactory = gtk_item_factory_from_widget(mainwin->menubar);
-
-	if (selection == SUMMARY_SELECTED_SINGLE)
-		sens = TRUE;
-	else
-		sens = FALSE;
-	if (!mainwin->summaryview->folder_item ||
-	    mainwin->summaryview->folder_item->folder->type == F_NEWS)
-		exec = FALSE;
-	else
-		exec = TRUE;
-
-	menu_set_sensitive(ifactory, "/File/Save as...", sens);
-	menu_set_sensitive(ifactory, "/Message/Reply", sens);
-	menu_set_sensitive(ifactory, "/Message/Reply to all", sens);
-	menu_set_sensitive(ifactory, "/Message/Forward", sens);
-	menu_set_sensitive(ifactory, "/Message/Forward as an attachment", sens);
-	menu_set_sensitive(ifactory, "/Message/Open in new window", sens);
-	menu_set_sensitive(ifactory, "/Message/Show all header", sens);
-	menu_set_sensitive(ifactory, "/Message/View source", sens);
-	if (sens && (!mainwin->summaryview->folder_item ||
-		     mainwin->summaryview->folder_item->stype != F_DRAFT))
-		sens = FALSE;
-	menu_set_sensitive(ifactory, "/Message/Reedit", sens);
+	selection = summary_get_selection_type(mainwin->summaryview);
 
+	if (mainwin->lock_count == 0)
+		state |= M_UNLOCKED;
+	if (selection != SUMMARY_NONE)
+		state |= M_MSG_EXIST;
 	if (selection == SUMMARY_SELECTED_SINGLE ||
 	    selection == SUMMARY_SELECTED_MULTIPLE)
-		sens = TRUE;
-	else
-		sens = FALSE;
+		state |= M_TARGET_EXIST;
+	if (selection == SUMMARY_SELECTED_SINGLE)
+		state |= M_SINGLE_TARGET_EXIST;
+	if (mainwin->summaryview->folder_item &&
+	    mainwin->summaryview->folder_item->folder->type != F_NEWS)
+		state |= M_EXEC;
+	if (selection == SUMMARY_SELECTED_SINGLE &&
+	    (mainwin->summaryview->folder_item &&
+	     (mainwin->summaryview->folder_item->stype == F_DRAFT ||
+	      mainwin->summaryview->folder_item->stype == F_QUEUE)))
+		state |= M_ALLOW_REEDIT;
+	if (cur_account)
+		state |= M_HAVE_ACCOUNT;
 
-	menu_set_sensitive(ifactory, "/File/Print..."  , sens);
-	menu_set_sensitive(ifactory, "/Message/Move...", sens && exec);
-	menu_set_sensitive(ifactory, "/Message/Copy...", sens && exec);
-	menu_set_sensitive(ifactory, "/Message/Delete" , sens && exec);
-	menu_set_sensitive(ifactory, "/Message/Mark"   , sens);
+	return state;
+}
 
-	if (selection != SUMMARY_NONE)
-		sens = TRUE;
-	else
-		sens = FALSE;
+void main_window_set_toolbar_sensitive(MainWindow *mainwin)
+{
+	SensitiveCond state;
+	gboolean sensitive;
+	gint i;
 
-	menu_set_sensitive(ifactory, "/Summary/Delete duplicated messages", sens && exec);
-	menu_set_sensitive(ifactory, "/Summary/Filter messages", sens && exec);
-	menu_set_sensitive(ifactory, "/Summary/Execute", sens);
-	menu_set_sensitive(ifactory, "/Summary/Prev message", sens);
-	menu_set_sensitive(ifactory, "/Summary/Next message", sens);
-	menu_set_sensitive(ifactory, "/Summary/Next unread message", sens);
-	menu_set_sensitive(ifactory, "/Summary/Sort", sens);
+	const struct {
+		GtkWidget *widget;
+		SensitiveCond cond;
+	} entry[] = {
+		{mainwin->get_btn     , M_HAVE_ACCOUNT|M_UNLOCKED},
+		{mainwin->getall_btn  , M_HAVE_ACCOUNT|M_UNLOCKED},
+		{mainwin->compose_btn , M_HAVE_ACCOUNT},
+		{mainwin->reply_btn   , M_HAVE_ACCOUNT|M_SINGLE_TARGET_EXIST},
+		{mainwin->replyall_btn, M_HAVE_ACCOUNT|M_SINGLE_TARGET_EXIST},
+		{mainwin->fwd_btn     , M_HAVE_ACCOUNT|M_SINGLE_TARGET_EXIST},
+		{mainwin->prefs_btn   , M_UNLOCKED},
+		{mainwin->account_btn , M_UNLOCKED},
+		{mainwin->next_btn    , M_MSG_EXIST},
+		{mainwin->delete_btn  , M_TARGET_EXIST|M_EXEC|M_UNLOCKED},
+		{mainwin->exec_btn    , M_MSG_EXIST|M_EXEC|M_UNLOCKED},
+		{NULL, 0}
+	};
+
+	state = main_window_get_current_state(mainwin);
+
+	for (i = 0; entry[i].widget != NULL; i++) {
+		sensitive = ((entry[i].cond & state) == entry[i].cond);
+		gtk_widget_set_sensitive(entry[i].widget, sensitive);
+	}
+}
+
+void main_window_set_menu_sensitive(MainWindow *mainwin)
+{
+	GtkItemFactory *ifactory;
+	SensitiveCond state;
+	gboolean sensitive;
+	gint i;
+
+	static const struct {
+		gchar *const entry;
+		SensitiveCond cond;
+	} entry[] = {
+		{"/File/Add mailbox..."        , M_UNLOCKED},
+		{"/File/Update folder tree"    , M_UNLOCKED},
+		{"/File/Folder"                , M_UNLOCKED},
+		{"/File/Import mbox file..."   , M_UNLOCKED},
+		{"/File/Export to mbox file...", M_UNLOCKED},
+		{"/File/Empty trash"           , M_UNLOCKED},
+		{"/File/Save as...", M_SINGLE_TARGET_EXIST|M_UNLOCKED},
+		{"/File/Print..."  , M_TARGET_EXIST|M_UNLOCKED},
+		{"/File/Close", M_UNLOCKED},
+		{"/File/Exit" , M_UNLOCKED},
+
+		{"/View/Show all header"      , M_SINGLE_TARGET_EXIST},
+		{"/View/View source"          , M_SINGLE_TARGET_EXIST},
+
+		{"/Message/Get new mail"         , M_HAVE_ACCOUNT|M_UNLOCKED},
+		{"/Message/Get from all accounts", M_HAVE_ACCOUNT|M_UNLOCKED},
+		{"/Message/Compose new message"  , M_HAVE_ACCOUNT},
+		{"/Message/Reply"                , M_HAVE_ACCOUNT|M_SINGLE_TARGET_EXIST},
+		{"/Message/Reply to sender"      , M_HAVE_ACCOUNT|M_SINGLE_TARGET_EXIST},
+		{"/Message/Reply to all"         , M_HAVE_ACCOUNT|M_SINGLE_TARGET_EXIST},
+		{"/Message/Forward"              , M_HAVE_ACCOUNT|M_SINGLE_TARGET_EXIST},
+		{"/Message/Forward as attachment", M_HAVE_ACCOUNT|M_SINGLE_TARGET_EXIST},
+		{"/Message/Open in new window"   , M_SINGLE_TARGET_EXIST},
+		{"/Message/Re-edit", M_HAVE_ACCOUNT|M_ALLOW_REEDIT},
+		{"/Message/Move...", M_TARGET_EXIST|M_EXEC|M_UNLOCKED},
+		{"/Message/Copy...", M_TARGET_EXIST|M_EXEC|M_UNLOCKED},
+		{"/Message/Delete" , M_TARGET_EXIST|M_EXEC|M_UNLOCKED},
+		{"/Message/Mark"   , M_TARGET_EXIST},
+		{"/Message/Add sender to address book", M_SINGLE_TARGET_EXIST},
+
+		{"/Summary/Delete duplicated messages", M_MSG_EXIST|M_EXEC|M_UNLOCKED},
+		{"/Summary/Filter messages"           , M_MSG_EXIST|M_EXEC|M_UNLOCKED},
+		{"/Summary/Execute"                   , M_MSG_EXIST|M_EXEC|M_UNLOCKED},
+		{"/Summary/Go to"                     , M_MSG_EXIST},
+		{"/Summary/Go to/Prev message"        , M_MSG_EXIST},
+		{"/Summary/Go to/Next message"        , M_MSG_EXIST},
+		{"/Summary/Go to/Next unread message" , M_MSG_EXIST},
+		{"/Summary/Go to/Prev marked message" , M_MSG_EXIST},
+		{"/Summary/Go to/Next marked message" , M_MSG_EXIST},
+		{"/Summary/Go to/Prev labeled message", M_MSG_EXIST},
+		{"/Summary/Go to/Next labeled message", M_MSG_EXIST},
+		{"/Summary/Sort"                      , M_MSG_EXIST},
+
+		{"/Configuration", M_UNLOCKED},
+
+		{NULL, 0}
+	};
+
+	ifactory = gtk_item_factory_from_widget(mainwin->menubar);
+	state = main_window_get_current_state(mainwin);
+
+	for (i = 0; entry[i].entry != NULL; i++) {
+		sensitive = ((entry[i].cond & state) == entry[i].cond);
+		menu_set_sensitive(ifactory, entry[i].entry, sensitive);
+	}
 }
 
 void main_window_popup(MainWindow *mainwin)
@@ -1085,6 +1238,7 @@ void main_window_popup(MainWindow *mainwin)
 	y %= sy; if (y < 0) y += sy;
 	gdk_window_move(mainwin->window->window, x, y);
 	gdk_window_raise(mainwin->window->window);
+	gdk_window_show(mainwin->window->window);
 
 	debug_print("window position: x = %d, y = %d\n", x, y);
 
@@ -1411,8 +1565,6 @@ static void main_window_toolbar_create(MainWindow *mainwin,
 					   toolbar_exec_cb,
 					   mainwin);
 
-	gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));
-
 	CREATE_TOOLBAR_ICON(stock_down_arrow_xpm);
 	next_btn = gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),
 					   _("Next"),
@@ -1566,7 +1718,12 @@ static void toolbar_account_button_pressed(GtkWidget *widget,
 	if (!event) return;
 	if (event->button != 3) return;
 
-	gtk_menu_popup(GTK_MENU(mainwin->ac_menu), NULL, NULL, NULL, NULL,
+	gtk_button_set_relief(GTK_BUTTON(widget), GTK_RELIEF_NORMAL);
+	gtk_object_set_data(GTK_OBJECT(mainwin->ac_menu), "menu_button",
+			    widget);
+
+	gtk_menu_popup(GTK_MENU(mainwin->ac_menu), NULL, NULL,
+		       menu_button_position, widget,
 		       event->button, event->time);
 }
 
@@ -1577,14 +1734,33 @@ static void ac_label_button_pressed(GtkWidget *widget, GdkEventButton *event,
 
 	if (!event) return;
 
-	gtk_menu_popup(GTK_MENU(mainwin->ac_menu), NULL, NULL, NULL, NULL,
+	gtk_button_set_relief(GTK_BUTTON(widget), GTK_RELIEF_NORMAL);
+	gtk_object_set_data(GTK_OBJECT(mainwin->ac_menu), "menu_button",
+			    widget);
+
+	gtk_menu_popup(GTK_MENU(mainwin->ac_menu), NULL, NULL,
+		       menu_button_position, widget,
 		       event->button, event->time);
 }
 
+static void ac_menu_popup_closed(GtkMenuShell *menu_shell, gpointer data)
+{
+	MainWindow *mainwin = (MainWindow *)data;
+	GtkWidget *button;
+
+	button = gtk_object_get_data(GTK_OBJECT(menu_shell), "menu_button");
+	gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
+	gtk_object_remove_data(GTK_OBJECT(mainwin->ac_menu), "menu_button");
+	manage_window_focus_in(mainwin->window, NULL, NULL);
+}
+
 static gint main_window_close_cb(GtkWidget *widget, GdkEventAny *event,
 				 gpointer data)
 {
-	app_exit_cb(data, 0, widget);
+	MainWindow *mainwin = (MainWindow *)data;
+
+	if (mainwin->lock_count == 0)
+		app_exit_cb(data, 0, widget);
 
 	return TRUE;
 }
@@ -1785,7 +1961,7 @@ static void send_queue_cb(MainWindow *mainwin, guint action, GtkWidget *widget)
 	GList *list;
 
 	if (procmsg_send_queue() < 0)
-		alertpanel_error(_("Sending queued message failed."));
+		alertpanel_error(_("Some errors occurred while sending queued messages."));
 
 	statusbar_pop_all();
 
@@ -1802,10 +1978,18 @@ static void send_queue_cb(MainWindow *mainwin, guint action, GtkWidget *widget)
 
 static void compose_cb(MainWindow *mainwin, guint action, GtkWidget *widget)
 {
-	if (mainwin->summaryview->folder_item)
-		compose_new(mainwin->summaryview->folder_item->folder->account);
-	else
-		compose_new(NULL);
+	PrefsAccount *ac = NULL;
+
+	if (mainwin->summaryview->folder_item) {
+		ac = mainwin->summaryview->folder_item->folder->account;
+		if (ac && ac->protocol == A_NNTP) {
+			compose_new_with_recipient
+				(ac, mainwin->summaryview->folder_item->path);
+			return;
+		}
+	}
+
+	compose_new(ac);
 }
 
 static void reply_cb(MainWindow *mainwin, guint action, GtkWidget *widget)
@@ -1820,10 +2004,16 @@ static void reply_cb(MainWindow *mainwin, guint action, GtkWidget *widget)
 
 	switch (action) {
 	case COMPOSE_REPLY:
-		compose_reply(msginfo, prefs_common.reply_with_quote, FALSE);
+		compose_reply(msginfo, prefs_common.reply_with_quote,
+			      FALSE, FALSE);
+		break;
+	case COMPOSE_REPLY_TO_SENDER:
+		compose_reply(msginfo, prefs_common.reply_with_quote,
+			      FALSE, TRUE);
 		break;
 	case COMPOSE_REPLY_TO_ALL:
-		compose_reply(msginfo, prefs_common.reply_with_quote, TRUE);
+		compose_reply(msginfo, prefs_common.reply_with_quote,
+			      TRUE, FALSE);
 		break;
 	case COMPOSE_FORWARD:
 		compose_forward(msginfo, FALSE);
@@ -1832,7 +2022,7 @@ static void reply_cb(MainWindow *mainwin, guint action, GtkWidget *widget)
 		compose_forward(msginfo, TRUE);
 		break;
 	default:
-		compose_reply(msginfo, prefs_common.reply_with_quote, FALSE);
+		g_warning("reply_cb(): invalid action type: %d\n", action);
 	}
 
 	summary_set_marks_selected(mainwin->summaryview);
@@ -1891,6 +2081,18 @@ static void mark_as_read_cb(MainWindow *mainwin, guint action,
 	summary_mark_as_read(mainwin->summaryview);
 }
 
+static void mark_all_read_cb(MainWindow *mainwin, guint action,
+			     GtkWidget *widget)
+{
+	summary_mark_all_read(mainwin->summaryview);
+}
+
+static void add_address_cb(MainWindow *mainwin, guint action,
+			   GtkWidget *widget)
+{
+	summary_add_address(mainwin->summaryview);
+}
+
 static void set_charset_cb(MainWindow *mainwin, guint action,
 			   GtkWidget *widget)
 {
@@ -1912,7 +2114,7 @@ static void thread_cb(MainWindow *mainwin, guint action, GtkWidget *widget)
 	ifactory = gtk_item_factory_from_widget(widget);
 
 	if (0 == action) {
-		summary_thread_build(mainwin->summaryview);
+		summary_thread_build(mainwin->summaryview, FALSE);
 		prefs_common.enable_thread = TRUE;
 		menu_set_sensitive(ifactory, "/Summary/Thread view",   FALSE);
 		menu_set_sensitive(ifactory, "/Summary/Unthread view", TRUE);
@@ -1985,18 +2187,48 @@ static void next_cb(MainWindow *mainwin, guint action, GtkWidget *widget)
 	summary_step(mainwin->summaryview, GTK_SCROLL_STEP_FORWARD);
 }
 
+static void prev_unread_cb(MainWindow *mainwin, guint action,
+			   GtkWidget *widget)
+{
+	summary_select_prev_unread(mainwin->summaryview);
+}
+
 static void next_unread_cb(MainWindow *mainwin, guint action,
 			   GtkWidget *widget)
 {
 	summary_select_next_unread(mainwin->summaryview);
 }
 
+static void prev_marked_cb(MainWindow *mainwin, guint action,
+			   GtkWidget *widget)
+{
+	summary_select_prev_marked(mainwin->summaryview);
+}
+
+static void next_marked_cb(MainWindow *mainwin, guint action,
+			   GtkWidget *widget)
+{
+	summary_select_next_marked(mainwin->summaryview);
+}
+
+static void prev_labeled_cb(MainWindow *mainwin, guint action,
+			    GtkWidget *widget)
+{
+	summary_select_prev_labeled(mainwin->summaryview);
+}
+
+static void next_labeled_cb(MainWindow *mainwin, guint action,
+			    GtkWidget *widget)
+{
+	summary_select_next_labeled(mainwin->summaryview);
+}
+
 static void goto_folder_cb(MainWindow *mainwin, guint action,
 			   GtkWidget *widget)
 {
 	FolderItem *to_folder;
 
-	to_folder = foldersel_folder_sel(NULL);
+	to_folder = foldersel_folder_sel(NULL, NULL);
 
 	if (to_folder)
 		folderview_select(mainwin->folderview, to_folder);
