@@ -163,7 +163,7 @@ void messageview_show(MessageView *messageview, MsgInfo *msginfo)
 
 		if (!MSG_IS_ENCRYPTED(msginfo->flags) &&
 		    rfc2015_is_encrypted(mimeinfo)) {
-			MSG_SET_FLAGS(msginfo->flags, MSG_ENCRYPTED);
+			MSG_SET_TMP_FLAGS(msginfo->flags, MSG_ENCRYPTED);
 		}
 		if (MSG_IS_ENCRYPTED(msginfo->flags) &&
 		    !msginfo->plaintext_file  &&
