@@ -26,6 +26,8 @@
 #include <glib.h>
 #include <dirent.h>
 #include <sys/stat.h>
+#include <unistd.h>
+#include <string.h>
 #include <errno.h>
 
 #undef MEASURE_TIME
@@ -128,7 +130,8 @@ gchar *mh_fetch_msg(Folder *folder, FolderItem *item, gint num)
 	return file;
 }
 
-gint mh_add_msg(Folder *folder, FolderItem *dest, const gchar *file)
+gint mh_add_msg(Folder *folder, FolderItem *dest, const gchar *file,
+		gboolean remove_source)
 {
 	gchar *destpath;
 	gchar *destfile;
@@ -145,10 +148,25 @@ gint mh_add_msg(Folder *folder, FolderItem *dest, const gchar *file)
 	g_return_val_if_fail(destpath != NULL, -1);
 	destfile = g_strdup_printf("%s%c%d", destpath, G_DIR_SEPARATOR,
 				   dest->last_num + 1);
-	if (copy_file(file, destfile) < 0) {
-		g_warning(_("can't copy message %s to %s\n"), file, destfile);
-		g_free(destfile);
-		return -1;
+
+	if (link(file, destfile) < 0) {
+		if (EXDEV == errno) {
+			if (copy_file(file, destfile) < 0) {
+				g_warning(_("can't copy message %s to %s\n"),
+					  file, destfile);
+				g_free(destfile);
+				return -1;
+			}
+		} else {
+			FILE_OP_ERROR(file, "link");
+			g_free(destfile);
+			return -1;
+		}
+	}
+
+	if (remove_source) {
+		if (unlink(file) < 0)
+			FILE_OP_ERROR(file, "unlink");
 	}
 
 	g_free(destfile);
@@ -188,31 +206,13 @@ gint mh_move_msg(Folder *folder, FolderItem *dest, MsgInfo *msginfo)
 				   dest->last_num + 1);
 	g_free(destdir);
 
-	if (is_file_exist(destfile)) {
-		g_warning(_("%s already exists."), destfile);
+	if (move_file(srcfile, destfile) < 0) {
 		g_free(srcfile);
 		g_free(destfile);
 		if (fp) fclose(fp);
 		return -1;
 	}
 
-	if (rename(srcfile, destfile) < 0) {
-		if (EXDEV == errno) {
-			if (copy_file(srcfile, destfile) < 0) {
-				g_free(srcfile);
-				g_free(destfile);
-				return -1;
-			}
-			unlink(srcfile);
-		} else {
-			FILE_OP_ERROR(srcfile, "rename");
-			g_free(srcfile);
-			g_free(destfile);
-			if (fp) fclose(fp);
-			return -1;
-		}
-	}
-
 	g_free(srcfile);
 	g_free(destfile);
 	dest->last_num++;
@@ -226,8 +226,8 @@ gint mh_move_msg(Folder *folder, FolderItem *dest, MsgInfo *msginfo)
 		    dest->stype == F_QUEUE  ||
 		    dest->stype == F_DRAFT  ||
 		    dest->stype == F_TRASH)
-			MSG_UNSET_FLAGS(newmsginfo.flags,
-					MSG_NEW|MSG_UNREAD|MSG_DELETED);
+			MSG_UNSET_PERM_FLAGS(newmsginfo.flags,
+					     MSG_NEW|MSG_UNREAD|MSG_DELETED);
 
 		procmsg_write_flags(&newmsginfo, fp);
 		fclose(fp);
@@ -272,29 +272,12 @@ gint mh_move_msgs_with_dest(Folder *folder, FolderItem *dest, GSList *msglist)
 		destfile = g_strdup_printf("%s%c%d", destdir, G_DIR_SEPARATOR,
 					   dest->last_num + 1);
 
-		if (is_file_exist(destfile)) {
-			g_warning(_("%s already exists."), destfile);
+		if (move_file(srcfile, destfile) < 0) {
 			g_free(srcfile);
 			g_free(destfile);
 			break;
 		}
 
-		if (rename(srcfile, destfile) < 0) {
-			if (EXDEV == errno) {
-				if (copy_file(srcfile, destfile) < 0) {
-					g_free(srcfile);
-					g_free(destfile);
-					break;
-				}
-				unlink(srcfile);
-			} else {
-				FILE_OP_ERROR(srcfile, "rename");
-				g_free(srcfile);
-				g_free(destfile);
-				break;
-			}
-		}
-
 		g_free(srcfile);
 		g_free(destfile);
 		dest->last_num++;
@@ -308,8 +291,9 @@ gint mh_move_msgs_with_dest(Folder *folder, FolderItem *dest, GSList *msglist)
 			    dest->stype == F_QUEUE  ||
 			    dest->stype == F_DRAFT  ||
 			    dest->stype == F_TRASH)
-				MSG_UNSET_FLAGS(newmsginfo.flags,
-						MSG_NEW|MSG_UNREAD|MSG_DELETED);
+				MSG_UNSET_PERM_FLAGS
+					(newmsginfo.flags,
+					 MSG_NEW|MSG_UNREAD|MSG_DELETED);
 
 			procmsg_write_flags(&newmsginfo, fp);
 		}
@@ -382,8 +366,8 @@ gint mh_copy_msg(Folder *folder, FolderItem *dest, MsgInfo *msginfo)
 		    dest->stype == F_QUEUE  ||
 		    dest->stype == F_DRAFT  ||
 		    dest->stype == F_TRASH)
-			MSG_UNSET_FLAGS(newmsginfo.flags,
-					MSG_NEW|MSG_UNREAD|MSG_DELETED);
+			MSG_UNSET_PERM_FLAGS(newmsginfo.flags,
+					     MSG_NEW|MSG_UNREAD|MSG_DELETED);
 		procmsg_write_flags(&newmsginfo, fp);
 		fclose(fp);
 	}
@@ -454,8 +438,9 @@ gint mh_copy_msgs_with_dest(Folder *folder, FolderItem *dest, GSList *msglist)
 			    dest->stype == F_QUEUE  ||
 			    dest->stype == F_DRAFT  ||
 			    dest->stype == F_TRASH)
-				MSG_UNSET_FLAGS(newmsginfo.flags,
-						MSG_NEW|MSG_UNREAD|MSG_DELETED);
+				MSG_UNSET_PERM_FLAGS
+					(newmsginfo.flags,
+					 MSG_NEW|MSG_UNREAD|MSG_DELETED);
 			procmsg_write_flags(&newmsginfo, fp);
 		}
 	}
@@ -488,32 +473,16 @@ gint mh_remove_msg(Folder *folder, FolderItem *item, gint num)
 gint mh_remove_all_msg(Folder *folder, FolderItem *item)
 {
 	gchar *path;
-	DIR *dp;
-	struct dirent *d;
+	gint val;
 
 	g_return_val_if_fail(item != NULL, -1);
 
 	path = folder_item_get_path(item);
 	g_return_val_if_fail(path != NULL, -1);
-	if (change_dir(path) < 0) {
-		g_free(path);
-		return -1;
-	}
+	val = remove_all_numbered_files(path);
 	g_free(path);
 
-	if ((dp = opendir(".")) == NULL) {
-		FILE_OP_ERROR(item->path, "opendir");
-		return -1;
-	}
-
-	while ((d = readdir(dp)) != NULL) {
-		if (to_number(d->d_name) < 0) continue;
-		if (unlink(d->d_name) < 0)
-			FILE_OP_ERROR(d->d_name, "unlink");
-	}
-
-	closedir(dp);
-	return 0;
+	return val;
 }
 
 gboolean mh_is_msg_changed(Folder *folder, FolderItem *item, MsgInfo *msginfo)
@@ -540,6 +509,8 @@ void mh_scan_folder(Folder *folder, FolderItem *item)
 
 	g_return_if_fail(item != NULL);
 
+	debug_print("mh_scan_folder(): Scanning %s ...\n", item->path);
+
 	path = folder_item_get_path(item);
 	g_return_if_fail(path != NULL);
 	if (change_dir(path) < 0) {
@@ -856,15 +827,18 @@ static MsgInfo *mh_parse_msg(const gchar *file, FolderItem *item)
 {
 	struct stat s;
 	MsgInfo *msginfo;
-	MsgFlags flags = MSG_NEW|MSG_UNREAD;
+	MsgFlags flags;
+
+	flags.perm_flags = MSG_NEW|MSG_UNREAD;
+	flags.tmp_flags = 0;
 
 	g_return_val_if_fail(item != NULL, NULL);
 	g_return_val_if_fail(file != NULL, NULL);
 
 	if (item->stype == F_QUEUE) {
-		MSG_SET_FLAGS(flags, MSG_QUEUED);
+		MSG_SET_TMP_FLAGS(flags, MSG_QUEUED);
 	} else if (item->stype == F_DRAFT) {
-		MSG_SET_FLAGS(flags, MSG_DRAFT);
+		MSG_SET_TMP_FLAGS(flags, MSG_DRAFT);
 	}
 
 	msginfo = procheader_parse(file, flags, FALSE);
