@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999,2000 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2001 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -36,6 +36,7 @@
 #include "main.h"
 #include "prefs.h"
 #include "prefs_account.h"
+#include "prefs_customheader.h"
 #include "account.h"
 #include "mainwindow.h"
 #include "manage_window.h"
@@ -70,6 +71,7 @@ static struct Basic {
 	GtkWidget *recvserv_entry;
 	GtkWidget *smtpserv_entry;
 	GtkWidget *nntpserv_entry;
+	GtkWidget *nntpauth_chkbtn;
 	GtkWidget *uid_label;
 	GtkWidget *pass_label;
 	GtkWidget *uid_entry;
@@ -82,6 +84,9 @@ static struct Receive {
 	GtkWidget *getall_chkbtn;
 	GtkWidget *recvatgetall_chkbtn;
 	GtkWidget *filter_on_recv_chkbtn;
+
+	GtkWidget *imap_frame;
+	GtkWidget *imapdir_entry;
 } receive;
 
 static struct Send {
@@ -114,11 +119,31 @@ static struct Privacy {
 } privacy;
 #endif /* USE_GPGME */
 
+#if USE_SSL
+static struct SSLPrefs {
+	GtkWidget *receive_frame;
+	GtkWidget *pop_chkbtn;
+	GtkWidget *imap_chkbtn;
+
+	GtkWidget *send_frame;
+	GtkWidget *smtp_nossl_radiobtn;
+	GtkWidget *smtp_ssltunnel_radiobtn;
+	GtkWidget *smtp_sslstarttls_radiobtn;
+} ssl;
+#endif /* USE_SSL */
+
 static struct Advanced {
 	GtkWidget *smtpport_chkbtn;
 	GtkWidget *smtpport_entry;
+	GtkWidget *popport_hbox;
 	GtkWidget *popport_chkbtn;
 	GtkWidget *popport_entry;
+	GtkWidget *imapport_hbox;
+	GtkWidget *imapport_chkbtn;
+	GtkWidget *imapport_entry;
+	GtkWidget *nntpport_hbox;
+	GtkWidget *nntpport_chkbtn;
+	GtkWidget *nntpport_entry;
 	GtkWidget *domain_chkbtn;
 	GtkWidget *domain_entry;
 } advanced;
@@ -126,10 +151,10 @@ static struct Advanced {
 static void prefs_account_protocol_set_data_from_optmenu(PrefParam *pparam);
 static void prefs_account_protocol_set_optmenu		(PrefParam *pparam);
 static void prefs_account_protocol_activated		(GtkMenuItem *menuitem);
-#if USE_GPGME
-static void prefs_account_sign_key_set_data_from_radiobtn (PrefParam *pparam);
-static void prefs_account_sign_key_set_radiobtn		  (PrefParam *pparam);
-#endif /* USE_GPGME */
+#if USE_GPGME || USE_SSL
+static void prefs_account_enum_set_data_from_radiobtn	(PrefParam *pparam);
+static void prefs_account_enum_set_radiobtn		(PrefParam *pparam);
+#endif /* USE_GPGME || USE_SSL */
 
 static PrefParam param[] = {
 	/* Basic */
@@ -164,7 +189,8 @@ static PrefParam param[] = {
 	 &basic.nntpserv_entry, prefs_set_data_from_entry, prefs_set_entry},
 
 	{"use_nntp_auth", "FALSE", &tmp_ac_prefs.use_nntp_auth, P_BOOL,
-	 NULL, NULL, NULL},
+	 &basic.nntpauth_chkbtn,
+	 prefs_set_data_from_toggle, prefs_set_toggle},
 
 	{"user_id", "ENV_USER", &tmp_ac_prefs.userid, P_STRING,
 	 &basic.uid_entry, prefs_set_data_from_entry, prefs_set_entry},
@@ -192,6 +218,9 @@ static PrefParam param[] = {
 	 &receive.filter_on_recv_chkbtn,
 	 prefs_set_data_from_toggle, prefs_set_toggle},
 
+	{"imap_directory", NULL, &tmp_ac_prefs.imap_dir, P_STRING,
+	 &receive.imapdir_entry, prefs_set_data_from_entry, prefs_set_entry},
+
 	/* Send */
 	{"add_date", "TRUE", &tmp_ac_prefs.add_date, P_BOOL,
 	 &send.date_chkbtn,
@@ -246,13 +275,25 @@ static PrefParam param[] = {
 	/* Privacy */
 	{"sign_key", NULL, &tmp_ac_prefs.sign_key, P_ENUM,
 	 &privacy.defaultkey_radiobtn,
-	 prefs_account_sign_key_set_data_from_radiobtn,
-	 prefs_account_sign_key_set_radiobtn},
+	 prefs_account_enum_set_data_from_radiobtn,
+	 prefs_account_enum_set_radiobtn},
 	{"sign_key_id", NULL, &tmp_ac_prefs.sign_key_id, P_STRING,
 	 &privacy.customkey_entry,
 	 prefs_set_data_from_entry, prefs_set_entry},
 #endif /* USE_GPGME */
 
+#if USE_SSL
+	/* SSL */
+	{"ssl_smtp", "0", &tmp_ac_prefs.ssl_smtp, P_ENUM,
+	 &ssl.smtp_nossl_radiobtn,
+	 prefs_account_enum_set_data_from_radiobtn,
+	 prefs_account_enum_set_radiobtn},
+	{"ssl_pop", "FALSE", &tmp_ac_prefs.ssl_pop, P_BOOL,
+	 &ssl.pop_chkbtn, prefs_set_data_from_toggle, prefs_set_toggle},
+	{"ssl_imap", "FALSE", &tmp_ac_prefs.ssl_imap, P_BOOL,
+	 &ssl.imap_chkbtn, prefs_set_data_from_toggle, prefs_set_toggle},
+#endif /* USE_SSL */
+
 	/* Advanced */
 	{"set_smtpport", "FALSE", &tmp_ac_prefs.set_smtpport, P_BOOL,
 	 &advanced.smtpport_chkbtn,
@@ -270,6 +311,22 @@ static PrefParam param[] = {
 	 &advanced.popport_entry,
 	 prefs_set_data_from_entry, prefs_set_entry},
 
+	{"set_imapport", "FALSE", &tmp_ac_prefs.set_imapport, P_BOOL,
+	 &advanced.imapport_chkbtn,
+	 prefs_set_data_from_toggle, prefs_set_toggle},
+
+	{"imap_port", "143", &tmp_ac_prefs.imapport, P_USHORT,
+	 &advanced.imapport_entry,
+	 prefs_set_data_from_entry, prefs_set_entry},
+
+	{"set_nntpport", "FALSE", &tmp_ac_prefs.set_nntpport, P_BOOL,
+	 &advanced.nntpport_chkbtn,
+	 prefs_set_data_from_toggle, prefs_set_toggle},
+
+	{"nntp_port", "119", &tmp_ac_prefs.nntpport, P_USHORT,
+	 &advanced.nntpport_entry,
+	 prefs_set_data_from_entry, prefs_set_entry},
+
 	{"set_domain", "FALSE", &tmp_ac_prefs.set_domain, P_BOOL,
 	 &advanced.domain_chkbtn,
 	 prefs_set_data_from_toggle, prefs_set_toggle},
@@ -289,8 +346,13 @@ static void prefs_account_compose_create	(void);
 #if USE_GPGME
 static void prefs_account_privacy_create	(void);
 #endif /* USE_GPGME */
+#if USE_SSL
+static void prefs_account_ssl_create		(void);
+#endif /* USE_SSL */
 static void prefs_account_advanced_create	(void);
 
+static void prefs_account_edit_custom_header	(void);
+
 static gint prefs_account_deleted		(GtkWidget	*widget,
 						 GdkEventAny	*event,
 						 gpointer	 data);
@@ -301,11 +363,6 @@ static void prefs_account_ok			(void);
 static gint prefs_account_apply			(void);
 static void prefs_account_cancel		(void);
 
-#define VSPACING		12
-#define VSPACING_NARROW		4
-#define BOX_BORDER		16
-#define DEFAULT_ENTRY_WIDTH	80
-
 void prefs_account_read_config(PrefsAccount *ac_prefs, const gchar *label)
 {
 	const gchar *p = label;
@@ -321,6 +378,8 @@ void prefs_account_read_config(PrefsAccount *ac_prefs, const gchar *label)
 	id = atoi(p);
 	if (id < 0) g_warning("wrong account id: %d\n", id);
 	ac_prefs->account_id = id;
+
+	prefs_custom_header_read_config(ac_prefs);
 }
 
 void prefs_account_save_config(PrefsAccount *ac_prefs)
@@ -401,7 +460,7 @@ PrefsAccount *prefs_account_open(PrefsAccount *ac_prefs)
 
 	debug_print(_("Opening account preferences window...\n"));
 
-	inc_autocheck_timer_remove();
+	inc_lock();
 
 	cancelled = FALSE;
 
@@ -452,7 +511,7 @@ PrefsAccount *prefs_account_open(PrefsAccount *ac_prefs)
 	gtk_main();
 	gtk_widget_hide(dialog.window);
 
-	inc_autocheck_timer_set();
+	inc_unlock();
 
 	if (cancelled && new_account) {
 		g_free(ac_prefs);
@@ -498,6 +557,10 @@ static void prefs_account_create(void)
 	prefs_account_privacy_create();
 	SET_NOTEBOOK_LABEL(dialog.notebook, _("Privacy"), page++);
 #endif /* USE_GPGME */
+#if USE_SSL
+	prefs_account_ssl_create();
+	SET_NOTEBOOK_LABEL(dialog.notebook, _("SSL"), page++);
+#endif /* USE_SSL */
 	prefs_account_advanced_create();
 	SET_NOTEBOOK_LABEL(dialog.notebook, _("Advanced"), page++);
 
@@ -538,6 +601,7 @@ static void prefs_account_basic_create(void)
 	GtkWidget *recvserv_entry;
 	GtkWidget *smtpserv_entry;
 	GtkWidget *nntpserv_entry;
+	GtkWidget *nntpauth_chkbtn;
 	GtkWidget *uid_label;
 	GtkWidget *pass_label;
 	GtkWidget *uid_entry;
@@ -546,7 +610,7 @@ static void prefs_account_basic_create(void)
 	vbox1 = gtk_vbox_new (FALSE, VSPACING);
 	gtk_widget_show (vbox1);
 	gtk_container_add (GTK_CONTAINER (dialog.notebook), vbox1);
-	gtk_container_set_border_width (GTK_CONTAINER (vbox1), BOX_BORDER);
+	gtk_container_set_border_width (GTK_CONTAINER (vbox1), VBOX_BORDER);
 
 	hbox = gtk_hbox_new (FALSE, 8);
 	gtk_widget_show (hbox);
@@ -653,12 +717,19 @@ static void prefs_account_basic_create(void)
 	gtk_widget_set_usize (inbox_entry, DEFAULT_ENTRY_WIDTH, -1);
 	gtk_box_pack_start (GTK_BOX (hbox), inbox_entry, TRUE, TRUE, 0);
 
-	serv_table = gtk_table_new (5, 4, FALSE);
+	serv_table = gtk_table_new (6, 4, FALSE);
 	gtk_widget_show (serv_table);
 	gtk_box_pack_start (GTK_BOX (vbox2), serv_table, FALSE, FALSE, 0);
 	gtk_table_set_row_spacings (GTK_TABLE (serv_table), VSPACING_NARROW);
+	gtk_table_set_row_spacing (GTK_TABLE (serv_table), 3, 0);
 	gtk_table_set_col_spacings (GTK_TABLE (serv_table), 8);
 
+	nntpauth_chkbtn = gtk_check_button_new_with_label
+		(_("This server requires authentication"));
+	gtk_widget_show (nntpauth_chkbtn);
+	gtk_table_attach (GTK_TABLE (serv_table), nntpauth_chkbtn, 0, 4, 4, 5,
+			  GTK_FILL, 0, 0, 0);
+
 	nntpserv_entry = gtk_entry_new ();
 	gtk_widget_show (nntpserv_entry);
 	gtk_table_attach (GTK_TABLE (serv_table), nntpserv_entry, 1, 4, 0, 1,
@@ -681,14 +752,14 @@ static void prefs_account_basic_create(void)
 	uid_entry = gtk_entry_new ();
 	gtk_widget_show (uid_entry);
 	gtk_widget_set_usize (uid_entry, DEFAULT_ENTRY_WIDTH, -1);
-	gtk_table_attach (GTK_TABLE (serv_table), uid_entry, 1, 2, 4, 5,
+	gtk_table_attach (GTK_TABLE (serv_table), uid_entry, 1, 2, 5, 6,
 			  GTK_EXPAND | GTK_SHRINK | GTK_FILL,
 			  GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
 
 	pass_entry = gtk_entry_new ();
 	gtk_widget_show (pass_entry);
 	gtk_widget_set_usize (pass_entry, DEFAULT_ENTRY_WIDTH, -1);
-	gtk_table_attach (GTK_TABLE (serv_table), pass_entry, 3, 4, 4, 5,
+	gtk_table_attach (GTK_TABLE (serv_table), pass_entry, 3, 4, 5, 6,
 			  GTK_EXPAND | GTK_SHRINK | GTK_FILL,
 			  GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
 	gtk_entry_set_visibility (GTK_ENTRY (pass_entry), FALSE);
@@ -714,15 +785,20 @@ static void prefs_account_basic_create(void)
 
 	uid_label = gtk_label_new (_("User ID"));
 	gtk_widget_show (uid_label);
-	gtk_table_attach (GTK_TABLE (serv_table), uid_label, 0, 1, 4, 5,
+	gtk_table_attach (GTK_TABLE (serv_table), uid_label, 0, 1, 5, 6,
 			  GTK_FILL, 0, 0, 0);
 	gtk_misc_set_alignment (GTK_MISC (uid_label), 1, 0.5);
 
 	pass_label = gtk_label_new (_("Password"));
 	gtk_widget_show (pass_label);
-	gtk_table_attach (GTK_TABLE (serv_table), pass_label, 2, 3, 4, 5,
+	gtk_table_attach (GTK_TABLE (serv_table), pass_label, 2, 3, 5, 6,
 			  0, 0, 0, 0);
 
+	SET_TOGGLE_SENSITIVITY (nntpauth_chkbtn, uid_label);
+	SET_TOGGLE_SENSITIVITY (nntpauth_chkbtn, pass_label);
+	SET_TOGGLE_SENSITIVITY (nntpauth_chkbtn, uid_entry);
+	SET_TOGGLE_SENSITIVITY (nntpauth_chkbtn, pass_entry);
+
 	basic.acname_entry   = acname_entry;
 	basic.default_chkbtn = default_chkbtn;
 
@@ -741,6 +817,7 @@ static void prefs_account_basic_create(void)
 	basic.smtpserv_entry   = smtpserv_entry;
 	basic.nntpserv_label   = nntpserv_label;
 	basic.nntpserv_entry   = nntpserv_entry;
+	basic.nntpauth_chkbtn  = nntpauth_chkbtn;
 	basic.uid_label        = uid_label;
 	basic.pass_label       = pass_label;
 	basic.uid_entry        = uid_entry;
@@ -756,15 +833,19 @@ static void prefs_account_receive_create(void)
 	GtkWidget *getall_chkbtn;
 	GtkWidget *recvatgetall_chkbtn;
 	GtkWidget *filter_on_recv_chkbtn;
+	GtkWidget *frame2;
+	GtkWidget *hbox1;
+	GtkWidget *imapdir_label;
+	GtkWidget *imapdir_entry;
 
 	vbox1 = gtk_vbox_new (FALSE, VSPACING);
 	gtk_widget_show (vbox1);
 	gtk_container_add (GTK_CONTAINER (dialog.notebook), vbox1);
-	gtk_container_set_border_width (GTK_CONTAINER (vbox1), BOX_BORDER);
+	gtk_container_set_border_width (GTK_CONTAINER (vbox1), VBOX_BORDER);
 
 	PACK_FRAME (vbox1, frame1, _("POP3"));
 
-	vbox2 = gtk_vbox_new (FALSE, VSPACING_NARROW);
+	vbox2 = gtk_vbox_new (FALSE, 0);
 	gtk_widget_show (vbox2);
 	gtk_container_add (GTK_CONTAINER (frame1), vbox2);
 	gtk_container_set_border_width (GTK_CONTAINER (vbox2), 8);
@@ -772,18 +853,40 @@ static void prefs_account_receive_create(void)
 	PACK_CHECK_BUTTON (vbox2, rmmail_chkbtn,
 			   _("Remove messages on server when received"));
 	PACK_CHECK_BUTTON (vbox2, getall_chkbtn,
-			   _("Receive all messages on server"));
+			   _("Download all messages on server"));
 	PACK_CHECK_BUTTON
 		(vbox2, recvatgetall_chkbtn,
-		 _("`Receive all' checks for new mail on this account"));
+		 _("`Get all' checks for new mail on this account"));
 	PACK_CHECK_BUTTON (vbox2, filter_on_recv_chkbtn,
 			   _("Filter messages on receiving"));
 
+	PACK_FRAME (vbox1, frame2, _("IMAP4"));
+
+	vbox2 = gtk_vbox_new (FALSE, VSPACING_NARROW);
+	gtk_widget_show (vbox2);
+	gtk_container_add (GTK_CONTAINER (frame2), vbox2);
+	gtk_container_set_border_width (GTK_CONTAINER (vbox2), 8);
+
+	hbox1 = gtk_hbox_new (FALSE, 8);
+	gtk_widget_show (hbox1);
+	gtk_box_pack_start (GTK_BOX (vbox2), hbox1, FALSE, FALSE, 0);
+
+	imapdir_label = gtk_label_new (_("IMAP server directory"));
+	gtk_widget_show (imapdir_label);
+	gtk_box_pack_start (GTK_BOX (hbox1), imapdir_label, FALSE, FALSE, 0);
+
+	imapdir_entry = gtk_entry_new();
+	gtk_widget_show (imapdir_entry);
+	gtk_box_pack_start (GTK_BOX (hbox1), imapdir_entry, TRUE, TRUE, 0);
+
 	receive.pop3_frame            = frame1;
 	receive.rmmail_chkbtn         = rmmail_chkbtn;
 	receive.getall_chkbtn         = getall_chkbtn;
 	receive.recvatgetall_chkbtn   = recvatgetall_chkbtn;
 	receive.filter_on_recv_chkbtn = filter_on_recv_chkbtn;
+
+	receive.imap_frame    = frame2;
+	receive.imapdir_entry = imapdir_entry;
 }
 
 static void prefs_account_send_create(void)
@@ -812,11 +915,11 @@ static void prefs_account_send_create(void)
 	vbox1 = gtk_vbox_new (FALSE, VSPACING);
 	gtk_widget_show (vbox1);
 	gtk_container_add (GTK_CONTAINER (dialog.notebook), vbox1);
-	gtk_container_set_border_width (GTK_CONTAINER (vbox1), BOX_BORDER);
+	gtk_container_set_border_width (GTK_CONTAINER (vbox1), VBOX_BORDER);
 
 	PACK_FRAME (vbox1, frame, _("Header"));
 
-	vbox2 = gtk_vbox_new (FALSE, VSPACING_NARROW);
+	vbox2 = gtk_vbox_new (FALSE, 0);
 	gtk_widget_show (vbox2);
 	gtk_container_add (GTK_CONTAINER (frame), vbox2);
 	gtk_container_set_border_width (GTK_CONTAINER (vbox2), 8);
@@ -828,8 +931,6 @@ static void prefs_account_send_create(void)
 	gtk_widget_show (hbox);
 	gtk_box_pack_start (GTK_BOX (vbox2), hbox, FALSE, FALSE, 0);
 
-	gtk_widget_set_sensitive(hbox, FALSE);
-
 	PACK_CHECK_BUTTON (hbox, customhdr_chkbtn,
 			   _("Add user-defined header"));
 
@@ -837,6 +938,9 @@ static void prefs_account_send_create(void)
 	gtk_widget_show (customhdr_edit_btn);
 	gtk_box_pack_start (GTK_BOX (hbox), customhdr_edit_btn,
 			    FALSE, FALSE, 0);
+	gtk_signal_connect (GTK_OBJECT (customhdr_edit_btn), "clicked",
+			    GTK_SIGNAL_FUNC (prefs_account_edit_custom_header),
+			    NULL);
 
 	SET_TOGGLE_SENSITIVITY (customhdr_chkbtn, customhdr_edit_btn);
 
@@ -846,7 +950,7 @@ static void prefs_account_send_create(void)
 	gtk_widget_show (table);
 	gtk_container_add (GTK_CONTAINER (frame2), table);
 	gtk_container_set_border_width (GTK_CONTAINER (table), 8);
-	gtk_table_set_row_spacings (GTK_TABLE (table), VSPACING_NARROW);
+	gtk_table_set_row_spacings (GTK_TABLE (table), VSPACING_NARROW_2);
 	gtk_table_set_col_spacings (GTK_TABLE (table), 8);
 
 	autocc_chkbtn = gtk_check_button_new_with_label (_("Cc"));
@@ -890,7 +994,7 @@ static void prefs_account_send_create(void)
 
 	PACK_FRAME (vbox1, frame3, _("Authentication"));
 
-	vbox3 = gtk_vbox_new (FALSE, VSPACING_NARROW);
+	vbox3 = gtk_vbox_new (FALSE, 0);
 	gtk_widget_show (vbox3);
 	gtk_container_add (GTK_CONTAINER (frame3), vbox3);
 	gtk_container_set_border_width (GTK_CONTAINER (vbox3), 8);
@@ -926,7 +1030,7 @@ static void prefs_account_compose_create(void)
 	vbox1 = gtk_vbox_new (FALSE, VSPACING);
 	gtk_widget_show (vbox1);
 	gtk_container_add (GTK_CONTAINER (dialog.notebook), vbox1);
-	gtk_container_set_border_width (GTK_CONTAINER (vbox1), BOX_BORDER);
+	gtk_container_set_border_width (GTK_CONTAINER (vbox1), VBOX_BORDER);
 
 	hbox = gtk_hbox_new (FALSE, 8);
 	gtk_widget_show (hbox);
@@ -959,7 +1063,7 @@ static void prefs_account_privacy_create(void)
 	vbox1 = gtk_vbox_new (FALSE, VSPACING);
 	gtk_widget_show (vbox1);
 	gtk_container_add (GTK_CONTAINER (dialog.notebook), vbox1);
-	gtk_container_set_border_width (GTK_CONTAINER (vbox1), BOX_BORDER);
+	gtk_container_set_border_width (GTK_CONTAINER (vbox1), VBOX_BORDER);
 
 	PACK_FRAME (vbox1, frame1, _("Sign key"));
 
@@ -1021,6 +1125,83 @@ static void prefs_account_privacy_create(void)
 }
 #endif /* USE_GPGME */
 
+#if USE_SSL
+static void prefs_account_ssl_create(void)
+{
+	GtkWidget *vbox1;
+
+	GtkWidget *receive_frame;
+	GtkWidget *vbox2;
+	GtkWidget *pop_chkbtn;
+	GtkWidget *imap_chkbtn;
+
+	GtkWidget *send_frame;
+	GtkWidget *vbox3;
+	GtkWidget *smtp_nossl_radiobtn;
+	GtkWidget *smtp_ssltunnel_radiobtn;
+	GtkWidget *smtp_sslstarttls_radiobtn;
+
+	vbox1 = gtk_vbox_new (FALSE, VSPACING);
+	gtk_widget_show (vbox1);
+	gtk_container_add (GTK_CONTAINER (dialog.notebook), vbox1);
+	gtk_container_set_border_width (GTK_CONTAINER (vbox1), VBOX_BORDER);
+
+	PACK_FRAME (vbox1, receive_frame, _("Receive"));
+
+	vbox2 = gtk_vbox_new (FALSE, 0);
+	gtk_widget_show (vbox2);
+	gtk_container_add (GTK_CONTAINER (receive_frame), vbox2);
+	gtk_container_set_border_width (GTK_CONTAINER (vbox2), 8);
+
+	PACK_CHECK_BUTTON (vbox2, pop_chkbtn,
+			   _("Use SSL for POP3 connection"));
+	PACK_CHECK_BUTTON (vbox2, imap_chkbtn,
+			   _("Use SSL for IMAP4 connection"));
+
+	PACK_FRAME (vbox1, send_frame, _("Send (SMTP)"));
+
+	vbox3 = gtk_vbox_new (FALSE, 0);
+	gtk_widget_show (vbox3);
+	gtk_container_add (GTK_CONTAINER (send_frame), vbox3);
+	gtk_container_set_border_width (GTK_CONTAINER (vbox3), 8);
+
+	smtp_nossl_radiobtn =
+		gtk_radio_button_new_with_label(NULL, _("Don't use SSL"));
+	gtk_widget_show (smtp_nossl_radiobtn);
+	gtk_box_pack_start (GTK_BOX (vbox3), smtp_nossl_radiobtn,
+			    FALSE, FALSE, 0);
+	gtk_object_set_user_data (GTK_OBJECT (smtp_nossl_radiobtn),
+				  GINT_TO_POINTER (SSL_SMTP_NONE));
+
+	smtp_ssltunnel_radiobtn = gtk_radio_button_new_with_label_from_widget
+		(GTK_RADIO_BUTTON (smtp_nossl_radiobtn),
+		 _("Use SSL for SMTP connection"));
+	gtk_widget_show (smtp_ssltunnel_radiobtn);
+	gtk_box_pack_start (GTK_BOX (vbox3), smtp_ssltunnel_radiobtn,
+			    FALSE, FALSE, 0);
+	gtk_object_set_user_data (GTK_OBJECT (smtp_ssltunnel_radiobtn),
+				  GINT_TO_POINTER (SSL_SMTP_TUNNEL));
+
+	smtp_sslstarttls_radiobtn = gtk_radio_button_new_with_label_from_widget
+		(GTK_RADIO_BUTTON (smtp_nossl_radiobtn),
+		 _("Use STARTTLS command to start SSL session"));
+	gtk_widget_show (smtp_sslstarttls_radiobtn);
+	gtk_box_pack_start (GTK_BOX (vbox3), smtp_sslstarttls_radiobtn,
+			    FALSE, FALSE, 0);
+	gtk_object_set_user_data (GTK_OBJECT (smtp_sslstarttls_radiobtn),
+				  GINT_TO_POINTER (SSL_SMTP_STARTTLS));
+
+	ssl.receive_frame = receive_frame;
+	ssl.pop_chkbtn    = pop_chkbtn;
+	ssl.imap_chkbtn   = imap_chkbtn;
+
+	ssl.send_frame                = send_frame;
+	ssl.smtp_nossl_radiobtn       = smtp_nossl_radiobtn;
+	ssl.smtp_ssltunnel_radiobtn   = smtp_ssltunnel_radiobtn;
+	ssl.smtp_sslstarttls_radiobtn = smtp_sslstarttls_radiobtn;
+}
+#endif /* USE_SSL */
+
 static void prefs_account_advanced_create(void)
 {
 	GtkWidget *vbox1;
@@ -1028,61 +1209,87 @@ static void prefs_account_advanced_create(void)
 	GtkWidget *hbox1;
 	GtkWidget *checkbtn_smtpport;
 	GtkWidget *entry_smtpport;
-	GtkWidget *hbox2;
+	GtkWidget *hbox_popport;
 	GtkWidget *checkbtn_popport;
 	GtkWidget *entry_popport;
-	GtkWidget *hbox3;
+	GtkWidget *hbox_imapport;
+	GtkWidget *checkbtn_imapport;
+	GtkWidget *entry_imapport;
+	GtkWidget *hbox_nntpport;
+	GtkWidget *checkbtn_nntpport;
+	GtkWidget *entry_nntpport;
 	GtkWidget *checkbtn_domain;
 	GtkWidget *entry_domain;
 
+#define PACK_HBOX(hbox) \
+{ \
+	hbox = gtk_hbox_new (FALSE, 8); \
+	gtk_widget_show (hbox); \
+	gtk_box_pack_start (GTK_BOX (vbox2), hbox, FALSE, FALSE, 0); \
+}
+
+#define PACK_PORT_ENTRY(box, entry) \
+{ \
+	entry = gtk_entry_new_with_max_length (5); \
+	gtk_widget_show (entry); \
+	gtk_box_pack_start (GTK_BOX (box), entry, FALSE, FALSE, 0); \
+	gtk_widget_set_usize (entry, 64, -1); \
+}
+
 	vbox1 = gtk_vbox_new (FALSE, VSPACING);
 	gtk_widget_show (vbox1);
 	gtk_container_add (GTK_CONTAINER (dialog.notebook), vbox1);
-	gtk_container_set_border_width (GTK_CONTAINER (vbox1), BOX_BORDER);
+	gtk_container_set_border_width (GTK_CONTAINER (vbox1), VBOX_BORDER);
 
-	vbox2 = gtk_vbox_new (FALSE, VSPACING_NARROW);
+	vbox2 = gtk_vbox_new (FALSE, VSPACING_NARROW_2);
 	gtk_widget_show (vbox2);
 	gtk_box_pack_start (GTK_BOX (vbox1), vbox2, FALSE, FALSE, 0);
 
-	hbox1 = gtk_hbox_new (FALSE, 8);
-	gtk_widget_show (hbox1);
-	gtk_box_pack_start (GTK_BOX (vbox2), hbox1, FALSE, FALSE, 0);
-
+	PACK_HBOX (hbox1);
 	PACK_CHECK_BUTTON (hbox1, checkbtn_smtpport, _("Specify SMTP port"));
+	PACK_PORT_ENTRY (hbox1, entry_smtpport);
+	SET_TOGGLE_SENSITIVITY (checkbtn_smtpport, entry_smtpport);
 
-	entry_smtpport = gtk_entry_new_with_max_length (5);
-	gtk_widget_show (entry_smtpport);
-	gtk_box_pack_start (GTK_BOX (hbox1), entry_smtpport, FALSE, FALSE, 0);
-	gtk_widget_set_usize (entry_smtpport, 64, -1);
-	SET_TOGGLE_SENSITIVITY(checkbtn_smtpport, entry_smtpport);
+	PACK_HBOX (hbox_popport);
+	PACK_CHECK_BUTTON (hbox_popport, checkbtn_popport,
+			   _("Specify POP3 port"));
+	PACK_PORT_ENTRY (hbox_popport, entry_popport);
+	SET_TOGGLE_SENSITIVITY (checkbtn_popport, entry_popport);
 
-	hbox2 = gtk_hbox_new (FALSE, 8);
-	gtk_widget_show (hbox2);
-	gtk_box_pack_start (GTK_BOX (vbox2), hbox2, FALSE, FALSE, 0);
+	PACK_HBOX (hbox_imapport);
+	PACK_CHECK_BUTTON (hbox_imapport, checkbtn_imapport,
+			   _("Specify IMAP4 port"));
+	PACK_PORT_ENTRY (hbox_imapport, entry_imapport);
+	SET_TOGGLE_SENSITIVITY (checkbtn_imapport, entry_imapport);
 
-	PACK_CHECK_BUTTON (hbox2, checkbtn_popport, _("Specify POP3 port"));
+	PACK_HBOX (hbox_nntpport);
+	PACK_CHECK_BUTTON (hbox_nntpport, checkbtn_nntpport,
+			   _("Specify NNTP port"));
+	PACK_PORT_ENTRY (hbox_nntpport, entry_nntpport);
+	SET_TOGGLE_SENSITIVITY (checkbtn_nntpport, entry_nntpport);
 
-	entry_popport = gtk_entry_new_with_max_length (5);
-	gtk_widget_show (entry_popport);
-	gtk_box_pack_start (GTK_BOX (hbox2), entry_popport, FALSE, FALSE, 0);
-	gtk_widget_set_usize (entry_popport, 64, -1);
-	SET_TOGGLE_SENSITIVITY(checkbtn_popport, entry_popport);
-
-	hbox3 = gtk_hbox_new (FALSE, 8);
-	gtk_widget_show (hbox3);
-	gtk_box_pack_start (GTK_BOX (vbox2), hbox3, FALSE, FALSE, 0);
-
-	PACK_CHECK_BUTTON (hbox3, checkbtn_domain, _("Specify domain name"));
+	PACK_HBOX (hbox1);
+	PACK_CHECK_BUTTON (hbox1, checkbtn_domain, _("Specify domain name"));
 
 	entry_domain = gtk_entry_new ();
 	gtk_widget_show (entry_domain);
-	gtk_box_pack_start (GTK_BOX (hbox3), entry_domain, TRUE, TRUE, 0);
-	SET_TOGGLE_SENSITIVITY(checkbtn_domain, entry_domain);
+	gtk_box_pack_start (GTK_BOX (hbox1), entry_domain, TRUE, TRUE, 0);
+	SET_TOGGLE_SENSITIVITY (checkbtn_domain, entry_domain);
+
+#undef PACK_HBOX
+#undef PACK_PORT_ENTRY
 
 	advanced.smtpport_chkbtn	= checkbtn_smtpport;
 	advanced.smtpport_entry		= entry_smtpport;
+	advanced.popport_hbox		= hbox_popport;
 	advanced.popport_chkbtn		= checkbtn_popport;
 	advanced.popport_entry		= entry_popport;
+	advanced.imapport_hbox		= hbox_imapport;
+	advanced.imapport_chkbtn	= checkbtn_imapport;
+	advanced.imapport_entry		= entry_imapport;
+	advanced.nntpport_hbox		= hbox_nntpport;
+	advanced.nntpport_chkbtn	= checkbtn_nntpport;
+	advanced.nntpport_entry		= entry_nntpport;
 	advanced.domain_chkbtn		= checkbtn_domain;
 	advanced.domain_entry		= entry_domain;
 }
@@ -1158,8 +1365,13 @@ static void prefs_account_cancel(void)
 	gtk_main_quit();
 }
 
-#if USE_GPGME
-static void prefs_account_sign_key_set_data_from_radiobtn(PrefParam *pparam)
+static void prefs_account_edit_custom_header(void)
+{
+	prefs_custom_header_open(&tmp_ac_prefs);
+}
+
+#if USE_GPGME || USE_SSL
+static void prefs_account_enum_set_data_from_radiobtn(PrefParam *pparam)
 {
 	GtkRadioButton *radiobtn;
 	GSList *group;
@@ -1169,7 +1381,7 @@ static void prefs_account_sign_key_set_data_from_radiobtn(PrefParam *pparam)
 	while (group != NULL) {
 		GtkToggleButton *btn = GTK_TOGGLE_BUTTON (group->data);
 		if (gtk_toggle_button_get_active (btn)) {
-			*((SignKeyType *)pparam->data) = GPOINTER_TO_INT
+			*((gint *)pparam->data) = GPOINTER_TO_INT
 				(gtk_object_get_user_data (GTK_OBJECT (btn)));
 			break;
 		}
@@ -1177,13 +1389,13 @@ static void prefs_account_sign_key_set_data_from_radiobtn(PrefParam *pparam)
 	}
 }
 
-static void prefs_account_sign_key_set_radiobtn(PrefParam *pparam)
+static void prefs_account_enum_set_radiobtn(PrefParam *pparam)
 {
 	GtkRadioButton *radiobtn;
 	GSList *group;
 	gpointer data;
 
-	data = GINT_TO_POINTER (*((RecvProtocol *)pparam->data));
+	data = GINT_TO_POINTER (*((gint *)pparam->data));
 	radiobtn = GTK_RADIO_BUTTON (*pparam->widget);
 	group = gtk_radio_button_group (radiobtn);
 	while (group != NULL) {
@@ -1246,6 +1458,7 @@ static void prefs_account_protocol_set_optmenu(PrefParam *pparam)
 static void prefs_account_protocol_activated(GtkMenuItem *menuitem)
 {
 	RecvProtocol protocol;
+	gboolean active;
 
 	protocol = GPOINTER_TO_INT
 		(gtk_object_get_user_data(GTK_OBJECT(menuitem)));
@@ -1256,73 +1469,113 @@ static void prefs_account_protocol_activated(GtkMenuItem *menuitem)
 		gtk_widget_set_sensitive(basic.inbox_entry, FALSE);
 		gtk_widget_show(basic.nntpserv_label);
 		gtk_widget_show(basic.nntpserv_entry);
+		gtk_widget_show(basic.nntpauth_chkbtn);
 		gtk_widget_hide(basic.recvserv_label);
 		gtk_widget_hide(basic.recvserv_entry);
 		gtk_widget_hide(basic.smtpserv_label);
 		gtk_widget_hide(basic.smtpserv_entry);
-		gtk_table_set_row_spacing (GTK_TABLE (basic.serv_table), 3, 0);
-		gtk_widget_set_sensitive(basic.uid_label,  FALSE);
-		gtk_widget_set_sensitive(basic.pass_label, FALSE);
-		gtk_widget_set_sensitive(basic.uid_entry,  FALSE);
-		gtk_widget_set_sensitive(basic.pass_entry, FALSE);
+		active = gtk_toggle_button_get_active
+			(GTK_TOGGLE_BUTTON(basic.nntpauth_chkbtn));
+		gtk_widget_set_sensitive(basic.uid_label,  active);
+		gtk_widget_set_sensitive(basic.pass_label, active);
+		gtk_widget_set_sensitive(basic.uid_entry,  active);
+		gtk_widget_set_sensitive(basic.pass_entry, active);
 		gtk_widget_set_sensitive(receive.pop3_frame, FALSE);
+		gtk_widget_set_sensitive(receive.imap_frame, FALSE);
+#if USE_SSL
+		gtk_widget_set_sensitive(ssl.receive_frame, FALSE);
+		gtk_widget_set_sensitive(ssl.pop_chkbtn, FALSE);
+		gtk_widget_set_sensitive(ssl.imap_chkbtn, FALSE);
+		gtk_widget_set_sensitive(ssl.send_frame, FALSE);
+#endif
+		gtk_widget_hide(advanced.popport_hbox);
+		gtk_widget_hide(advanced.imapport_hbox);
+		gtk_widget_show(advanced.nntpport_hbox);
 		break;
 	case A_LOCAL:
 		gtk_widget_set_sensitive(basic.inbox_label, TRUE);
 		gtk_widget_set_sensitive(basic.inbox_entry, TRUE);
 		gtk_widget_hide(basic.nntpserv_label);
 		gtk_widget_hide(basic.nntpserv_entry);
+		gtk_widget_hide(basic.nntpauth_chkbtn);
 		gtk_widget_set_sensitive(basic.recvserv_label, FALSE);
 		gtk_widget_set_sensitive(basic.recvserv_entry, FALSE);
 		gtk_widget_show(basic.recvserv_label);
 		gtk_widget_show(basic.recvserv_entry);
 		gtk_widget_show(basic.smtpserv_label);
 		gtk_widget_show(basic.smtpserv_entry);
-		gtk_table_set_row_spacing
-			(GTK_TABLE (basic.serv_table), 3, VSPACING_NARROW);
 		gtk_widget_set_sensitive(basic.uid_label,  FALSE);
 		gtk_widget_set_sensitive(basic.pass_label, FALSE);
 		gtk_widget_set_sensitive(basic.uid_entry,  FALSE);
 		gtk_widget_set_sensitive(basic.pass_entry, FALSE);
 		gtk_widget_set_sensitive(receive.pop3_frame, FALSE);
+		gtk_widget_set_sensitive(receive.imap_frame, FALSE);
+#if USE_SSL
+		gtk_widget_set_sensitive(ssl.receive_frame, FALSE);
+		gtk_widget_set_sensitive(ssl.pop_chkbtn, FALSE);
+		gtk_widget_set_sensitive(ssl.imap_chkbtn, FALSE);
+		gtk_widget_set_sensitive(ssl.send_frame, TRUE);
+#endif
+		gtk_widget_hide(advanced.popport_hbox);
+		gtk_widget_hide(advanced.imapport_hbox);
+		gtk_widget_hide(advanced.nntpport_hbox);
 		break;
 	case A_IMAP4:
 		gtk_widget_set_sensitive(basic.inbox_label, TRUE);
 		gtk_widget_set_sensitive(basic.inbox_entry, TRUE);
 		gtk_widget_hide(basic.nntpserv_label);
 		gtk_widget_hide(basic.nntpserv_entry);
+		gtk_widget_hide(basic.nntpauth_chkbtn);
 		gtk_widget_set_sensitive(basic.recvserv_label, TRUE);
 		gtk_widget_set_sensitive(basic.recvserv_entry, TRUE);
 		gtk_widget_show(basic.recvserv_label);
 		gtk_widget_show(basic.recvserv_entry);
 		gtk_widget_show(basic.smtpserv_label);
 		gtk_widget_show(basic.smtpserv_entry);
-		gtk_table_set_row_spacing
-			(GTK_TABLE (basic.serv_table), 3, VSPACING_NARROW);
 		gtk_widget_set_sensitive(basic.uid_label,  TRUE);
 		gtk_widget_set_sensitive(basic.pass_label, TRUE);
 		gtk_widget_set_sensitive(basic.uid_entry,  TRUE);
 		gtk_widget_set_sensitive(basic.pass_entry, TRUE);
 		gtk_widget_set_sensitive(receive.pop3_frame, FALSE);
+		gtk_widget_set_sensitive(receive.imap_frame, TRUE);
+#if USE_SSL
+		gtk_widget_set_sensitive(ssl.receive_frame, TRUE);
+		gtk_widget_set_sensitive(ssl.pop_chkbtn, FALSE);
+		gtk_widget_set_sensitive(ssl.imap_chkbtn, TRUE);
+		gtk_widget_set_sensitive(ssl.send_frame, TRUE);
+#endif
+		gtk_widget_hide(advanced.popport_hbox);
+		gtk_widget_show(advanced.imapport_hbox);
+		gtk_widget_hide(advanced.nntpport_hbox);
 		break;
+	case A_POP3:
 	default:
 		gtk_widget_set_sensitive(basic.inbox_label, TRUE);
 		gtk_widget_set_sensitive(basic.inbox_entry, TRUE);
 		gtk_widget_hide(basic.nntpserv_label);
 		gtk_widget_hide(basic.nntpserv_entry);
+		gtk_widget_hide(basic.nntpauth_chkbtn);
 		gtk_widget_set_sensitive(basic.recvserv_label, TRUE);
 		gtk_widget_set_sensitive(basic.recvserv_entry, TRUE);
 		gtk_widget_show(basic.recvserv_label);
 		gtk_widget_show(basic.recvserv_entry);
 		gtk_widget_show(basic.smtpserv_label);
 		gtk_widget_show(basic.smtpserv_entry);
-		gtk_table_set_row_spacing
-			(GTK_TABLE (basic.serv_table), 3, VSPACING_NARROW);
 		gtk_widget_set_sensitive(basic.uid_label,  TRUE);
 		gtk_widget_set_sensitive(basic.pass_label, TRUE);
 		gtk_widget_set_sensitive(basic.uid_entry,  TRUE);
 		gtk_widget_set_sensitive(basic.pass_entry, TRUE);
 		gtk_widget_set_sensitive(receive.pop3_frame, TRUE);
+		gtk_widget_set_sensitive(receive.imap_frame, FALSE);
+#if USE_SSL
+		gtk_widget_set_sensitive(ssl.receive_frame, TRUE);
+		gtk_widget_set_sensitive(ssl.pop_chkbtn, TRUE);
+		gtk_widget_set_sensitive(ssl.imap_chkbtn, FALSE);
+		gtk_widget_set_sensitive(ssl.send_frame, TRUE);
+#endif
+		gtk_widget_show(advanced.popport_hbox);
+		gtk_widget_hide(advanced.imapport_hbox);
+		gtk_widget_hide(advanced.nntpport_hbox);
 		break;
 	}
 
