@@ -32,14 +32,21 @@
 
 typedef struct _PrefsCommon	PrefsCommon;
 
+typedef enum {
+	RECV_DIALOG_ALWAYS,
+	RECV_DIALOG_ACTIVE,
+	RECV_DIALOG_NEVER
+} RecvDialogMode;
+
 struct _PrefsCommon
 {
 	/* Receive */
 	gboolean use_extinc;
-	gchar *extinc_path;
+	gchar *extinc_cmd;
 	gboolean inc_local;
 	gboolean filter_on_inc;
 	gchar *spool_path;
+	gboolean scan_all_after_inc;
 	gboolean autochk_newmail;
 	gint autochk_itv;
 	gboolean chk_on_startup;
@@ -47,7 +54,7 @@ struct _PrefsCommon
 
 	/* Send */
 	gboolean use_extsend;
-	gchar *extsend_path;
+	gchar *extsend_cmd;
 	gboolean savemsg;
 	gboolean queue_msg;
 	gchar *outgoing_charset;
@@ -61,17 +68,24 @@ struct _PrefsCommon
 	gint linewrap_len;
 	gboolean linewrap_quote;
 	gboolean linewrap_at_send;
+	gboolean auto_exteditor;
+	gboolean reply_account_autosel;
 	gboolean show_ruler;
 
 	/* Display */
 	gchar *widgetfont;
 	gchar *textfont;
+	gchar *normalfont;
+	gchar *boldfont;
+	gchar *smallfont;
 	gboolean display_folder_unread;
 	ToolbarStyle toolbar_style;
 	gboolean show_statusbar;
 	gboolean trans_hdr;
 	gboolean enable_thread;
 	gboolean enable_hscrollbar;
+	gboolean expand_thread;
+	gboolean bold_unread;
 	gboolean swap_from;
 	gchar *date_format;
 
@@ -152,9 +166,11 @@ struct _PrefsCommon
 	/* Privacy */
 	gboolean default_encrypt;
 	gboolean default_sign;
-        gboolean auto_check_signatures;
+	gboolean auto_check_signatures;
+	gboolean gpg_signature_popup;
 	gboolean passphrase_grab;
 	gchar *default_signkey;
+	gboolean gpg_warning;
 
 	/* Interface */
 	gboolean sep_folder;
@@ -163,6 +179,7 @@ struct _PrefsCommon
 	gboolean open_unread_on_enter;
 	gboolean open_inbox_on_inc;
 	gboolean immediate_exec;
+	RecvDialogMode recv_dialog_mode;
 	gboolean add_address_by_click;
 
 	gboolean confirm_on_exit;
