@@ -81,11 +81,6 @@ static struct Filter {
    pointer to the function for widget setting
  */
 
-#define VSPACING		12
-#define VSPACING_NARROW		4
-#define DEFAULT_ENTRY_WIDTH	80
-#define PREFSBUFSIZE		1024
-
 /* widget creating functions */
 static void prefs_filter_create		(void);
 
@@ -254,8 +249,8 @@ static void prefs_filter_create(void)
 	gtk_widget_set_usize (hdr_combo1, 96, -1);
 	gtkut_combo_set_items (GTK_COMBO (hdr_combo1),
 			       "Subject", "From", "To", "Cc", "Reply-To",
-			       "Sender", "X-ML-Name", "X-List", "X-Sequence",
-			       "X-Mailer",
+			       "Sender", "List-Id",
+			       "X-ML-Name", "X-List", "X-Sequence", "X-Mailer",
 			       NULL);
 
 	hdr_combo2 = gtk_combo_new ();
@@ -265,8 +260,8 @@ static void prefs_filter_create(void)
 	gtk_widget_set_usize (hdr_combo2, 96, -1);
 	gtkut_combo_set_items (GTK_COMBO (hdr_combo2), _("(none)"),
 			       "Subject", "From", "To", "Cc", "Reply-To",
-			       "Sender", "X-ML-Name", "X-List", "X-Sequence",
-			       "X-Mailer",
+			       "Sender", "List-Id",
+			       "X-ML-Name", "X-List", "X-Sequence", "X-Mailer",
 			       NULL);
 
 	key_label = gtk_label_new (_("Keyword"));
@@ -678,7 +673,7 @@ static void prefs_filter_select_dest_cb(void)
 {
 	FolderItem *dest;
 
-	dest = foldersel_folder_sel(NULL);
+	dest = foldersel_folder_sel(NULL, NULL);
 	if (!dest) return;
 
 	gtk_entry_set_text(GTK_ENTRY(filter.dest_entry), dest->path);
@@ -718,7 +713,7 @@ static void prefs_filter_delete_cb(void)
 
 	if (alertpanel(_("Delete rule"),
 		       _("Do you really want to delete this rule?"),
-		       _("Yes"), _("No"), NULL) == G_ALERTALTERNATE)
+		       _("Yes"), _("No"), NULL) != G_ALERTDEFAULT)
 		return;
 
 	flt = gtk_clist_get_row_data(clist, row);
