@@ -414,7 +414,7 @@ MsgInfo *procheader_parse(const gchar *file, MsgFlags flags, gboolean full)
 	}
 
 	msginfo = g_new0(MsgInfo, 1);
-	msginfo->flags = flags != 0 ? flags : MSG_NEW|MSG_UNREAD;
+	msginfo->flags = flags;
 	msginfo->inreplyto = NULL;
 
 	while ((hnum = procheader_get_one_field(buf, sizeof(buf), fp, hentry))
@@ -489,11 +489,11 @@ MsgInfo *procheader_parse(const gchar *file, MsgFlags flags, gboolean full)
 			break;
 		case H_CONTENT_TYPE:
 			if (!strncasecmp(hp, "multipart", 9))
-				msginfo->flags |= MSG_MIME;
+				MSG_SET_TMP_FLAGS(msginfo->flags, MSG_MIME);
 			break;
 		case H_SEEN:
 			/* mnews Seen header */
-			MSG_UNSET_FLAGS(msginfo->flags, MSG_NEW|MSG_UNREAD);
+			MSG_UNSET_PERM_FLAGS(msginfo->flags, MSG_NEW|MSG_UNREAD);
 			break;
 		case H_X_FACE:
 			if (msginfo->xface) break;
