@@ -20,7 +20,8 @@
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
 #endif
-#ifdef USE_GPGME
+
+#if USE_GPGME
 
 #include "defs.h"
 
@@ -279,10 +280,11 @@ static void check_signature (MimeInfo *mimeinfo, MimeInfo *partinfo, FILE *fp)
     GpgmeError err;
     GpgmeData sig = NULL, text = NULL;
     GpgmeSigStat status = GPGME_SIG_STAT_NONE;
-    GpgmegtkSigStatus statuswindow;
+    GpgmegtkSigStatus statuswindow = NULL;
     const char *result = NULL;
 
-    statuswindow = gpgmegtk_sig_status_create ();
+    if (prefs_common.gpg_signature_popup)
+	statuswindow = gpgmegtk_sig_status_create ();
 
     err = gpgme_new (&ctx);
     if (err) {
@@ -317,7 +319,8 @@ static void check_signature (MimeInfo *mimeinfo, MimeInfo *partinfo, FILE *fp)
 leave:
     result = gpgmegtk_sig_status_to_string(status);
     debug_print("verification status: %s\n", result);
-    gpgmegtk_sig_status_update(statuswindow,ctx);
+    if (prefs_common.gpg_signature_popup)
+	gpgmegtk_sig_status_update (statuswindow,ctx);
 
     g_assert (!err); /* FIXME: Hey: this may indeed happen */
     g_free (partinfo->sigstatus);
@@ -326,7 +329,8 @@ leave:
     gpgme_data_release (sig);
     gpgme_data_release (text);
     gpgme_release (ctx);
-    gpgmegtk_sig_status_destroy(statuswindow);
+    if (prefs_common.gpg_signature_popup)
+	gpgmegtk_sig_status_destroy (statuswindow);
 }
 
 static const char *
@@ -496,7 +500,7 @@ int rfc2015_is_encrypted (MimeInfo *mimeinfo)
 {
     if (!mimeinfo)
         return 0;
-    if (strcasecmp (mimeinfo->content_type, "multipart/encrypted"))
+    if (g_strcasecmp (mimeinfo->content_type, "multipart/encrypted"))
         return 0;
     /* fixme: we should schek the protocol parameter */
     return 1;
@@ -571,13 +575,13 @@ void rfc2015_decrypt_message (MsgInfo *msginfo, MimeInfo *mimeinfo, FILE *fp)
     /* check that we have the 2 parts */
     n = found = 0;
     for (partinfo = mimeinfo->children; partinfo; partinfo = partinfo->next) {
-        if (++n == 1 && !strcmp (partinfo->content_type,
-				 "application/pgp-encrypted")) {
+        if (++n == 1 && !g_strcasecmp (partinfo->content_type,
+				       "application/pgp-encrypted")) {
             /* Fixme: check that the version is 1 */
             ver_okay = 1;
         }
-        else if (n == 2 && !strcmp (partinfo->content_type,
-				    "application/octet-stream")) {
+        else if (n == 2 && !g_strcasecmp (partinfo->content_type,
+					  "application/octet-stream")) {
             if (partinfo->next)
                 g_warning ("** oops: pgp_encrypted with more than 2 parts");
             break;
