@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999,2000 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2001 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,15 +31,11 @@
 static gint verbose = 1;
 static gchar smtp_response[MSGBUFSIZE];
 
-gint smtp_helo(SockInfo *sock, const char *hostname, gboolean use_smtp_auth)
+gint smtp_helo(SockInfo *sock, const gchar *hostname, gboolean esmtp)
 {
-	if (use_smtp_auth) {
-		sock_printf(sock, "EHLO %s\r\n", hostname);
-		if (verbose)
-			log_print("ESMTP> EHLO %s\n", hostname);
-
-		return esmtp_ok(sock);
-	} else {
+	if (esmtp)
+		return esmtp_ehlo(sock, hostname);
+	else {
 		sock_printf(sock, "HELO %s\r\n", hostname);
 		if (verbose)
 			log_print("SMTP> HELO %s\n", hostname);
@@ -66,7 +62,7 @@ gint smtp_from(SockInfo *sock, const gchar *from,
 		} else
 			authtype = SMTPAUTH_CRAM_MD5;
 
-		esmtp_auth(sock, authtype, userid, passwd, use_smtp_auth);
+		esmtp_auth(sock, authtype, userid, passwd);
 	}
 
 	if (strchr(from, '<'))
@@ -136,7 +132,7 @@ gint smtp_eom(SockInfo *sock)
 gint smtp_ok(SockInfo *sock)
 {
 	while ((sock_gets(sock, smtp_response, sizeof(smtp_response) - 1))
-	       != 1) {
+	       != -1) {
 		if (strlen(smtp_response) < 4)
 			return SM_ERROR;
 		strretchomp(smtp_response);
@@ -145,7 +141,8 @@ gint smtp_ok(SockInfo *sock)
 			log_print("SMTP< %s\n", smtp_response);
 
 		if ((smtp_response[0] == '1' || smtp_response[0] == '2' ||
-		     smtp_response[0] == '3') && smtp_response[3] == ' ')
+		     smtp_response[0] == '3') &&
+		     (smtp_response[3] == ' ' || smtp_response[3] == '\0'))
 			return SM_OK;
 		else if (smtp_response[3] != '-')
 			return SM_ERROR;
