@@ -58,7 +58,12 @@ typedef enum
 	SORT_BY_SIZE,
 	SORT_BY_DATE,
 	SORT_BY_FROM,
-	SORT_BY_SUBJECT
+	SORT_BY_SUBJECT,
+	SORT_BY_SCORE,
+	SORT_BY_LABEL,
+	SORT_BY_MARK,
+	SORT_BY_UNREAD,
+	SORT_BY_MIME
 } SummarySortType;
 
 typedef enum
@@ -89,6 +94,7 @@ struct _SummaryView
 	GtkWidget *toggle_eventbox;
 	GtkWidget *toggle_arrow;
 	GtkWidget *popupmenu;
+	GtkWidget *colorlabel_menu;
 
 	GtkItemFactory *popupfactory;
 
@@ -102,7 +108,6 @@ struct _SummaryView
 
 	GdkColor color_marked;
 	GdkColor color_dim;
-	GdkColor color_normal;
 
 	MainWindow   *mainwin;
 	FolderView   *folderview;
@@ -146,11 +151,18 @@ gboolean summary_show		  (SummaryView		*summaryview,
 void summary_clear_list		  (SummaryView		*summaryview);
 void summary_clear_all		  (SummaryView		*summaryview);
 
+SummarySelection summary_get_selection_type	(SummaryView	*summaryview);
+
+void summary_select_prev_unread	  (SummaryView		*summaryview);
 void summary_select_next_unread	  (SummaryView		*summaryview);
+void summary_select_prev_marked	  (SummaryView		*summaryview);
+void summary_select_next_marked	  (SummaryView		*summaryview);
+void summary_select_prev_labeled  (SummaryView		*summaryview);
+void summary_select_next_labeled  (SummaryView		*summaryview);
 void summary_select_by_msgnum	  (SummaryView		*summaryview,
 				   guint		 msgnum);
-guint summary_get_current_msgnum  (SummaryView		*summaryview);
-void summary_thread_build	  (SummaryView		*summaryview);
+void summary_thread_build	  (SummaryView		*summaryview,
+				   gboolean		 init);
 void summary_unthread		  (SummaryView		*summaryview);
 void summary_filter		  (SummaryView		*summaryview);
 void summary_sort		  (SummaryView		*summaryview,
@@ -184,7 +196,16 @@ void summary_mark		  (SummaryView		*summaryview);
 void summary_unmark		  (SummaryView		*summaryview);
 void summary_mark_as_unread	  (SummaryView		*summaryview);
 void summary_mark_as_read	  (SummaryView		*summaryview);
+void summary_mark_all_read	  (SummaryView		*summaryview);
+void summary_add_address	  (SummaryView		*summaryview);
 void summary_select_all		  (SummaryView		*summaryview);
 void summary_unselect_all	  (SummaryView		*summaryview);
 
+void summary_set_colorlabel	  (SummaryView		*summaryview,
+				   guint		 labelcolor,
+				   GtkWidget		*widget);
+void summary_set_colorlabel_color (GtkCTree		*ctree,
+				   GtkCTreeNode		*node,
+				   guint		 labelcolor);
+
 #endif /* __SUMMARY_H__ */
