@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999,2000 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2001 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -150,6 +150,8 @@ TextView *textview_create(void)
 	/* create GtkText widgets for single-byte and multi-byte character */
 	text_sb = gtk_text_new(NULL, NULL);
 	text_mb = gtk_text_new(NULL, NULL);
+	GTK_TEXT(text_sb)->default_tab_width = 8;
+	GTK_TEXT(text_mb)->default_tab_width = 8;
 	gtk_widget_show(text_sb);
 	gtk_widget_show(text_mb);
 	gtk_text_set_word_wrap(GTK_TEXT(text_sb), TRUE);
@@ -161,7 +163,7 @@ TextView *textview_create(void)
 
 		style = gtk_style_copy(text_sb->style);
 		gdk_font_unref(style->font);
-		FONT_LOAD(style->font, NORMAL_FONT);
+		FONT_LOAD(style->font, prefs_common.normalfont);
 		gtk_widget_set_style(text_sb, style);
 	}
 	if (text_mb->style && text_mb->style->font->type == GDK_FONT_FONT) {
@@ -169,7 +171,7 @@ TextView *textview_create(void)
 
 		style = gtk_style_copy(text_mb->style);
 		gdk_font_unref(style->font);
-		style->font = gdk_fontset_load(NORMAL_FONT);
+		style->font = gdk_fontset_load(prefs_common.normalfont);
 		gtk_widget_set_style(text_mb, style);
 	}
 	gtk_widget_ref(scrolledwin_sb);
@@ -306,11 +308,13 @@ void textview_show_part(TextView *textview, MimeInfo *mimeinfo, FILE *fp)
 
 	/* display attached RFC822 single text message */
 	if (mimeinfo->parent && mimeinfo->mime_type == MIME_MESSAGE_RFC822) {
+		if (headers) procheader_header_array_destroy(headers);
 		if (!mimeinfo->sub || mimeinfo->sub->children) return;
 		headers = textview_scan_header(textview, fp);
 		mimeinfo = mimeinfo->sub;
 	} else if (!mimeinfo->parent &&
 		   mimeinfo->mime_type == MIME_MESSAGE_RFC822) {
+		if (headers) procheader_header_array_destroy(headers);
 		if (!mimeinfo->sub) return;
 		headers = textview_scan_header(textview, fp);
 		mimeinfo = mimeinfo->sub;
@@ -348,6 +352,9 @@ void textview_show_part(TextView *textview, MimeInfo *mimeinfo, FILE *fp)
 	gtk_text_thaw(text);
 }
 
+#define TEXT_INSERT(str) \
+	gtk_text_insert(text, textview->msgfont, NULL, NULL, str, -1)
+
 void textview_show_mime_part(TextView *textview, MimeInfo *partinfo)
 {
 	GtkText *text;
@@ -360,26 +367,17 @@ void textview_show_mime_part(TextView *textview, MimeInfo *partinfo)
 
 	gtk_text_freeze(text);
 
-	gtk_text_insert
-		(text, textview->msgfont, NULL, NULL,
-		 _("To save this part, pop up the context menu with\n"), -1);
-	gtk_text_insert
-		(text, textview->msgfont, NULL, NULL,
-		 _("right click and select `Save as...', or press `y' key.\n\n"), -1);
+	TEXT_INSERT(_("To save this part, pop up the context menu with "));
+	TEXT_INSERT(_("right click and select `Save as...', "));
+	TEXT_INSERT(_("or press `y' key.\n\n"));
 
-	gtk_text_insert
-		(text, textview->msgfont, NULL, NULL,
-		 _("To display this part as a text message, select\n"), -1);
-	gtk_text_insert
-		(text, textview->msgfont, NULL, NULL,
-		 _("`Display as text', or press `t' key.\n\n"), -1);
+	TEXT_INSERT(_("To display this part as a text message, select "));
+	TEXT_INSERT(_("`Display as text', or press `t' key.\n\n"));
 
-	gtk_text_insert
-		(text, textview->msgfont, NULL, NULL,
-		 _("To open this part with external program, select `Open',\n"), -1);
-	gtk_text_insert
-		(text, textview->msgfont, NULL, NULL,
-		 _("or double-click, or click the center button, or press `l' key."), -1);
+	TEXT_INSERT(_("To open this part with external program, select "));
+	TEXT_INSERT(_("`Open' or `Open with...', "));
+	TEXT_INSERT(_("or double-click, or click the center button, "));
+	TEXT_INSERT(_("or press `l' key."));
 
 	gtk_text_thaw(text);
 }
@@ -398,24 +396,19 @@ void textview_show_signature_part(TextView *textview, MimeInfo *partinfo)
 	gtk_text_freeze(text);
 
 	if (partinfo->sigstatus_full == NULL) {
-		gtk_text_insert
-			(text, textview->msgfont, NULL, NULL,
-			 _("This signature has not been checked yet.\n"), -1);
-		gtk_text_insert
-			(text, textview->msgfont, NULL, NULL,
-			 _("To check it, pop up the context menu with\n"), -1);
-		gtk_text_insert
-			(text, textview->msgfont, NULL, NULL,
-			 _("right click and select `Check signature'.\n"), -1);
+		TEXT_INSERT(_("This signature has not been checked yet.\n"));
+		TEXT_INSERT(_("To check it, pop up the context menu with\n"));
+		TEXT_INSERT(_("right click and select `Check signature'.\n"));
 	} else {
-		gtk_text_insert(text, textview->msgfont, NULL, NULL,
-				partinfo->sigstatus_full, -1);
+		TEXT_INSERT(partinfo->sigstatus_full);
 	}
 
 	gtk_text_thaw(text);
 }
 #endif /* USE_GPGME */
 
+#undef TEXT_INSERT
+
 static void textview_show_html(TextView *textview, FILE *fp,
 			       CodeConverter *conv)
 {
@@ -721,8 +714,8 @@ static void textview_write_line(TextView *textview, const gchar *str,
 		textview_make_clickable_parts(textview, textview->msgfont,
 					      fg_color, &uri_color, buf);
 	else
-		gtk_text_insert(text, textview->msgfont, fg_color, NULL,
-				buf, -1);
+		textview_make_clickable_parts(textview, textview->msgfont,
+					      fg_color, NULL, buf);
 }
 
 void textview_clear(TextView *textview)
@@ -837,7 +830,7 @@ void textview_set_font(TextView *textview, const gchar *codeset)
 	}
 
 	if (!textview->boldfont)
-		FONT_LOAD(textview->boldfont, BOLD_FONT);
+		FONT_LOAD(textview->boldfont, prefs_common.boldfont);
 	if (!spacingfont)
 		spacingfont = gdk_font_load("-*-*-medium-r-normal--6-*");
 }
@@ -910,7 +903,7 @@ static void textview_show_header(TextView *textview, GPtrArray *headers)
 
 		gtk_text_insert(text, textview->boldfont, NULL, NULL,
 				header->name, -1);
-		gtk_text_insert(text, textview->boldfont, NULL, NULL, ":", 2);
+		gtk_text_insert(text, textview->boldfont, NULL, NULL, ":", 1);
 
 		if (!g_strcasecmp(header->name, "Subject") ||
 		    !g_strcasecmp(header->name, "From")    ||
@@ -929,8 +922,9 @@ static void textview_show_header(TextView *textview, GPtrArray *headers)
 						      NULL, NULL, &uri_color,
 						      header->body);
 		} else {
-			gtk_text_insert(text, NULL, NULL, NULL,
-					header->body, -1);
+			textview_make_clickable_parts(textview,
+						      NULL, NULL, NULL,
+						      header->body);
 		}
 		gtk_text_insert(text, textview->msgfont, NULL, NULL, "\n", 1);
 	}
