@@ -81,6 +81,17 @@ void hash_free_strings(GHashTable *table)
 	g_hash_table_foreach(table, hash_free_strings_func, NULL);
 }
 
+static void hash_free_value_mem_func(gpointer key, gpointer value,
+				     gpointer data)
+{
+	g_free(value);
+}
+
+void hash_free_value_mem(GHashTable *table)
+{
+	g_hash_table_foreach(table, hash_free_value_mem_func, NULL);
+}
+
 void ptr_array_free_strings(GPtrArray *array)
 {
 	gint i;
@@ -106,18 +117,25 @@ gint to_number(const gchar *nstr)
 	return atoi(nstr);
 }
 
+/* convert integer into string,
+   nstr must be not lower than 11 characters length */
+gchar *itos_buf(gchar *nstr, gint n)
+{
+	g_snprintf(nstr, 11, "%d", n);
+	return nstr;
+}
+
 /* convert integer into string */
 gchar *itos(gint n)
 {
 	static gchar nstr[11];
 
-	g_snprintf(nstr, 11, "%d", n);
-	return nstr;
+	return itos_buf(nstr, n);
 }
 
 gchar *to_human_readable(off_t size)
 {
-	static gchar str[9];
+	static gchar str[10];
 	gint count;
 	guint32 div = 1;
 
@@ -132,10 +150,10 @@ gchar *to_human_readable(off_t size)
 	case 0: g_snprintf(str, sizeof(str), "%dB",    (gint)size);   break;
 	case 1: g_snprintf(str, sizeof(str), "%.1fKB", (gfloat)size / div);
 		break;
-	case 2: g_snprintf(str, sizeof(str), "%.1fMB", (gfloat)size / div);
+	case 2: g_snprintf(str, sizeof(str), "%.2fMB", (gfloat)size / div);
 		break;
 	default:
-		g_snprintf(str, sizeof(str), "%.1fGB", (gfloat)size / div);
+		g_snprintf(str, sizeof(str), "%.2fGB", (gfloat)size / div);
 		break;
 	}
 
@@ -183,6 +201,21 @@ gchar *strretchomp(gchar *str)
 	return str;
 }
 
+/* remove trailing character */
+gchar *strtailchomp(gchar *str, gchar tail_char)
+{
+	register gchar *s;
+
+	if (!*str) return str;
+	if (tail_char == '\0') return str;
+
+	for (s = str + strlen(str) - 1; s >= str && *s == tail_char; s--)
+		*s = '\0';
+
+	return str;
+}
+
+
 /* Similar to `strstr' but this function ignores the case of both strings.  */
 gchar *strcasestr(const gchar *haystack, const gchar *needle)
 {
@@ -551,6 +584,35 @@ void extract_parenthesis(gchar *str, gchar op, gchar cl)
 	*destp = '\0';
 }
 
+void extract_one_parenthesis_with_skip_quote(gchar *str, gchar quote_chr,
+					     gchar op, gchar cl)
+{
+	register gchar *srcp, *destp;
+	gint in_brace;
+	gboolean in_quote = FALSE;
+
+	srcp = destp = str;
+
+	if ((srcp = strchr_with_skip_quote(destp, quote_chr, op))) {
+		memmove(destp, srcp + 1, strlen(srcp));
+		in_brace = 1;
+		while(*destp) {
+			if (*destp == op && !in_quote)
+				in_brace++;
+			else if (*destp == cl && !in_quote)
+				in_brace--;
+			else if (*destp == quote_chr)
+				in_quote ^= TRUE;
+
+			if (in_brace == 0)
+				break;
+
+			destp++;
+		}
+	}
+	*destp = '\0';
+}
+
 void extract_parenthesis_with_skip_quote(gchar *str, gchar quote_chr,
 					 gchar op, gchar cl)
 {
@@ -560,7 +622,7 @@ void extract_parenthesis_with_skip_quote(gchar *str, gchar quote_chr,
 
 	srcp = destp = str;
 
-	while ((srcp = strchr_with_skip_quote(destp, '"', op))) {
+	while ((srcp = strchr_with_skip_quote(destp, quote_chr, op))) {
 		if (destp > str)
 			*destp++ = ' ';
 		memmove(destp, srcp + 1, strlen(srcp));
@@ -874,55 +936,45 @@ gboolean is_ascii_str(const guchar *str)
 
 gint get_quote_level(const gchar *str)
 {
-	size_t firstquotepos;
-	size_t lastquotepos = -1;
+	const gchar *first_pos;
+	const gchar *last_pos;
 	const gchar *p = str;
-	const gchar *pos;
-	gint quotelevel = -1;
-	gint i = 0;
+	gint quote_level = -1;
 
 	/* speed up line processing by only searching to the last '>' */
-	if ((pos = strchr(str, '>')) != NULL) {
-		firstquotepos = pos - str;
-		lastquotepos = strrchr(str, '>') - str + 1;
-
+	if ((first_pos = strchr(str, '>')) != NULL) {
 		/* skip a line if it contains a '<' before the initial '>' */
-		if (memchr(str, '<', pos - str) != NULL)
+		if (memchr(str, '<', first_pos - str) != NULL)
 			return -1;
+		last_pos = strrchr(first_pos, '>');
 	} else
 		return -1;
 
-	while (i < lastquotepos) {
-		while (i < lastquotepos) {
-			if (isspace(*p) || (*p == '\t')) {
+	while (p <= last_pos) {
+		while (p < last_pos) {
+			if (isspace(*p))
 				p++;
-				i++;
-			} else
+			else
 				break;
 		}
-		if (i >= lastquotepos)
-			break;
 
 		if (*p == '>')
-			quotelevel++;
-		else if ((*p != '-') && !isspace(*p) && (i < lastquotepos)) {
+			quote_level++;
+		else if (*p != '-' && !isspace(*p) && p <= last_pos) {
 			/* any characters are allowed except '-' and space */
-			while ((*p != '-') && (*p != '>') && !isspace(*p) &&
-			       (i < lastquotepos)) {
+			while (*p != '-' && *p != '>' && !isspace(*p) &&
+			       p < last_pos)
 				p++;
-				i++;
-			}
 			if (*p == '>')
-				quotelevel++;
-			else if ((i >= lastquotepos) || isspace(*p))
+				quote_level++;
+			else
 				break;
 		}
 
 		p++;
-		i++;
 	}
 
-	return quotelevel;
+	return quote_level;
 }
 
 GList *uri_list_extract_filenames(const gchar *uri_list)
@@ -1162,6 +1214,33 @@ off_t get_file_size(const gchar *file)
 	return s.st_size;
 }
 
+off_t get_left_file_size(FILE *fp)
+{
+	glong pos;
+	glong end;
+	off_t size;
+
+	if ((pos = ftell(fp)) < 0) {
+		perror("ftell");
+		return -1;
+	}
+	if (fseek(fp, 0L, SEEK_END) < 0) {
+		perror("fseek");
+		return -1;
+	}
+	if ((end = ftell(fp)) < 0) {
+		perror("fseek");
+		return -1;
+	}
+	size = end - pos;
+	if (fseek(fp, pos, SEEK_SET) < 0) {
+		perror("fseek");
+		return -1;
+	}
+
+	return size;
+}
+
 gboolean file_exist(const gchar *file, gboolean allow_fifo)
 {
 	struct stat s;
@@ -1288,6 +1367,51 @@ gint remove_all_files(const gchar *dir)
 	return 0;
 }
 
+gint remove_numbered_files(const gchar *dir, guint first, guint last)
+{
+	DIR *dp;
+	struct dirent *d;
+	gchar *prev_dir;
+	gint fileno;
+
+	prev_dir = g_get_current_dir();
+
+	if (chdir(dir) < 0) {
+		FILE_OP_ERROR(dir, "chdir");
+		return -1;
+	}
+
+	if ((dp = opendir(".")) == NULL) {
+		FILE_OP_ERROR(dir, "opendir");
+		return -1;
+	}
+
+	while ((d = readdir(dp)) != NULL) {
+		fileno = to_number(d->d_name);
+		if (fileno >= 0 && first <= fileno && fileno <= last) {
+			if (unlink(d->d_name) < 0)
+				FILE_OP_ERROR(d->d_name, "unlink");
+		}
+	}
+
+	closedir(dp);
+
+	if (chdir(prev_dir) < 0) {
+		FILE_OP_ERROR(prev_dir, "chdir");
+		g_free(prev_dir);
+		return -1;
+	}
+
+	g_free(prev_dir);
+
+	return 0;
+}
+
+gint remove_all_numbered_files(const gchar *dir)
+{
+	return remove_numbered_files(dir, 0, UINT_MAX);
+}
+
 gint remove_dir_recursive(const gchar *dir)
 {
 	struct stat s;
@@ -1379,84 +1503,6 @@ gint remove_dir_recursive(const gchar *dir)
 	return 0;
 }
 
-#if 0
-/* this seems to be slower than the stdio version... */
-gint copy_file(const gchar *src, const gchar *dest)
-{
-	gint src_fd, dest_fd;
-	gint n_read;
-	gint n_write;
-	gchar buf[BUFSIZ];
-	gchar *dest_bak = NULL;
-
-	if ((src_fd = open(src, O_RDONLY)) < 0) {
-		FILE_OP_ERROR(src, "open");
-		return -1;
-	}
-
-	if (is_file_exist(dest)) {
-		dest_bak = g_strconcat(dest, ".bak", NULL);
-		if (rename(dest, dest_bak) < 0) {
-			FILE_OP_ERROR(dest, "rename");
-			close(src_fd);
-			g_free(dest_bak);
-			return -1;
-		}
-	}
-
-	if ((dest_fd = open(dest, O_RDWR|O_CREAT, S_IRUSR|S_IWUSR)) < 0) {
-		FILE_OP_ERROR(dest, "open");
-		close(src_fd);
-		if (dest_bak) {
-			if (rename(dest_bak, dest) < 0)
-				FILE_OP_ERROR(dest_bak, "rename");
-			g_free(dest_bak);
-		}
-		return -1;
-	}
-
-	while ((n_read = read(src_fd, buf, sizeof(buf))) > 0) {
-		gint len = n_read;
-		gchar *bufp = buf;
-
-		while (len > 0) {
-			n_write = write(dest_fd, bufp, len);
-			if (n_write <= 0) {
-				g_warning(_("writing to %s failed.\n"), dest);
-				close(dest_fd);
-				close(src_fd);
-				unlink(dest);
-				if (dest_bak) {
-					if (rename(dest_bak, dest) < 0)
-						FILE_OP_ERROR(dest_bak, "rename");
-					g_free(dest_bak);
-				}
-				return -1;
-			}
-			len -= n_write;
-			bufp += n_write;
-		}
-	}
-
-	close(src_fd);
-	close(dest_fd);
-
-	if (n_read < 0 || get_file_size(src) != get_file_size(dest)) {
-		g_warning(_("File copy from %s to %s failed.\n"), src, dest);
-		unlink(dest);
-		if (dest_bak) {
-			if (rename(dest_bak, dest) < 0)
-				FILE_OP_ERROR(dest_bak, "rename");
-			g_free(dest_bak);
-		}
-		return -1;
-	}
-	g_free(dest_bak);
-
-	return 0;
-}
-#endif
-
 gint copy_file(const gchar *src, const gchar *dest)
 {
 	FILE *src_fp, *dest_fp;
@@ -1537,6 +1583,27 @@ gint copy_file(const gchar *src, const gchar *dest)
 	return 0;
 }
 
+gint move_file(const gchar *src, const gchar *dest)
+{
+	if (is_file_exist(dest)) {
+		g_warning(_("move_file(): file %s already exists."), dest);
+		return -1;
+	}
+
+	if (rename(src, dest) == 0) return 0;
+
+	if (EXDEV != errno) {
+		FILE_OP_ERROR(src, "rename");
+		return -1;
+	}
+
+	if (copy_file(src, dest) < 0) return -1;
+
+	unlink(src);
+
+	return 0;
+}
+
 gint change_file_mode_rw(FILE *fp, const gchar *file)
 {
 #if HAVE_FCHMOD
@@ -1618,7 +1685,28 @@ gint execute_async(gchar *const argv[])
 	return 0;
 }
 
-gint execute_command_line(const gchar *cmdline)
+gint execute_sync(gchar *const argv[])
+{
+	pid_t pid;
+
+	if ((pid = fork()) < 0) {
+		perror("fork");
+		return -1;
+	}
+
+	if (pid == 0) {		/* child process */
+		execvp(argv[0], argv);
+
+		perror("execvp");
+		_exit(1);
+	}
+
+	waitpid(pid, NULL, 0);
+
+	return 0;
+}
+
+gint execute_command_line(const gchar *cmdline, gboolean async)
 {
 	gchar **argv;
 	gint i;
@@ -1640,7 +1728,10 @@ gint execute_command_line(const gchar *cmdline)
 		}
 	}
 
-	ret = execute_async(argv);
+	if (async)
+		ret = execute_async(argv);
+	else
+		ret = execute_sync(argv);
 	g_strfreev(argv);
 
 	return ret;
@@ -1665,7 +1756,7 @@ gint open_uri(const gchar *uri, const gchar *cmdline)
 		g_snprintf(buf, sizeof(buf), default_cmdline, uri);
 	}
 
-	execute_command_line(buf);
+	execute_command_line(buf, TRUE);
 
 	return 0;
 }
@@ -1814,6 +1905,34 @@ void get_rfc822_date(gchar *buf, gint len)
 		   day, dd, mon, yyyy, hh, mm, ss, tzoffset(&t));
 }
 
+static FILE *log_fp = NULL;
+
+void set_log_file(const gchar *filename)
+{
+	if (log_fp) return;
+	log_fp = fopen(filename, "w");
+	if (!log_fp)
+		FILE_OP_ERROR(filename, "fopen");
+}
+
+void close_log_file(void)
+{
+	if (log_fp) {
+		fclose(log_fp);
+		log_fp = NULL;
+	}
+}
+
+static guint log_verbosity_count = 0;
+
+void log_verbosity_set(gboolean verbose)
+{
+	if (verbose)
+		log_verbosity_count++;
+	else if (log_verbosity_count > 0)
+		log_verbosity_count--;
+}
+
 void debug_print(const gchar *format, ...)
 {
 	va_list args;
@@ -1839,7 +1958,12 @@ void log_print(const gchar *format, ...)
 
 	if (debug_mode) fputs(buf, stdout);
 	log_window_append(buf, LOG_NORMAL);
-	statusbar_puts_all(buf);
+	if (log_fp) {
+		fputs(buf, log_fp);
+		fflush(log_fp);
+	}
+	if (log_verbosity_count)
+		statusbar_puts_all(buf);
 }
 
 void log_message(const gchar *format, ...)
@@ -1853,6 +1977,12 @@ void log_message(const gchar *format, ...)
 
 	if (debug_mode) g_message("%s", buf);
 	log_window_append(buf, LOG_MSG);
+	if (log_fp) {
+		fputs("message: ", log_fp);
+		fputs(buf, log_fp);
+		fflush(log_fp);
+	}
+	statusbar_puts_all(buf);
 }
 
 void log_warning(const gchar *format, ...)
@@ -1866,6 +1996,11 @@ void log_warning(const gchar *format, ...)
 
 	g_warning("%s", buf);
 	log_window_append(buf, LOG_WARN);
+	if (log_fp) {
+		fputs("*** warning: ", log_fp);
+		fputs(buf, log_fp);
+		fflush(log_fp);
+	}
 }
 
 void log_error(const gchar *format, ...)
@@ -1879,4 +2014,9 @@ void log_error(const gchar *format, ...)
 
 	g_warning("%s", buf);
 	log_window_append(buf, LOG_ERROR);
+	if (log_fp) {
+		fputs("*** error: ", log_fp);
+		fputs(buf, log_fp);
+		fflush(log_fp);
+	}
 }
