@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999,2000 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2001 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -125,11 +125,14 @@ void list_free_strings		(GList		*list);
 void slist_free_strings		(GSList		*list);
 
 void hash_free_strings		(GHashTable	*table);
+void hash_free_value_mem	(GHashTable	*table);
 
 void ptr_array_free_strings	(GPtrArray	*array);
 
 /* number-string conversion */
 gint to_number			(const gchar *nstr);
+gchar *itos_buf			(gchar	     *nstr,
+				 gint	      n);
 gchar *itos			(gint	      n);
 gchar *to_human_readable	(off_t	      size);
 
@@ -139,6 +142,8 @@ gint strcmp2		(const gchar	*s1,
 gint path_cmp		(const gchar	*s1,
 			 const gchar	*s2);
 gchar *strretchomp	(gchar		*str);
+gchar *strtailchomp	(gchar		*str,
+			 gchar		 tail_char);
 gchar *strcasestr	(const gchar	*haystack,
 			 const gchar	*needle);
 gchar *strncpy2		(gchar		*dest,
@@ -194,10 +199,16 @@ void eliminate_parenthesis		(gchar		*str,
 void extract_parenthesis		(gchar		*str,
 					 gchar		 op,
 					 gchar		 cl);
-void extract_parenthesis_with_skip_quote(gchar		*str,
-					 gchar		 quote_chr,
-					 gchar		 op,
-					 gchar		 cl);
+
+void extract_one_parenthesis_with_skip_quote	(gchar		*str,
+						 gchar		 quote_chr,
+						 gchar		 op,
+						 gchar		 cl);
+void extract_parenthesis_with_skip_quote	(gchar		*str,
+						 gchar		 quote_chr,
+						 gchar		 op,
+						 gchar		 cl);
+
 void eliminate_quote			(gchar		*str,
 					 gchar		 quote_chr);
 void extract_quote			(gchar		*str,
@@ -243,15 +254,22 @@ gchar *get_domain_name		(void);
 
 /* file / directory handling */
 off_t get_file_size		(const gchar	*file);
+off_t get_left_file_size	(FILE		*fp);
 gboolean file_exist		(const gchar	*file,
 				 gboolean	 allow_fifo);
 gboolean is_dir_exist		(const gchar	*dir);
 gint change_dir			(const gchar	*dir);
 gint make_dir_hier		(const gchar	*dir);
 gint remove_all_files		(const gchar	*dir);
+gint remove_numbered_files	(const gchar	*dir,
+				 guint		 first,
+				 guint		 last);
+gint remove_all_numbered_files	(const gchar	*dir);
 gint remove_dir_recursive	(const gchar	*dir);
 gint copy_file			(const gchar	*src,
 				 const gchar	*dest);
+gint move_file			(const gchar	*src,
+				 const gchar	*dest);
 gint change_file_mode_rw	(FILE		*fp,
 				 const gchar	*file);
 FILE *my_tmpfile		(void);
@@ -261,7 +279,9 @@ FILE *my_tmpfile		(void);
 
 /* process execution */
 gint execute_async		(gchar *const	 argv[]);
-gint execute_command_line	(const gchar	*cmdline);
+gint execute_sync		(gchar *const	 argv[]);
+gint execute_command_line	(const gchar	*cmdline,
+				 gboolean	 async);
 
 /* open URI with external browser */
 gint open_uri(const gchar *uri, const gchar *cmdline);
@@ -274,10 +294,13 @@ void get_rfc822_date		(gchar		*buf,
 				 gint		 len);
 
 /* logging */
-void debug_print(const gchar *format, ...) G_GNUC_PRINTF(1, 2);
-void log_print	(const gchar *format, ...) G_GNUC_PRINTF(1, 2);
-void log_message(const gchar *format, ...) G_GNUC_PRINTF(1, 2);
-void log_warning(const gchar *format, ...) G_GNUC_PRINTF(1, 2);
-void log_error	(const gchar *format, ...) G_GNUC_PRINTF(1, 2);
+void set_log_file	(const gchar *filename);
+void close_log_file	(void);
+void log_verbosity_set	(gboolean verbose);
+void debug_print	(const gchar *format, ...) G_GNUC_PRINTF(1, 2);
+void log_print		(const gchar *format, ...) G_GNUC_PRINTF(1, 2);
+void log_message	(const gchar *format, ...) G_GNUC_PRINTF(1, 2);
+void log_warning	(const gchar *format, ...) G_GNUC_PRINTF(1, 2);
+void log_error		(const gchar *format, ...) G_GNUC_PRINTF(1, 2);
 
 #endif /* __UTILS_H__ */
